/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __BEHAVIOUR_CPP
#define __BEHAVIOUR_CPP

#include "behaviour.h"
#include "logging.h"
#include "ibusutil.h"
#include "statisti.h"
#include <stdlib.h>

Behaviour::Behaviour()
{
	cover_id = NULL_TYPE;
	age = cover_id = NULL_TYPE;
	gender = FEMALE;
	step_angle = NULL;
	step_length = NULL;
	offspring = NULL;
	survival = NULL_FLOAT;
	resource_factor = 1;
}

Behaviour::Behaviour(cover_type _cover_id, age_type _age, gender_types _gender)
{
	cover_id = _cover_id;
	age = _age;
	gender = _gender;
	step_angle = NULL;
	step_length = NULL;
	offspring = NULL;
	survival = NULL_FLOAT;
	resource_factor = 1;
}

Behaviour::~Behaviour()
{
}

void Behaviour::reset()
{
	if(step_angle)
		delete step_angle;
	if(step_length)
		delete step_length;
	if(offspring)
		delete offspring;
}

bool Behaviour::operator==(const Behaviour& _behaviour) const
{
	return	(short)cover_id == (short)_behaviour.cover_id &&
			(short)age == (short)_behaviour.age &&
			(short)gender == (short)_behaviour.gender;
}

bool Behaviour::operator!=(const Behaviour& _behaviour) const
{
	return !((*this)==_behaviour);
}

void Behaviour::setCrossingProbability(cover_type _cover_id, float _probability)
{
	crossing_probabilities[_cover_id] = _probability;
}

float Behaviour::getCrossingProbability(cover_type _cover_id)
{
	if(crossing_probabilities.empty())
		throw IbusepmException("Behaviour::getCrossingProbability(",
		Utility::int_to_char(_cover_id),") crossing_probabilities are empty.\n"); 
	 return crossing_probabilities[_cover_id];
}

void Behaviour::setStep_angle(rdist* _distr, unsigned int seed) throw (IbusepmException)
{	
	if(!_distr)
		throw IbusepmException("Behaviour::setStep_angle() rdist is NULL!");
	if(step_angle)
		delete step_angle;
	step_angle = _distr;		
	step_angle->set_seed(seed);
}

void Behaviour::setStep_length(rdist* _distr, unsigned int seed) throw (IbusepmException)
{
	if(!_distr)
		throw IbusepmException("Behaviour::setStep_length() rdist is NULL!");
	if(step_length)
		delete step_length;
	step_length = _distr;
	step_length->set_seed(seed);
}

void Behaviour::setOffspring(poisson* _distr, unsigned int seed) throw (IbusepmException)
{
	if(!_distr)
		throw IbusepmException("Behaviour::setOffspring() rdist is NULL!");
	if(offspring)
		delete offspring;
	offspring = _distr;
	offspring->set_seed(seed);
}

float Behaviour::getStep_length() const throw (IbusepmException)
{
	if(!step_length)
		throw IbusepmException("Behaviour::getStep_length() rdist is NULL!");
	return step_length->sample();
}

float Behaviour::getStep_angle() const throw (IbusepmException)
{
	if(!step_angle)
		throw IbusepmException("Behaviour::getStep_angle() rdist is NULL!");
	return step_angle->sample();
}

float Behaviour::getOffspring() const throw (IbusepmException)
{
	if(!offspring)
		throw IbusepmException("Behaviour::getOffspring() poisson is NULL!");
	return offspring->sample();
}

float Behaviour::getOffspring_mean() const throw(IbusepmException)
{
	if(!offspring)
		throw IbusepmException("Behaviour::getOffspring_mean() poisson is NULL!");
	return offspring->get_a() == 0.001 ? 0 : offspring->get_a();
}

void Behaviour::report(ostream& strm) const
{
	strm<<"Behaviour: "<<this<<" ";
	strm<<"cover: "<<(int)cover_id<<" age: "<<(int)age;
	strm<<" gender="<<((gender == MALE) ? "male" : "female");
	strm<<" surv: "<<survival<<" res_fac: "<<resource_factor<<"\n";
	strm<<"step_angle: "<<step_angle<<" step_length: "<<step_length<<" offspring: "<<offspring<<"\n";
	boundary_crossing_probabilities::const_iterator cross_prob_iterator = crossing_probabilities.begin();
	while(cross_prob_iterator != crossing_probabilities.end())
	{
		strm<<"cross_prob: ";
		strm<<"to: "<<(int)(*cross_prob_iterator).first;
		strm<<" = "<<(*cross_prob_iterator++).second;
		strm<<"\n";
	}
}



#endif