/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __BEHAVIOUR_H
#define __BEHAVIOUR_H

#include "defs.h"
#include "ibusepmexception.h"
#include <map>
#include <iostream>

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class rdist;
class poisson;

typedef map<cover_type, float, less<cover_type> > boundary_crossing_probabilities;

/**
	A Behaviour composes all information regarding an individual's behaviour, 
	i.e. fecundity, mortality, movement, boundary crossing probabilities, ressource
	use. A Behaviour object is uniquely defined by three keys: landcover, gender and
	age, reflecting the dependencies of behaviour on an individual's state and
	environment. Behaviour objects are stored in a list and managed by a Population
	object.
*/

class Behaviour
{
private:
	cover_type		cover_id;			//id of the landcover type (0..255)	
	age_type		age;				//age of the individual
	gender_types	gender;				//gender of the individual

	rdist*			step_angle;			//random generator for step angles
	rdist*			step_length;		//random generator for step lengths
	poisson*		offspring;			//random generator for offspring 
	float			survival;			//survival prob. per time step (0..1)
	float			resource_factor;	//ressource use for the individual
	boundary_crossing_probabilities	crossing_probabilities;
										//crossing probabilities to other landcover types

public:
	Behaviour();
	Behaviour(cover_type _cover_id, age_type _age, gender_types _gender);
	~Behaviour();
	void reset();

	bool operator==(const Behaviour& _behaviour) const;
	bool operator!=(const Behaviour& _behaviour) const;
	
	cover_type getCover_id() const				{ return cover_id; };
	void setCover_id(cover_type _cover_id)		{ cover_id = _cover_id; };

	age_type getAge() const						{ return age; };
	void setAge(age_type _age)					{ age = _age; };

	gender_types getGender() const				{ return gender; };
	void setGender(gender_types _gender)		{ gender = _gender; };

	float getSurvival() const					{ return survival; };
	void setSurvival(float _survival)			{ survival = _survival; };

	void setResource_factor(float _resource_factor)		{ resource_factor = _resource_factor; };
	float getResource_factor() const					{ return resource_factor; };

	void setCrossingProbability(cover_type _cover_id, float _probability);
	float getCrossingProbability(cover_type _cover_id);

	void setStep_angle(rdist* _distr, unsigned int seed) throw (IbusepmException);
	void setStep_length(rdist* _distr, unsigned int seed) throw (IbusepmException);
	void setOffspring(poisson* _distr, unsigned int seed) throw (IbusepmException);
	
	float getStep_length() const throw (IbusepmException);
	float getStep_angle() const throw (IbusepmException);
	float getOffspring() const throw(IbusepmException);
	float getOffspring_mean() const throw(IbusepmException);

	void report(ostream& strm=cout) const;
};
 
#endif