/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __DYNINTPAIR_CPP
#define __DYNINTPAIR_CPP

#include "dipair.h"
#include "logging.h"
#include "ibusutil.h"
#include <stdlib.h>


dipairlist::dipairlist()
{
        leftpair=NULL;
        rightpair=NULL;
}

dipairlist::dipairlist(int x)
{
        leftpair=NULL;
        rightpair=NULL;
        for(int i=0; i<x; i++)
                add(0,0);
}

dipairlist::dipairlist(dipairlist& ilist)
{
        leftpair=NULL;
        rightpair=NULL;
        int listsize=ilist.entries();
        for(int i=0; i<listsize; i++)
                add(ilist.get_xval_at(i), ilist.get_yval_at(i));
}

dipairlist::~dipairlist()
{
        reset();
}

void dipairlist::reset()
{
        intpair_el* current=leftpair;
        intpair_el* next;
        while(current)
        {
                next=current->right;
                delete current;
                current=next;
        }
        leftpair=NULL;
        rightpair=NULL;
}

void dipairlist::add(int x, int y)
{
        intpair_el* new_el=new intpair_el;
        if(!new_el)
        {
                error(1);
                exit(1);
        }
        if(!leftpair)
                leftpair=new_el;
        new_el->xval=x;
	new_el->yval=y;
        if(rightpair)
                rightpair->right=new_el;
        new_el->right=NULL;
        rightpair=new_el;
}

int dipairlist::entries() const
{
	intpair_el* current=leftpair;
        int count=0;
        while(current)
        {
                count++;
                current=current->right;
        }
        return count;
}

int dipairlist::get_xval_at(int i) const
{
	intpair_el* current=leftpair;
	int count=0;
	while(current)
        {
                if(count==i)
                        return current->xval;
                current=current->right;
                count++;
        }
        error(2,i);
        return -1;
}

int dipairlist::get_yval_at(int i) const 
{ 
        intpair_el* current=leftpair; 
        int count=0; 
        while(current)
        {
                if(count==i) 
                        return current->yval; 
                current=current->right; 
                count++; 
        } 
        error(3,i); 
        return -1;
} 

int dipairlist::index_of(int x, int y) const
{
        intpair_el* current=leftpair;
        int count=0;
        while(current)
        {
                if(current->xval==x && current->yval==y)
                        return count;
                count++;
                current=current->right;
        }
        return -1;
}

int dipairlist::operator=(dipairlist& ilist)
{
        int listsize=ilist.entries();
        reset();
        for(int i=0; i<listsize; i++)
                add(ilist.get_xval_at(i), ilist.get_yval_at(i));
        return 1;
}

int dipairlist::operator==(dipairlist& ilist)
{
        int listsize=ilist.entries();
        if(listsize!=entries())
                return 0;
        for(int i=0; i<listsize; i++)
        {
                if(this->get_xval_at(i)!=ilist.get_xval_at(i) ||
		   this->get_yval_at(i)!=ilist.get_yval_at(i))
                        return 0;
        }
        return 1;
}

int dipairlist::operator!=(dipairlist& ilist)
{
        if(*this==ilist)
                return 0;
        return 1;
}

void dipairlist::report(ostream& strm) const
{
        strm<<"[ ";
        intpair_el* current=leftpair;
        while(current)
        {
                strm.width(3);
                strm<<current->xval<<" ";
                current=current->right;
        }
        strm<<"]"<<endl;
	strm<<"[ ";
        current=leftpair;
        while(current)
        {
                strm.width(3); 
                strm<<current->yval<<" "; 
                current=current->right; 
        } 
        strm<<"]"<<endl; 
}
 
void dipairlist::error(int n, float val) const
{
	switch(n)
	{
		case 1:		Logging::log_error("dipairlist::add(): lack of memory!",NULL); 
					break;
		case 2:		Logging::log_error("dipairlist::get_xval_at(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL); 
					break;
		case 3:		Logging::log_error("dipairlist::get_yval_at(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL); 
					break;
		default:	Logging::log_error("dipairlist::error(): no error et all?",NULL);
	}
};

#endif
