/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __DYNVOIDARRAY_CPP
#define __DYNVOIDARRAY_CPP

#include "dvoidarr.h"
#include "logging.h"
#include "ibusutil.h"
#include <stdlib.h>

dvoidlist::dvoidlist()
{
	leftvoid=NULL;
	rightvoid=NULL;
}

dvoidlist::dvoidlist(int x, int* dummy)
{
	leftvoid=NULL;
	rightvoid=NULL;
	for(int i=0; i<x; i++)
		add(dummy);
}

dvoidlist::~dvoidlist()
{
	reset();
}

void dvoidlist::reset()
{
        void_el* current=leftvoid;
    	void_el* next;
	while(current)
	{
        	next=current->right;
        	delete current;
        	current=next;
    	}
    	leftvoid=NULL;
	rightvoid=NULL;
}

void dvoidlist::add(void* v)
{
	void_el* new_el=new void_el;
	if(!new_el)
	{
		error(1);
		exit(1);
	}
	if(!leftvoid)
                leftvoid=new_el;
	new_el->val=v;
	if(rightvoid)
		rightvoid->right=new_el;
	new_el->right=NULL;
	rightvoid=new_el;
}

void* dvoidlist::operator[](int x)
{
	void_el* current=leftvoid;
        int count=0;
        while(current)
        {
                if(count==x)
                        return current->val;
                current=current->right;
                count++;
        }
        error(5,x);
        return 0;
}

int dvoidlist::elem_index(void* v) const
{
	void_el* current=leftvoid;
        int index=0;
        while(current)
        {
                if(v==current->val)
                        return index;
                current=current->right;
                index++;
        }
        return -1;
}

int dvoidlist::set(int x, void* p)
{
	void_el* current=leftvoid;
        int count=0;
        while(current)
        {
                if(count==x)
		{
                        current->val=p;
			return 1;
		}
                current=current->right;
                count++;
        }
        error(4,x);
        return 0;
}

void* dvoidlist::get(int x) const
{
	void_el* current=leftvoid;
        int count=0;
        while(current)
        {
                if(count==x)
                        return current->val;
                current=current->right;
                count++;
        }
        error(5,x);
        return 0;
}

int dvoidlist::entries() const
{
    	void_el* current=leftvoid;
	int count=0;
    	while(current)
        {
                count++;
                current=current->right;
    	}
    	return count;
}

void dvoidlist::report(ostream& strm) const
{
	strm<<"[ ";
	void_el* current=leftvoid;
	while(current)
	{
		strm.width(3);
		strm<<*(int*)current->val<<" ";
		current=current->right;
	}
	strm<<"]"<<endl;
}

void dvoidlist::error(int n, float val) const
{
	switch(n)
	{
		case 1:		Logging::log_error("dvoidlist::add(): lack of memory!",NULL);
					break;
		case 2:		Logging::log_error("dvoidlist::set(): position: ",
										Utility::int_to_char((int)val),
										"out of permitted range: ",
										Utility::int_to_char((int)entries()),NULL);
					break;
		case 3:		Logging::log_error("dvoidlist::[]: index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		case 4:		Logging::log_error("dvoidlist::set(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		case 5:		Logging::log_error("dvoidlist::get(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		default:	Logging::log_error("dvoidlist::error(): no error et all?",NULL);
	}
}

//********************************************************************************

dvoidarray::dvoidarray()
{
	toplist=NULL;
	bottomlist=NULL;
}

dvoidarray::dvoidarray(int x, int y)
{
	int* dummy=new int(1);
	toplist=NULL;
        bottomlist=NULL;
	for(int i=0; i<y; i++)
		add(new dvoidlist(x,dummy));
}

dvoidarray::~dvoidarray()
{
	reset();
}

void dvoidarray::reset()
{
	voidlist_el* current=toplist;
	voidlist_el* next;
	while(current)
	{
		next=current->right;
		current->lval->reset();
		delete current;
		current=next;
	}
	toplist=NULL;
	bottomlist=NULL;
}

int dvoidarray::add(dvoidlist* l)
{
	voidlist_el* new_el=new voidlist_el;
	if(!new_el)
	{
		error(1);
		exit(1);
	}
	if(!l)
	{
		error(2);
		return 0;
	}
	if(!toplist)
                toplist=new_el;
        new_el->lval=l;
        if(bottomlist)
                bottomlist->right=new_el;
        new_el->right=NULL;
        bottomlist=new_el;
	return 1;
}

dvoidlist& dvoidarray::operator[](int y)
{
	voidlist_el* current=toplist;
        int count=0;
        while(current)
        {
                if(count==y)
                        return *current->lval;
                current=current->right;
                count++;
        }
        error(5,y);
	dvoidlist* dummy=new dvoidlist;
        return *dummy;
}

int dvoidarray::set(int x, int y, void* p)
{
	voidlist_el* current=toplist;
        int count=0;
        while(current)
        {
                if(count==y)
		{
                        current->lval->set(x,p);
			return 1;
		}
                current=current->right;
                count++;
        }
        error(4,y);
        return 0;
}

void* dvoidarray::get(int x, int y) const
{ 
        voidlist_el* current=toplist; 
        int count=0; 
        while(current) 
        { 
                if(count==y) 
                        return current->lval->get(x); 
                current=current->right; 
                count++;
        } 
        error(5,y); 
        return 0;
}

int dvoidarray::entries() const
{
	voidlist_el* current=toplist;
        int count=0;
        while(current)
        {
                count++;
                current=current->right;
        }
        return count;
}

void dvoidarray::report(ostream& strm) const
{
	voidlist_el* current=toplist; 
        while(current) 
        { 
		current->lval->report(strm);
                current=current->right; 
        } 
}

void dvoidarray::error(int n, float val) const
{
	switch(n)
	{
		case 1:		Logging::log_error("dvoidarray::add(): lack of memory!",NULL);
					break;
		case 2:		Logging::log_error("add(): list to be added was NULL!",NULL);
					break;
		case 3:		Logging::log_error("dvoidarray::[]: index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		case 4:		Logging::log_error("dvoidarray::set(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		case 5:		Logging::log_error("dvoidarray::get(): index: ",
										Utility::int_to_char((int)val),
										"out of range: [0..",
										Utility::int_to_char((int)entries()),"]",NULL);
					break;	
		default:	Logging::log_error("dvoidarray::error(): no error et all?",NULL);
	}
};

#endif
		
