/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __INDIVIDUAL_CPP
#define __INDIVIDUAL_CPP

#include "individual.h"
#include "population.h"
#include "landscape.h"
#include "landcovertype.h"
#include "statisti.h"
#include "cell.h"
#include "patch.h"
#include "logging.h"
#include "ibusutil.h"
#include <math.h>
#include <algorithm>
#include <string>

Individual::Individual()
{
	id = NULL_INT;
	patch_id = NULL_INT;
	population = NULL;
	behaviour = NULL;
	cell = NULL;
	gender = FEMALE;
	age = NULL_TYPE;
	xpos = NULL_FLOAT;
	ypos = NULL_FLOAT;
	vxpos = NULL_FLOAT;
	vypos = NULL_FLOAT;
	ref_angle = NULL_FLOAT;
	marked = true;		
	dead = false;
	search_steps = 0;
}

Individual::Individual(int _id, Population* _population, gender_types _gender, 
					   age_type _age, float _xpos, float _ypos)
{
	id = _id; 
	patch_id = NULL_INT;
	population = _population;
	cell = population->registry->landscape->getCell(_xpos,_ypos);
	gender = _gender;
	age = _age;
	xpos = _xpos;
	ypos = _ypos;
	vxpos = _xpos;
	vypos = _ypos;
	ref_angle = NULL_FLOAT;
	marked = true;	
	dead = false;
	behaviour = NULL;
	search_steps = 0;
}	

bool Individual::operator==(const Individual& _individual) const
{
	return id == _individual.id;
}

bool Individual::operator!=(const Individual& _individual) const
{
	return !((*this) == _individual);
}

cover_type Individual::getCurrentCoverType() const
{
	if(cell)
		return cell->getCover_id();
	return NULL;
}

void Individual::move() throw (IbusepmException)
{	
	if(!dead)
	{
		float new_xpos;
		float new_ypos;
		search_steps++;

		calc_new_position(new_xpos,new_ypos);

		Patch* current_patch = cell->getPatch();
		Patch* new_patch = population->registry->landscape->getPatch(new_xpos,new_ypos);

		if(new_xpos>0 && new_ypos>0 && (int)new_xpos==(int)xpos && (int)new_ypos==(int)ypos) //same cell
		{
			xpos=new_xpos;
			ypos=new_ypos;
		}
		else
		{
			if(current_patch && new_patch)				//within landscape extent
			{
				if(*new_patch == *current_patch)		//same patch
				{
					cell->release_Resources(this);
					xpos=new_xpos;
					ypos=new_ypos; 		
					population->registry->landscape->getCell(xpos,ypos)->use_Resources(this);
				}
				else									//boundary 
				{	
					cover_type current_cover_id = current_patch->getCover_id();
					cover_type new_cover_id = new_patch->getCover_id();	
					unsigned int new_patch_id = new_patch->getPatch_id();
					float cross_prob = behaviour->getCrossingProbability(new_cover_id);

					if(Utility::randfloat() < cross_prob) //cross boundary
					{
						if(current_cover_id == LandCoverType::habitat && new_cover_id == LandCoverType::matrix)	//emigration event
						{
							patch_id = current_patch->getPatch_id();
							search_steps = 0;
						}

						if(current_cover_id == LandCoverType::matrix && new_cover_id == LandCoverType::habitat)	//immigration event
						{
							if(patch_id != NULL_INT && patch_id != new_patch_id)
							{
								population->registry->updatePatchImmigrationObserver(population->getId(), patch_id, new_patch->getPatch_id());
								population->registry->updateSearchTimeObserver(population->getId(), search_steps);
							}
						}
						checkout_from_old_patch();
						xpos=new_xpos;
						ypos=new_ypos;
						register_with_new_patch(xpos, ypos);
						population->registry->updateCrossingObserver(population->getId(), current_cover_id, new_cover_id);
					} 	
					else								//reflection at boundaries
					{
						cell->release_Resources(this);
						reflection();
						population->registry->landscape->getCell(xpos,ypos)->use_Resources(this);
					}
				}
			}
			else if(current_patch)
			{
				if(population->registry->landscape->getBorder_type() == REFLECT)
				{
					cell->release_Resources(this);
					reflection();
					population->registry->landscape->getCell(xpos,ypos)->use_Resources(this);
				}
				else if(population->registry->landscape->getBorder_type() == ABSORB)
				{
					checkout_from_old_patch();
					dead = true;
				}
			}
			if(!dead && current_patch->getCover_id() == 1)
				population->registry->updateCellImmigrationObserver(population->getId(), id, current_patch->getCover_id(), xpos, ypos);
		} 
		population->addStep();
	}
}

void Individual::reproduce() throw (IbusepmException)
{
	if(!dead)
	{
		int babies = behaviour->getOffspring_mean() == 0 ? 0 : behaviour->getOffspring();
		Individual* baby;

		for(int i=0; i<babies; i++)
		{
			baby = new Individual(population->individual_id++, population, population->determine_Gender(),1,xpos,ypos);
			baby->setRef_angle(population->start_angle->sample());
			baby->setMark(population->determine_Sample());
			baby->register_with_new_patch(xpos,ypos);
			population->individuals.push_back(baby);
			population->registry->updateDistanceObserver(population->getId(), baby->getId(), true, xpos, ypos);
		}
	}
}

void Individual::mature()
{
	//if(!dead)
	//	age++; comes later
}

void Individual::die()
{
	if(!dead)
	{
		float survival = behaviour->getSurvival();
		if(Utility::randfloat() >= survival)
		{
			dead = true;
			checkout_from_old_patch();
		}
	}
}

void Individual::report(ostream& strm) const
{
	strm<<"Individual: "<<id<<"\n";
}

void Individual::register_with_new_patch(float& _xpos, float& _ypos)
{	
	setCell(population->registry->landscape->getCell(_xpos,_ypos));
	findBehaviour();
	cell->use_Resources(this);
	cell->getPatch()->addIndividual(population->getId());
}

void Individual::checkout_from_old_patch()
{
	cell->getPatch()->removeIndividual(population->getId());
	cell->release_Resources(this);
}

void Individual::findBehaviour() throw (IbusepmException)
{
	Behaviour myBehaviour = Behaviour(cell->getCover_id(), age, gender);	
	behaviour_list::const_iterator behaviour_iterator = find(population->behaviours.begin(), 
															 population->behaviours.end(), 
															 myBehaviour);
	if(behaviour_iterator == population->behaviours.end())
	{
		string message("Individual::...() no Behaviour for ");
		message+="cover_id ";
		message+=Utility::int_to_char((int)(population->registry->landscape->getCell(xpos,ypos)->getCover_id()));
		message+="age ";
		message+=Utility::int_to_char((int)age);
		message+="gender ";
		message+=(gender == MALE) ? "MALE" : "FEMALE";
		message+=" !";
		throw IbusepmException(message.c_str());
	}
	behaviour = (Behaviour*)behaviour_iterator;
}

void Individual::calc_new_position(float& xnew, float& ynew)
{
    float turn = behaviour->getStep_angle();
    float step = behaviour->getStep_length();
    ref_angle+=turn;

	float trace_dist = TRACE_RESOLUTION;
	float trace_step = TRACE_RESOLUTION;

	while(trace_step < (int)step)	//tracing of cells between source and destination
	{
		xnew=xpos+(cos(ref_angle)*trace_step);
		ynew=ypos+(sin(ref_angle)*trace_step);
		Cell* current_cell = population->registry->landscape->getCell(xnew,ynew);

		if(!current_cell || current_cell->getCover_id() != cell->getCover_id())
		{
			//virtual coordinates to deal with absolute distances in torus situations
			vxpos+=(cos(ref_angle)*trace_step);
			vypos+=(sin(ref_angle)*trace_step);
			return;
		}
		else
			trace_step+=trace_dist;
	}
	
	xnew=xpos+(cos(ref_angle)*step);
    ynew=ypos+(sin(ref_angle)*step);
    
	//virtual coordinates to deal with absolute distances in torus situations
    vxpos+=(cos(ref_angle)*step);
    vypos+=(sin(ref_angle)*step);
}

//this function gets a new position within the patch of origin after
//boundary crossing has been denied in move, recursion as long as successful
void Individual::reflection()
{
    float new_xpos;
    float new_ypos;

	ref_angle-=population->getRefl_angle();
	calc_new_position(new_xpos,new_ypos);

	Patch* current_patch = cell->getPatch();
    Patch* new_patch = population->registry->landscape->getPatch(new_xpos,new_ypos);

    if(current_patch && new_patch && (*current_patch == *new_patch)) //same patch, go ahead
    {
        xpos=new_xpos;
        ypos=new_ypos; 
    }
    else	//try again
        reflection();
}   

#endif
