/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __INDIVIDUAL_H
#define __INDIVIDUAL_H

#include "defs.h"
#include "ibusepmexception.h"
#include "diarr.h"
#include <iostream>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

class Population;
class Behaviour;
class Cell;
class Patch;

/**
	An Individual object represents one individual of a population. The principal 
	state of an Individual object is defined by its gender, age and location. Each
	Individual object is furthermore uniquely identified by an id. An Individual
	'knows' the Population it belongs to. It is also connected to the Cell object
	of its current location. The behaviour of an Individual is governed by the 
	Behaviour object, which is identified by the Individuals age, gender and position.
	Pointers to Behaviour and Cell objects are dynamic and may be re-assigned
	during simulation. The pointer to the Population is static, because an individual
	does not change its population over time.
*/

class Individual
{
private:
	unsigned int	id;				//unique identifier for each individual
	Cell*			cell;			//pointer to cell of current location
	Population*		population;		//pointer to the Population to which the Individual belongs to
	Behaviour*		behaviour;		//pointer to Behaviour 
	gender_types	gender;			//gender of this Individual
	age_type		age;			//age of this Individual
	float			xpos;			//current x coordinate
	float			ypos;			//current y coordinate
	float			vxpos;			//current virtual x coordinate for torus situation
	float			vypos;			//current virtual x coordinate for torus situation
	float			ref_angle;		//reference angle for correlated movement
	bool			marked;			//marking tag for individual - to sample Individuals
	bool			dead;			//marker for dead individual
	unsigned int	patch_id;		//id of last visited habitat patch - used for tracking dispersal between patches
	unsigned int	search_steps;	//counter for steps between habitat patches - used for search time

public:
	Individual(); 
	Individual(int _id, Population* _population, gender_types _gender, age_type _age, float _xpos, float _ypos);
	~Individual() {};

	bool operator==(const Individual& _individual) const;
	bool operator!=(const Individual& _individual) const;

	unsigned int getId() const					{ return id; };  

	age_type getAge() const						{ return age; };
	void setAge(age_type _age)					{ age = _age; };

	gender_types getGender() const				{ return gender; };
	void setGender(gender_types _gender)		{ gender = _gender; };

	float getXpos() const						{ return xpos; };
	void setXpos(float _xpos)					{ xpos = _xpos; };

	float getYpos() const						{ return ypos; };
	void setYpos(float _ypos)					{ ypos = _ypos; };

	float getVxpos() const						{ return vxpos; };
	void setVxpos(float _vxpos)					{ vxpos = _vxpos; };

	float getVypos() const						{ return vypos; };
	void setVypos(float _vypos)					{ vypos = _vypos; };

	float getRef_angle() const					{ return ref_angle; };
	void setRef_angle(float _ref_angle)			{ ref_angle = _ref_angle; };

	void setMark(bool _marked)					{ marked = _marked; };
	bool isMarked() const  						{ return marked; };	

	void setCell(Cell* _cell)					{ cell = _cell; };
	Behaviour* getBehaviour() const				{ return behaviour; };

	bool isDead()								{ return dead; };
	void setDead(bool _dead)					{ dead = _dead; };

	cover_type getCurrentCoverType() const;

	void move() throw (IbusepmException);
	void reproduce() throw (IbusepmException);
	void mature();
	void die();
	
	void register_with_new_patch(float& _xpos, float& _ypos);
	void checkout_from_old_patch();  	
	void report(ostream& strm=cout) const;

private:	
	void calc_new_position(float& xnew, float& ynew);
	void reflection();
	void findBehaviour() throw (IbusepmException);
};

#endif
