/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __LANDSCAPE_CPP
#define __LANDSCAPE_CPP

#include <iostream>
#include <math.h>
#include "cell.h"
#include "ibusutil.h"
#include "logging.h"
#include "landscape.h"
#include "landcovertype.h"
#include "ibusepmexception.h"
#include "patch.h"

Cell grid[XMAX][YMAX];
cover_type_map	cover_types;

Landscape::Landscape(ModelRegistry*  _registry)
{	
	xdim = NULL_INT;
	ydim = NULL_INT;
	border_type = TORUS;
	source = NOT_DEFINED;
	status = NOT_INITIALIZED;
	patch_id = 0;
	xcurr = 0;
	ycurr = 0;
	diagonal = false;
	buffer1 = 0;
	buffer2 = 0;
	buffer3 = 0;
	neighb_patches = NULL_INT;
	min_patch_size = NULL_INT;
	max_patch_size = NULL_INT;
	filename = NULL;
	registry = _registry;
	registry->register_landscape(this);
}

Landscape::Landscape(ModelRegistry*  _registry, int _xdim, int _ydim, border_types _border_type)
{
	setXdim(_xdim);
	setYdim(_ydim);
	border_type = _border_type;
	source = NOT_DEFINED;
	status = NOT_INITIALIZED;
	patch_id = 0;
	xcurr = 0;
	ycurr = 0;
	diagonal = false;
	buffer1 = 0;
	buffer2 = 0;
	buffer3 = 0;
	neighb_patches = NULL_INT;
	min_patch_size = NULL_INT;
	max_patch_size = NULL_INT;
	filename = NULL;
	registry = _registry;
	registry->register_landscape(this);
}

void Landscape::setXdim(int _xdim)					
{ 
	xdim = _xdim > XMAX ? XMAX : _xdim;
	if(_xdim > XMAX)
	{
		char* message = new char[100];
		sprintf(message,"Landscape::setXdim(%d) GRID_WIDTH > %d!, Will continue with %d!\n",_xdim,XMAX,XMAX);
		Logging::log_warning(message,NULL);
		delete message;
	}
}

void Landscape::setYdim(int _ydim)					
{ 
	ydim = _ydim > YMAX ? YMAX : _ydim; 
	if(_ydim > YMAX)
	{
		char* message = new char[100];
		sprintf(message,"Landscape::setYdim(%d) GRID_LENGTH > %d!, Will continue with %d!\n",_ydim,YMAX,YMAX);
		Logging::log_warning(message,NULL);
		delete message;
	}
}

void Landscape::reset()
{
	Logging::log_debug("Landscape::reset() ...",NULL);

	for(int y=0; y<YMAX; y++)
		for(int x=0; x<XMAX; x++)
			grid[x][y].reset();
	
	patch_list::iterator patch_iterator = patches.begin();
	while(patch_iterator != patches.end())
	{
		Patch* tmpPatch = *patch_iterator;
		if(tmpPatch)
		{
			tmpPatch->reset();
			delete tmpPatch;
		}
		patch_iterator++;			        
    }
	patches.clear();

	status = NOT_INITIALIZED;
	source = NOT_DEFINED;
	patch_id = 0;
	xcurr = 0;
	ycurr = 0;
	diagonal = false;

	resetLandCoverTypes();
	Logging::log_debug("Landscape::reset() DONE.",NULL);
}

void Landscape::resetLandCoverTypes()
{
	Logging::log_debug("Landscape::resetLandCoverTypes() ...",NULL);

	cover_type_map::iterator lc_iterator = cover_types.begin();
	while(lc_iterator != cover_types.end())
		((*lc_iterator++).second)->reset();

	Logging::log_debug("Landscape::resetLandCoverTypes() done.",NULL);
}

void Landscape::addLandCoverType(LandCoverType* _landcovertype)
{	
	cover_types[_landcovertype->getCover_id()] = _landcovertype;
}

LandCoverType* Landscape::getLandCoverType(cover_type _cover_id) throw (IbusepmException)
{
	if(cover_types.empty())
		throw IbusepmException("Landscape::getLandCoverType(",
			Utility::int_to_char((int&)_cover_id),") cover_types are empty.\n"); 
	
	LandCoverType* lct = cover_types[_cover_id];
	if(lct->getCover_name() == NULL)
		throw IbusepmException("Landscape::getLandCoverType(",
			  Utility::int_to_char((int&)_cover_id),") not in cover_types.\n"); 
	else
		return lct;
}

long Landscape::getLandCoverAmount(cover_type _cover_id) const
{
	long count=0;
	patch_list::const_iterator patch_iterator = patches.begin();
	while(patch_iterator != patches.end())
	{
		if(((Patch*)*patch_iterator)->getCover_id() == _cover_id)
			count+=((Patch*)*patch_iterator)->getSize();
        patch_iterator++;
    }
	return count;
}

void Landscape::create_all_cox_pattern() throw (IbusepmException)
{
	int last_sequence = 0;
	
	cover_type_map::iterator landcover_iterator = cover_types.begin();
	while(landcover_iterator != cover_types.end())
	{
		LandCoverType* tempLandCoverType = (*landcover_iterator).second;
		
		int creation_sequence = tempLandCoverType->getSequence();
		if(creation_sequence > last_sequence)
		{
			last_sequence = creation_sequence;
			create_cox_pattern(*tempLandCoverType);
			landcover_iterator = cover_types.begin();
		}
		else
		{
			landcover_iterator++;
		}
	}
}

void Landscape::create_cox_pattern(LandCoverType& _landcovertype) throw (IbusepmException)
{
	Logging::log_debug("Landscape::create_cox_pattern(",_landcovertype.getCover_name(),")...",NULL);
	
	cover_type cover_id = _landcovertype.getCover_id();
	float cover = _landcovertype.getCoverage();
	float frag  = _landcovertype.getFragmentation();
	bool generated = _landcovertype.isGenerated();
	const char* name = _landcovertype.getCover_name();

	if(status > CELL_TOPOLOGY)
	{
		Logging::log_warning("Landscape::create_cox_pattern(",_landcovertype.getCover_name(),
							") landscape was already initialized. Landscape will be reset!",NULL);
		reset();
	}

	if(generated)
		throw IbusepmException("Landscape::create_cox_pattern(",name,") already generated.\n");
	if(cover <= 0 || cover > 1)
		throw IbusepmException("Landscape::create_cox_pattern(",name,") cover out of range.\n");
	if(frag <= 0 || frag > 1)
		throw IbusepmException("Landscape::create_cox_pattern(",name,") frag out of range.\n");

	int xselect,yselect;
	long covered=0;
	long tocover=(long)(cover * xdim * ydim); 
	frag = pow(frag,2);

  	while(covered < tocover)
	{
		xselect=Utility::randint(xdim-1);
		yselect=Utility::randint(ydim-1);

		if(grid[xselect][yselect].getCover_id() == 0)
		{
			if(Utility::randfloat() <= frag || equal_neighbour_state(xselect,yselect,cover_id)) 
			{
				grid[xselect][yselect].setCover_id(cover_id);
				covered++;
            }
		}
	}
	_landcovertype.setGenerated(true);
	source = GENERATED;
	status = CELL_TOPOLOGY;
	Logging::log_debug("Landscape::create_cox_pattern(",_landcovertype.getCover_name(),") DONE.",NULL);
}


bool Landscape::equal_neighbour_state(int x, int y, cover_type cover_id) const
{
	if((x>0) && (y>0) && (x<xdim-1) && (y<ydim-1))
	{
		if(grid[x-1][y].getCover_id() == cover_id)		//l
			return true;
		if(grid[x-1][y-1].getCover_id() == cover_id)	//ul
			return true;
		if(grid[x-1][y+1].getCover_id() == cover_id)	//bl
			return true;
		if(grid[x][y-1].getCover_id() == cover_id)		//u
			return true;
		if(grid[x][y+1].getCover_id() == cover_id)		//b
			return true;
		if(grid[x+1][y].getCover_id() == cover_id)		//r
			return true;
		if(grid[x+1][y-1].getCover_id() == cover_id)	//ur
			return true;
		if(grid[x+1][y+1].getCover_id() == cover_id)	//br
			return true;
	}
	return false;
}

//this method transforms coordinates outside of the grid into corresponding
//torus coordinates
bool Landscape::torus_xy_transform(int& x, int& y) const
{
	if(y<0 && x>=0 && x<xdim)		//above top edge
	{
		y+=ydim; return true;
	}
	if(x>=xdim && y>=0 && y<ydim)	//right section
	{
		x-=xdim; return true;
	}
	if(y>=ydim && x>=0 && x<xdim)	//below lower edge
	{
		y-=ydim; return true;
	}
	if(x<0 && y>= 0 && y<=ydim)		//left section
	{
		x+=xdim; return true;
	}
	return false;
}

//the parameter c is for distinguishing the use of the following methods
//if c is 0, which is the default value, the torus part is not executed,
//the methods are than used for the flooding  algorithm in landscap.cpp
//if c != 0, the function is used by a scanning algorithm to calculate
//some patch isolation indices, here the torus feature is of importance
//because the neighborhood depends on whether the landscape is wrapped or not!

Cell* Landscape::get_right(int& x, int& y, int c) const
{
	if(x>=0 && x<xdim-1 && y>=0 && y<ydim)
		return &grid[++x][y];

	x++;								//for scan ring in landscape.cpp
	if(border_type == TORUS && c)		//cel values for a torus situation are given back
	{
		int xtor=x; int ytor=y;
		if(torus_xy_transform(xtor,ytor))
			return &grid[xtor][ytor];
	}
	return NULL; 
}
			

Cell* Landscape::get_left(int& x, int& y, int c) const 
{ 
	if(x>0 && x<xdim && y>=0 && y<ydim) 
		return &grid[--x][y];         

     x--;								//for scan ring in landscape.cpp
	if(border_type == TORUS && c)		//cel values for a torus situation are given back
	{
		int xtor=x; int ytor=y;
		if(torus_xy_transform(xtor,ytor))
			return &grid[xtor][ytor];
	}
	return NULL;         
}		

Cell* Landscape::get_below(int& x, int& y, int c) const 
{    
	if(x>=0 && x<xdim && y>=0 && y<ydim-1)   
		return &grid[x][++y];              

	y++;								//for scan ring in landscape.cpp
	if(border_type == TORUS && c)		//cel values for a torus situation are given back
	{
		int xtor=x; int ytor=y;
		if(torus_xy_transform(xtor,ytor))
			return &grid[xtor][ytor];
	}
	return NULL; 
} 

Cell* Landscape::get_above(int& x, int& y, int c) const 
{ 
	if(x>=0 && x<xdim && y>0 && y<ydim)   
		return &grid[x][--y];              

	y--;								//for scan ring in landscape.cpp
	if(border_type == TORUS && c)		//cel values for a torus situation are given back 
	{ 
		int xtor=x; int ytor=y; 
		if(torus_xy_transform(xtor,ytor))
			return &grid[xtor][ytor];
	}
	return NULL;           
	      
}

Cell* Landscape::get_ra(int& x, int& y) const 
{ 
	if(x>=0 && x<xdim-1 && y>0 && y<ydim)
		return &grid[++x][--y];              
	
	x++;y--;	
    return NULL;              
}  

Cell* Landscape::get_rb(int& x, int& y) const 
{ 
	if(x>=0 && x<xdim-1 && y>=0 && y<ydim-1)    
		return &grid[++x][++y];               

	x++;y++;
	return NULL;               
} 

Cell* Landscape::get_la(int& x, int& y) const 
{ 
	if(x>0 && x<xdim && y>0 && y<ydim)    
		return &grid[--x][--y];               

	x--;y--;
	return NULL;               
}   

Cell* Landscape::get_lb(int& x, int& y) const 
{ 
	if(x>0 && x<xdim && y>=0 && y<ydim-1)    
	return &grid[--x][++y];
	
	x--;y++;	
	return NULL;               
}

void Landscape::create_patch_topology() throw (IbusepmException)
{
	Logging::log_debug("Landscape::create_patch_topology()...",NULL);
	if(!patches.empty())
		Logging::log_error("Landscape::create_patch_topology() patch topology already created.",
							"# patches: ", Utility::int_to_char(patches.size()));
	
	int xref=0;
	int yref=0;
	while(setrefcel(xref,yref))
			patches.push_back(allocate_patch(xref,yref));

	check_area_sums();
	calc_all_patch_perimeters();
	status = PATCH_TOPOLOGY;
	
	Logging::log_debug("Landscape::create_patch_topology() DONE",NULL);
}

bool Landscape::setrefcel(int& xref, int& yref) const
{
	for(int y=yref;y<ydim;y++)
		for(int x=0; x<xdim;x++)
		{
			if(patches.empty() || grid[x][y].getPatch() == NULL)
			{
				xref=x;
				yref=y;
				return true;
			}
		}
	return false;	
}

Patch* Landscape::allocate_patch(int x, int y)
{
	cover_type cover_id = grid[x][y].getCover_id();
	Patch* p_new = new Patch((unsigned short)x,(unsigned short)y,++patch_id,cover_id);
 
	assign_cell_to_patch(x,y,p_new);

	allocate_right_cel(x,y,cover_id,p_new);
	allocate_below_cel(x,y,cover_id,p_new);
	allocate_above_cel(x,y,cover_id,p_new);
	allocate_left_cel(x,y,cover_id,p_new);

	if(diagonal)
	{
		allocate_ra_cel(x,y,cover_id,p_new);
		allocate_rb_cel(x,y,cover_id,p_new);
		allocate_la_cel(x,y,cover_id,p_new);
		allocate_lb_cel(x,y,cover_id,p_new);
	}
	return p_new;
}
		
void Landscape::assign_cell_to_patch(int x, int y, Patch* _patch)
{
	xcurr=x;
	ycurr=y;

	grid[x][y].setPatch(_patch);
	_patch->register_cell(x,y);
}

void Landscape::allocate_right_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{
	Cell* right = get_right(x,y);
	if(right && right->getPatch() != _patch && right->getCover_id() == _cover_id)
	{
		assign_cell_to_patch(x,y,_patch);

		allocate_right_cel(x,y,_cover_id,_patch);
		allocate_below_cel(x,y,_cover_id,_patch);
		allocate_above_cel(x,y,_cover_id,_patch);

		if(diagonal)
		{
			allocate_ra_cel(x,y,_cover_id,_patch);
			allocate_rb_cel(x,y,_cover_id,_patch);
			allocate_la_cel(x,y,_cover_id,_patch);
			allocate_lb_cel(x,y,_cover_id,_patch);
		}

	}
}

void Landscape::allocate_left_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{
	Cell* left=get_left(x,y); 

    if(left && left->getPatch() != _patch && left->getCover_id() == _cover_id)
	{ 
		assign_cell_to_patch(x,y,_patch);

		allocate_left_cel(x,y,_cover_id,_patch); 
		allocate_below_cel(x,y,_cover_id,_patch); 
		allocate_above_cel(x,y,_cover_id,_patch);

		if(diagonal) 
		{ 
			allocate_ra_cel(x,y,_cover_id,_patch); 
			allocate_rb_cel(x,y,_cover_id,_patch); 
			allocate_la_cel(x,y,_cover_id,_patch); 
			allocate_lb_cel(x,y,_cover_id,_patch); 
		} 
	}
}

void Landscape::allocate_above_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{ 
	Cell* above=get_above(x,y); 

	if(above && above->getPatch() != _patch && above->getCover_id() == _cover_id)
	{ 
		assign_cell_to_patch(x,y,_patch);

		allocate_above_cel(x,y,_cover_id,_patch); 
		allocate_right_cel(x,y,_cover_id,_patch); 
		allocate_left_cel(x,y,_cover_id,_patch);
		
		if(diagonal) 
		{ 
			allocate_ra_cel(x,y,_cover_id,_patch); 
			allocate_rb_cel(x,y,_cover_id,_patch); 
			allocate_la_cel(x,y,_cover_id,_patch); 
			allocate_lb_cel(x,y,_cover_id,_patch); 
		} 
	}
}

void Landscape::allocate_below_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{
	Cell* below=get_below(x,y);  

	if(below && below->getPatch() != _patch && below->getCover_id() == _cover_id) 
	{  
		assign_cell_to_patch(x,y,_patch); 

		allocate_below_cel(x,y,_cover_id,_patch); 
		allocate_right_cel(x,y,_cover_id,_patch); 
		allocate_left_cel(x,y,_cover_id,_patch);  
		
		if(diagonal) 
		{ 
			allocate_ra_cel(x,y,_cover_id,_patch); 
			allocate_rb_cel(x,y,_cover_id,_patch); 
			allocate_la_cel(x,y,_cover_id,_patch);  
			allocate_lb_cel(x,y,_cover_id,_patch);  
		} 
	}
}

void Landscape::allocate_ra_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{
	Cell* ra=get_ra(x,y);

	if(ra && ra->getPatch() != _patch && ra->getCover_id() == _cover_id)
	{
		assign_cell_to_patch(x,y,_patch); 

		allocate_ra_cel(x,y,_cover_id,_patch); 
		allocate_rb_cel(x,y,_cover_id,_patch); 
		allocate_la_cel(x,y,_cover_id,_patch); 
		allocate_right_cel(x,y,_cover_id,_patch); 
		allocate_left_cel(x,y,_cover_id,_patch); 
		allocate_above_cel(x,y,_cover_id,_patch); 
		allocate_below_cel(x,y,_cover_id,_patch);
	}
}

void Landscape::allocate_rb_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{ 
	Cell* rb=get_rb(x,y);

	if(rb && rb->getPatch() != _patch &&  rb->getCover_id() == _cover_id)
	{ 
		assign_cell_to_patch(x,y,_patch); 

		allocate_rb_cel(x,y,_cover_id,_patch); 
		allocate_ra_cel(x,y,_cover_id,_patch); 
		allocate_lb_cel(x,y,_cover_id,_patch);  
		allocate_right_cel(x,y,_cover_id,_patch); 
		allocate_left_cel(x,y,_cover_id,_patch); 
		allocate_above_cel(x,y,_cover_id,_patch); 
		allocate_below_cel(x,y,_cover_id,_patch); 
	} 
}

void Landscape::allocate_la_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{ 
	Cell* la=get_la(x,y); 

	if(la && la->getPatch() != _patch && la->getCover_id() == _cover_id)
	{ 
		assign_cell_to_patch(x,y,_patch); 

		allocate_la_cel(x,y,_cover_id,_patch); 
		allocate_ra_cel(x,y,_cover_id,_patch);  
		allocate_lb_cel(x,y,_cover_id,_patch); 
		allocate_right_cel(x,y,_cover_id,_patch);  
		allocate_left_cel(x,y,_cover_id,_patch); 
		allocate_above_cel(x,y,_cover_id,_patch);  
		allocate_below_cel(x,y,_cover_id,_patch); 
	} 
}

void Landscape::allocate_lb_cel(int x, int y, cover_type _cover_id, Patch* _patch)
{ 
	Cell* lb=get_lb(x,y); 

	if(lb && lb->getPatch() != _patch && lb->getCover_id() == _cover_id)
	{ 
		assign_cell_to_patch(x,y,_patch); 

		allocate_lb_cel(x,y,_cover_id,_patch); 
		allocate_rb_cel(x,y,_cover_id,_patch); 
		allocate_la_cel(x,y,_cover_id,_patch); 
		allocate_right_cel(x,y,_cover_id,_patch); 
		allocate_left_cel(x,y,_cover_id,_patch); 
		allocate_above_cel(x,y,_cover_id,_patch);  
		allocate_below_cel(x,y,_cover_id,_patch); 
	} 
}

void Landscape::check_area_sums() const throw (IbusepmException)
{
	Logging::log_debug("Landscape::check_area_sums()...",NULL);
	long array_size=xdim*ydim;
	long tot_patch_size=0;
	patch_list::const_iterator patch_iterator = patches.begin();

	while(patch_iterator != patches.end())
		tot_patch_size+=((Patch*)*patch_iterator++)->getSize();

	if(tot_patch_size != array_size)
		throw IbusepmException("Landscape::check_area_sums() total patch size != array size",
								Utility::int_to_char(tot_patch_size),
								Utility::int_to_char(array_size), "\n");
	Logging::log_debug("Landscape::check_area_sums() DONE",NULL);
}


void Landscape::calc_all_patch_perimeters() const throw (IbusepmException)
{
	Logging::log_debug("Landscape::calc_all_patch_perimeters()...",NULL);
	if(patches.empty())
		throw IbusepmException("Landscape::calc_patch_perimeters() #patches = 0!\n");

	patch_list::const_iterator patch_iterator = patches.begin();
	while(patch_iterator != patches.end())
		calc_patch_perimeter(*patch_iterator++);

	Logging::log_debug("Landscape::calc_all_patch_perimeters() DONE",NULL);
}

void Landscape::calc_patch_perimeter(Patch* _patch) const
{
	long perim=0;
	long pcells=0;
	
	long psize=_patch->getSize();
	int ystart=_patch->getYref();
	int xstart=_patch->getXref();
	int xend=_patch->getXmax();
	int yend=_patch->getYmax();

	for(int y=ystart; y<=yend;y++)
    {            
		for(int x=xstart;x<=xend;x++)
		{
			if(grid[x][y].getPatch() == _patch)
			{	
				pcells++;
				perim += cel_edges(x,y,_patch);
				if(pcells==psize)	
					goto ENDPERIM;
			}
		}
	}
	ENDPERIM:
	_patch->setPerimeter(perim);
}

int Landscape::cel_edges(int x, int y, Patch* _patch) const
{
	int edge=0;

	if(x==0)										//right grid edge
		edge++;
	else if(grid[x-1][y].getPatch() != _patch)		//l
		edge++;
	if(x==xdim-1)									//left grid edge
		edge++;
	else if(grid[x+1][y].getPatch() != _patch)		//r 
		edge++;
	if(y==0)										//top grid edge
		edge++;
	else if(grid[x][y-1].getPatch() != _patch)		//u 
		edge++;
	if(y==ydim-1)									//bottom grid edge
		edge++;
	else if(grid[x][y+1].getPatch() != _patch)		//b 
		edge++;

	return edge;
}


void Landscape::calc_all_patchiso_vars(cover_type _cover_id, float meandispdist) throw (IbusepmException)
{
	Logging::log_debug("Landscape::calc_all_patchiso_vars(",
						 Utility::int_to_char((int)_cover_id),
						 Utility::float_to_char(meandispdist),")...",NULL);

	if(patches.empty() || status < PATCH_TOPOLOGY)
		throw IbusepmException("Landscape::calc_all_patchiso_vars() #patches = 0!\n");

	if(buffer1 < 2 || buffer1 >= xdim || buffer1 >= ydim)
		throw IbusepmException("Landscape::calc_all_patchiso_vars() buffer1 ",
								Utility::int_to_char(buffer1), "out of range!\n");
	
	if(buffer2 < 2 || buffer2 >= xdim || buffer2 >= ydim)
        throw IbusepmException("Landscape::calc_all_patchiso_vars() buffer2 ",
								Utility::int_to_char(buffer2), "out of range!\n");
	
	if(buffer3 < 2 || buffer3 >= xdim || buffer3 >= ydim)
        throw IbusepmException("Landscape::calc_all_patchiso_vars() buffer3 ",
								Utility::int_to_char(buffer3), "out of range!\n");
	
	if(neighb_patches == NULL_INT)
		throw IbusepmException("Landscape::calc_all_patchiso_vars() neighb_patches not initialized\n");

	if(cover_type_patches(_cover_id) <= neighb_patches)
        throw IbusepmException("Landscape::calc_all_patchiso_vars() cover_type_patches are less ",
								Utility::int_to_char(cover_type_patches(_cover_id))," than neighb_patches!\n");
	
	if(status == PATCH_ISOL_METRICS)
		Logging::log_error("Landscape::calc_all_patchiso_vars(",
							 Utility::int_to_char((int)_cover_id),
							 Utility::float_to_char(meandispdist),") already calculated.\n");

	int tmpPatchsize;
	patch_list::const_iterator patch_iterator = patches.begin();

	while(patch_iterator != patches.end())
	{
		Patch* tmpPatch = *patch_iterator++;
		tmpPatchsize=tmpPatch->getSize();
		if(tmpPatch->getCover_id() == _cover_id && tmpPatchsize >= min_patch_size && tmpPatchsize <= max_patch_size) 
			calc_patch_iso_vars(tmpPatch,meandispdist);
	}
	calc_patch_cohesion(_cover_id);

	status = PATCH_ISOL_METRICS;
	Logging::log_debug("Landscape::calc_all_patchiso_vars() DONE",NULL);
}

//this method calculates all patch isolation variables for the focal patch 'p'
//this is basically done by calling 'scan_ring_around_cel' for every cel of
//the focal patch, the 'scan_ring...' method calculates closest distances
//to neigbor patches, counts the number of cels of other patches whith the
//same patch_typ within three bufferzones
//all patch isolation variables are than saved in each patch

void Landscape::calc_patch_iso_vars(Patch* _patch, float meandispdist) const
{
	int xstart=_patch->getXref();
	int ystart=_patch->getYref();
	int xend=_patch->getXmax();
	int yend=_patch->getYmax();
	int x,y,i;  
	int ring=1; 
	
	int b1_number;
	int b2_number;
	int b3_number;
	float prox_sum=0;
	float dprox_sum=0;	
	float maxdist=0;
	int psize;
	int ptyp_cells=0;
	int dist_vars_ok=0;
	float dist;

	dvoidlist patchlist;
	dfloatlist distlist;
	dipairlist coordrefs;

	while(ring<=buffer3 || (patchlist.entries()<neighb_patches || maxdist>ring-1))
	{
		ring++;
		for(y=ystart;y<=yend;y++)
		{
			for(x=xstart;x<=xend;x++)
			{
				if(grid[x][y].getPatch() == _patch)  
				{
					maxdist=scan_ring_around_cell(x,y,ring,_patch,patchlist,distlist,
												  coordrefs,ptyp_cells);
				}
			}
		}
		if(patchlist.entries()>=neighb_patches && maxdist<=ring-1 && !dist_vars_ok)
		{
			dfloatlist dtemplist=distlist;
			dtemplist.asc_sort();
			dfloatlist dlist;
                		
			//only the 'neigb_patches' distances go into mean calculation
			for(i=0; i<neighb_patches; i++)
				dlist.add(dtemplist[i]);
			float av_dist=dlist.mean();
        		dfloatlist ddlist;
			float dav_dist;

			//calc exp meandispdist weighted distances
			if(meandispdist != NULL_FLOAT)	//default value in population, means that no distances have been calculated
									//because no dist_obs was defined
			{
				for(i=0; i<neighb_patches; i++)
					ddlist.add(exp(-dlist[i]/meandispdist));
					//calc mean for exp. mdd weighted distances
        			dav_dist=ddlist.mean();
			}
			//single distance to nearest neigb.
        	_patch->add_iso_var(new PatchIsoVar("DIST",1,dlist[0]));
			//average distance to # nearest n.
        	_patch->add_iso_var(new PatchIsoVar("AVDIST",neighb_patches,av_dist));

			if(meandispdist != NULL_FLOAT)				//default value in population, means that no distances have been calculated 
                                                //because no dist_obs was defined
			{
				//exp. mdd weighted single distance
        		_patch->add_iso_var(new PatchIsoVar("DDIST",1,exp(-dlist[0]/meandispdist)));
				//average of exp mdd weighted distances to nearest neigb.
        		_patch->add_iso_var(new PatchIsoVar("AVDDIST",neighb_patches,dav_dist));
			}
			dist_vars_ok++;
		}
		if(ring==buffer1)
		{
			b1_number=patchlist.entries();
			for(i=0; i<b1_number; i++)
			{
				dist=distlist[i];
				if(dist<=ring)			//only patches which are closer than buffer distance
				{
					psize=((Patch*)patchlist[i])->getSize();
					prox_sum+=psize/dist;
					if(meandispdist != NULL_FLOAT)
						dprox_sum+=exp(-dist/meandispdist)*psize;
				}
			}
			_patch->add_iso_var(new PatchIsoVar("PROX1",buffer1,prox_sum));
			
			if(meandispdist != NULL_FLOAT)
				_patch->add_iso_var(new PatchIsoVar("DPRX1",buffer1,dprox_sum));
			
			_patch->add_iso_var(new PatchIsoVar("ABUF",buffer1,ptyp_cells));
		}
		if(ring==buffer2)
		{
			b2_number=patchlist.entries();
			for(i=b1_number; i<b2_number; i++)
			{
				dist=distlist[i];
				if(dist<=ring)	//only patches which are closer than buffer distance
				{
					psize=((Patch*)patchlist[i])->getSize();
					prox_sum+=psize/dist;
					if(meandispdist != NULL_FLOAT)
						dprox_sum+=exp(-dist/meandispdist)*psize;
				}
			}
			 _patch->add_iso_var(new PatchIsoVar("PROX2",buffer2,prox_sum));
				 
			if(meandispdist != NULL_FLOAT)
				_patch->add_iso_var(new PatchIsoVar("DPROX2",buffer2,dprox_sum));
	
			_patch->add_iso_var(new PatchIsoVar("ABUF2",buffer2,ptyp_cells));
		}
		if(ring==buffer3)
		{
			b3_number=patchlist.entries();  
			for(i=b2_number; i<b3_number; i++)
			{ 
				dist=distlist[i]; 
				if(dist<=ring)	//only patches which are closer than buffer distance
				{
                    psize=((Patch*)patchlist[i])->getSize(); 
                    prox_sum+=psize/dist; 
					if(meandispdist != NULL_FLOAT)
						dprox_sum+=exp(-dist/meandispdist)*psize; 
				}
			} 
			_patch->add_iso_var(new PatchIsoVar("PROX3",buffer3,prox_sum));

			if(meandispdist != NULL_FLOAT)
				_patch->add_iso_var(new PatchIsoVar("DPROX3",buffer3,dprox_sum));

			_patch->add_iso_var(new PatchIsoVar("ABUF3",buffer3,ptyp_cells));
		}
	} //while
}

//this method basically goes a ring, as defined by r, around the focal cel, as
//defined by x and y and scans for other patches. If a cel of another patch is found
//the distance is calculated an added into a distance list whose index is identical
//to the patchlist. With growing rings, smaller distances for a already indicated
//patch may be found and are corrected in the distance list
//the return value is the maximum distance in the distance list, this distance should
//be smaller or equal the ring-1 which has been circled
//this method also counts the number of cels of other patches of equal type in
//ptyp_cells  

float Landscape::scan_ring_around_cell(int x, int y, int r, Patch* p1, dvoidlist& pl, 
									   dfloatlist& dl, dipairlist& crefs, int& ptyp_cells) const
{
	int xorig=x;     
	int yorig=y;   
	cover_type p1_type = p1->getCover_id();
	Patch* p2;
	float dist;
    int i, index, xtor, ytor;    
    int go=2*r;
    for(i=0; i<r; i++)      //left above as starting position
		get_la(x,y);

	for(i=0; i<go; i++)     //scan top edge
	{
		Cell* tmpCell = get_right(x,y,1);
		p2 = tmpCell != NULL ? tmpCell->getPatch() : NULL;

		if(p2 && p2!=p1 && p2->getCover_id() == p1_type)
		{
			//calculate distances to remote patch
			dist=calc_cell_dist(xorig,yorig,x,y);
			index = pl.elem_index(p2);
			if(index==-1)		//patch not yet touched
			{					//add patch and dist into lists
				pl.add(p2);
				dl.add(dist);
			}
			else if(dist<dl[index])	//correct dist in list, if smallest 
				dl[index]=dist;
			
			//increase amount of patch cels
			xtor=x; ytor=y;
			if(border_type == TORUS)
				torus_xy_transform(xtor,ytor);
                if(crefs.index_of(xtor,ytor) == -1) //x,y cel not yet added
                {
                    crefs.add(xtor,ytor); 
					ptyp_cells++;
                }
		}
	}
	for(i=0; i<go; i++)     //scan right edge
	{    
		Cell* tmpCell = get_below(x,y,1);
		p2 = tmpCell != NULL ? tmpCell->getPatch() : NULL;

        if(p2 && p2!=p1 && p2->getCover_id() == p1_type)
		{
			//calculate distances to remote patch
			dist=calc_cell_dist(xorig,yorig,x,y);
			index = pl.elem_index(p2);
            if(index==-1)           //patch not yet touched
            {                       //add patch and dist into lists
				pl.add(p2);dl.add(dist);
            }
            else if(dist<dl[index]) //correct dist in list, if smallest
			dl[index]=dist;
			//increase amount of patch cels 
            xtor=x; ytor=y;  
            if(border_type == TORUS) 
				torus_xy_transform(xtor,ytor); 
            if(crefs.index_of(xtor,ytor) == -1) //x,y cel not yet added
            { 
				crefs.add(xtor,ytor); 
				ptyp_cells++; 
            } 
		}
	}
    for(i=0; i<go; i++)     //scan below edge
    {    
		Cell* tmpCell = get_left(x,y,1);
		p2 = tmpCell != NULL ? tmpCell->getPatch() : NULL;

        if(p2 && p2!=p1 && p2->getCover_id() == p1_type)
        {
			//calculate distances to remote patch
			dist=calc_cell_dist(xorig,yorig,x,y);
			index = pl.elem_index(p2);
            if(index==-1)           //patch not yet touched
            {                       //add patch and dist into lists
                pl.add(p2);
				dl.add(dist);
            }
			else if(dist<dl[index]) //correct dist in list, if smallest
				dl[index]=dist;
			//increase amount of patch cels 
            xtor=x; ytor=y;  
            if(border_type == TORUS) 
				torus_xy_transform(xtor,ytor); 
            if(crefs.index_of(xtor,ytor) == -1) //x,y cel not yet added
            { 
                crefs.add(xtor,ytor); 
				ptyp_cells++; 
            } 
		}
    }
    for(i=0; i<go; i++)     //scan left edge
    {    
		Cell* tmpCell = get_above(x,y,1);
		p2 = tmpCell != NULL ? tmpCell->getPatch() : NULL;

        if(p2 && p2!=p1 && p2->getCover_id() == p1_type)
        {
			//calculate distances to remote patch
			dist=calc_cell_dist(xorig,yorig,x,y);
			index = pl.elem_index(p2);
            if(index==-1)           //patch not yet touched
            {                       //add patch and dist into lists
				pl.add(p2);
				dl.add(dist);
            }
            else if(dist<dl[index]) //correct dist in list, if smallest
				dl[index]=dist;
			//increase amount of patch cels  
            xtor=x; ytor=y;   
            if(border_type == TORUS)  
				torus_xy_transform(xtor,ytor);  
            if(crefs.index_of(xtor,ytor) == -1) //x,y cel not yet added 
            { 
                crefs.add(xtor,ytor); 
				ptyp_cells++;  
            }  
		}
    }
	if(dl.entries()>=neighb_patches)
    {
        dfloatlist npdlist;
        for(i=0; i<neighb_patches; i++)
			npdlist.add(dl[i]);
        return npdlist.max();
    }
	return dl.max();
}


//calculates the real distance between two cells within a grid
//taking x,y coordinates from the left upper corner of the cell
//works only in diagonal mode, neigbors connected by corners as well

float Landscape::calc_cell_dist(int x1, int y1, int x2, int y2) const
{
	if(x1>x2 && y1>y2)			//cel2 is left above cel1
		return sqrt(pow(x1-x2-1,2)+pow(y1-y2-1,2));
	else if(x1==x2 && y1>y2)	//cel2 is above cel1
		return y1-y2-1;
	else if(x1<x2 && y1>y2)		//cel2 is right above cel1
		return sqrt(pow(x2-x1-1,2)+pow(y1-y2-1,2));
	else if(x1<x2 && y1==y2)	//cel2 is rigth cel1
		return x2-x1-1;
	else if(x1<x2 && y1<y2)		//cel2 is right below cel1
		return sqrt(pow(x2-x1-1,2)+pow(y2-y1-1,2));
	else if(x1==x2 && y1<y2)	//cel2 is below cel1
		return y2-y1-1;
	else if(x1>x2 && y1<y2)		//cel2 is left below cel1
		return sqrt(pow(x1-x2-1,2)+pow(y2-y1-1,2));
	else if(x1>x2 && y1==y2)	//cel2 is left cel1
		return x1-x2-1;
	return 0;
}

long Landscape::cover_type_patches(cover_type _cover_id) const
{
	patch_list::const_iterator patch_iterator = patches.begin();
	long count=0;

	while(patch_iterator != patches.end())
	{
		if(((Patch*)*patch_iterator++)->getCover_id() == _cover_id)
			count++;
    }
	return count;
}

float Landscape::calc_patch_cohesion(cover_type _cover_id) throw (IbusepmException)
{
	Logging::log_debug("Landscape::calc_patch_cohesion(",
						Utility::int_to_char((int)_cover_id), ") ...",NULL);
	if(patches.empty())
		throw IbusepmException("Landscape::calc_patch_cohesion(",
								Utility::int_to_char((int)_cover_id),
								") #patches = 0!\n");
	float lsize=xdim*ydim;
	Patch* tmpatch;
	float nom_sum=0;	//sum over perimeters
	float denom_sum=0;	//sum over p*sqrt(a)
	float perim;	
	float area;
	float patch_cohesion;

	patch_list::const_iterator patch_iterator = patches.begin();
	while(patch_iterator != patches.end())
	{
		tmpatch=*patch_iterator++;
		if(tmpatch->getCover_id() == _cover_id)
		{
			perim=(float)tmpatch->getPerimeter();
			area=(float)tmpatch->getSize();
			nom_sum+=perim;
			denom_sum+=perim*sqrt(area);
		}
    }
	if(nom_sum && denom_sum && lsize)
		patch_cohesion=(1-nom_sum/denom_sum)/(1-1/sqrt(lsize));

	Logging::log_debug("Landscape::calc_patch_cohesion() DONE",NULL);
	return patch_cohesion;
}

indices_map* Landscape::calc_landscape_indices(cover_type _cover_id) throw (IbusepmException)
{
	Logging::log_debug("Landscape::calc_landscape_indices(",
						Utility::int_to_char((int)_cover_id), ") ...",NULL);
    
	unsigned int f =  getLandCoverAmount(_cover_id);
    double fi = 0;
    double fif = 0;
	unsigned int edge = 0;
	unsigned int num_patches = 0;

	patch_list::const_iterator patch_iterator = patches.begin();
	while(patch_iterator != patches.end())
	{
		Patch* tmpPatch = *patch_iterator++;
		if(tmpPatch->getCover_id() == _cover_id)
		{
			fi+=pow((double)tmpPatch->getSize(),2);
			fif+=pow((double)tmpPatch->getSize()/f,2);
			edge += tmpPatch->getPerimeter();
			num_patches++;
		}
    }
	indices["COVER"] = (float)f;
	indices["EDGE"] = (float)edge;
	indices["EDG/COV"] = (float)edge / (float)(f>0?f:1);
	indices["PATCHES"] = (float)num_patches;
	indices["PTCH_COH"] = calc_patch_cohesion(_cover_id);
	if(f!=0)
		indices["MESHSIZE"] = fi/f;
	indices["F_INDEX"] = pow(f,2)/fi;
	indices["F_DENSIT"] = f/fi;
	indices["F_GRAD"] = 1-fif;
	
	Logging::log_debug("Landscape::calc_landscape_indices() DONE",NULL);
	return &indices;
}

Patch* Landscape::getPatch(float& x, float& y) const throw (IbusepmException)
{
	if(patches.empty())
		throw IbusepmException("Landscape::getPatch(",
								Utility::int_to_char((int)x), Utility::int_to_char((int)y),
								") #patches = 0!\n");

	if((x<0 || x>xdim || y<0 || y>ydim) && border_type != TORUS )   
        return NULL;  
							//caused reflection in individual behaviour
	if(x<0)					//all other cases are torus, walking around a sphere
		while(x<0)			//while's are for very seldom situations, where step > xdim || ydim
			x+=xdim; 
	else if(x>xdim)
		while(x>xdim)
			x-=xdim;
	if(y<0)
		while(y<0)
			y+=ydim;
	else if(y>ydim)
		while(y>ydim)
			y-=ydim;
	return grid[(int)x][(int)y].getPatch();
}

Cell* Landscape::getCell(float& x, float& y) const
{
	if((x<0 || x>xdim || y<0 || y>ydim) && border_type != TORUS )   
        return NULL;  
							//caused reflection in individual behaviour
	if(x<0)					//all other cases are torus, walking around a sphere
		while(x<0)			//while's are for very seldom situations, where step > xdim || ydim
			x+=xdim; 
	else if(x>xdim)
		while(x>xdim)
			x-=xdim;
	if(y<0)
		while(y<0)
			y+=ydim;
	else if(y>ydim)
		while(y>ydim)
			y-=ydim;
	return &grid[(int)x][(int)y];
}

void Landscape::write_to_ascii(const char* _filename) const
{
	Logging::log_debug("Landscape::write_to_ascii(",_filename,") ...",NULL);

	if(status == NOT_INITIALIZED)
		Logging::log_warning("Landscape::write_to_ascii(",_filename,
							") landscape was not initialized!",NULL);

	FILE* _file = fopen(_filename,"wb");
	if(_file == NULL)
	{
		Logging::log_warning("Landscape::write_to_ascii(",_filename,") file could not be opened!",NULL);
		return;
	}
	
	for(int y=0;y<ydim;y++)
	{
		for (int x=0; x<xdim;x++)
		{
			fprintf(_file,"%d ", (int)grid[x][y].getCover_id());
		}
		fprintf(_file,"\n");
	}
	fclose(_file);
	Logging::log_debug("Landscape::write_to_ascii() DONE.",NULL);
}

void Landscape::read_from_ascii(const char* _filename) throw (IbusepmException)
{
	Logging::log_debug("Landscape::read_from_ascii(",_filename,") ...",NULL);

	if(!patches.empty() || status > NOT_INITIALIZED)
	{
		Logging::log_error("Landscape::read_from_ascii(",_filename,
							") will overwrite existing Cell and Patch topology!",NULL);
		reset();
	}

	ifstream is(_filename, ios::in);
	if(is == NULL)
		throw IbusepmException("Landscape::read_from_ascii(",_filename,
							   ") file could not be opened!");

	int xd=0;
	int yd=0;
	char ch;
	int count=0;
	// initial scanning for determining array size
	while(is.get(ch))
    {
		if(ch != '\n' && !yd)
		{
			count++;
			if(ch != ' ')
			{
				if(count == 1)
					xd++;
			}
			else
				count = 0;
		}
		else if(ch == '\n')
			yd++;
    }
	is.close();
	
	// range check
	int x,y;
	if(xd==0 || xd>XMAX)
		throw IbusepmException("Landscape::read_from_ascii(",_filename,
								Utility::int_to_char(xd), 
							   "xd is out of range!");
	if(yd==0 || yd>YMAX)
		throw IbusepmException("Landscape::read_from_ascii(",_filename,
								Utility::int_to_char(yd), 
							   "yd is out of range!");
	xdim=xd;
	ydim=yd;
	ifstream is2(_filename, ios::in);
	if(is2 == NULL)
		throw IbusepmException("Landscape::read_from_ascii(",_filename,
							   ") file could not be opened!");
	//reading of the array
	long arraysize=0;
	char cover_ch[4] = "   ";
	int index;
	for(y=0; y<ydim; y++)
	{
		for(x=0; x<xdim; x++)
		{
			index = 0;
			while(!isdigit(ch))		//skip potential ' ' and \n' digits
				is2.get(ch);

			while(isdigit(ch))		//put contiguous digits into one number
			{	
				cover_ch[index] = ch;
				is2.get(ch);
				index++;
			}

			int cover_id=atoi(cover_ch);
			
			for(int i=0; i<index; i++)
				cover_ch[i]=' ';

			if(cover_id > NULL_TYPE)
				throw IbusepmException("Landscape::read_from_ascii() cover_id: ",
										Utility::int_to_char(cover_id),
										"larger than ",
										Utility::int_to_char(NULL_TYPE-1));
			grid[x][y].setCover_id((cover_type)cover_id); 
			arraysize++;
		}
	}
	if(arraysize!=xdim*ydim)
		throw IbusepmException("Landscape::read_from_ascii() checksum error",
							   " after reading from file.");
	is2.close();

	source = IMPORTED;
	status = CELL_TOPOLOGY;
	Logging::log_debug("Landscape::read_from_ascii(",_filename,") DONE.",NULL);
}

void Landscape::report(ostream& strm) const
{
	patch_list::const_iterator patch_iterator = patches.begin();

	while(patch_iterator != patches.end())
		((Patch*)*patch_iterator++)->report(strm);

	for(int y=0; y<ydim; y++)
	{
		for(int x=0; x<xdim; x++)
			strm<<(int)Utility::round(grid[x][y].getUsed_resources())<<" ";
		strm<<"\n";
	}
}

void Landscape::report_landcovertypes(ostream& strm) const
{
	cover_type_map::const_iterator covertype_iterator = cover_types.begin();

	while(covertype_iterator != cover_types.end())
	{
		pair<cover_type, LandCoverType*> map_element = *covertype_iterator++;
		LandCoverType* tmpLandCoverType = map_element.second;
		tmpLandCoverType->report(strm);
	}
}

#endif 