/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __LANDSCAPE_H
#define __LANDSCAPE_H

#include "defs.h"
#include "cell.h"
#include "landcovertype.h"
#include "ibusepmexception.h"
#include "diarr.h"
#include "dfarr.h"
#include "dipair.h"
#include "dvoidarr.h"
#include "modelregistry.h"

#include <map>
#include <list>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

class Patch;

typedef map<cover_type, LandCoverType*, less<cover_type> > cover_type_map;
typedef list<Patch*> patch_list;
typedef map<const char*, float> indices_map;

/* this are the only global variables, and a compromise
   to accomodate the gifsave function, which requires access
   to this array and map */

extern Cell				grid[XMAX][YMAX];
extern cover_type_map	cover_types;			//stores all cover types

/**
	A Landscape represents a two dimensional array of cells. The location of each 
	cell is defined by its x,y coordinate (index) in the array. A landscape also
	maintains a list of Patch objects and provides algorithms for creating a patch
	topology and a variety of patch related and landscape related indices. 
*/

class Landscape 
{
private:

	int				xdim;					//actual x-dimension
	int				ydim;					//actual y-dimension
	int				xcurr;					//stores current cell during flooding
	int				ycurr;					//stores current cell during flooding
	int				buffer1;				//buffer zone1 in cells for patch isolation metrics
	int				buffer2;				//buffer zone2 in cells for patch isolation metrics
	int				buffer3;				//buffer zone3 in cells for patch isolation metrics
	int				neighb_patches;			//number of nearest neighbour patches 
	int				min_patch_size;			//minimum size of a patch in cells to be included in patch isolation calculation
	int				max_patch_size;			//maximum size of a patch in cells to be included in patch isolation calculation
											//to be involved in distance calculation
	border_types	border_type;			//actual border type
	bool			diagonal;				//if true, diagonal cells belong to one patch
	long			patch_id;				//counter for patch id assignment
	sources			source;					//imported or generated
	status_types	status;					//actual current status of landscape
	patch_list		patches;				//stores pointers to each patch	
	ModelRegistry*  registry;				//central registry for all major model components
	indices_map		indices;				//storage for landscape indices
	char*			filename;				//file name for ascii grid


public:
	Landscape(ModelRegistry*  _registry);
	Landscape(ModelRegistry*  _registry, int _xdim, int _ydim, border_types _border_type);
	~Landscape() {};
	void reset();

	void setFileName(char* _filename)		{ filename = _filename; };
	char* getFileName() const				{ return filename; };

	void setXdim(int _xdim);
	int getXdim() const						{ return xdim; };
	
	void setYdim(int _ydim);
	int getYdim() const						{ return ydim; };

	void setDiagonal(bool _diagonal)        { diagonal=_diagonal; };
	bool isDiagonal() const					{ return diagonal; };	

	void setBuffer1(int _buffer1)			{ buffer1 = _buffer1; };
	int getBuffer1() const					{ return buffer1; };

	void setBuffer2(int _buffer2)			{ buffer2 = _buffer2; };
	int getBuffer2() const					{ return buffer2; };

	void setBuffer3(int _buffer3)			{ buffer3 = _buffer3; };
	int getBuffer3() const					{ return buffer3; };

	void setMinPatchSize(int _size)			{ min_patch_size = _size; };
	int getMinPatchSize() const				{ return min_patch_size; };

	void setMaxPatchSize(int _size)			{ max_patch_size = _size; };
	int getMaxPatchSize() const				{ return max_patch_size; };

	void setNeighbPatches(int _neighb_patches)		{ neighb_patches = _neighb_patches; };
	int getNeighbPatches() const					{ return neighb_patches; };
	
	void setBorder_type(border_types _border_type)	{ border_type = _border_type; };
	border_types getBorder_type() const				{ return border_type; };

	status_types getStatus() const			{ return status; };

	long getLandCoverAmount(cover_type _cover_id) const;
	
	Cell* getCell(float& x, float& y) const;
	Patch* getPatch(float& x, float& y) const throw (IbusepmException);

	void addLandCoverType(LandCoverType* _landcovertype);
	LandCoverType* getLandCoverType(cover_type _cover_id) throw (IbusepmException);
	cover_type_map* getLandCoverTypMap() const		{ return &cover_types; };

	patch_list* getPatchList() { return &patches; };

	void create_all_cox_pattern() throw (IbusepmException);

	void create_patch_topology() throw (IbusepmException);
	void calc_all_patchiso_vars(cover_type _cover_id, float meandispdist) throw (IbusepmException);
	float calc_patch_cohesion(cover_type _cover_id) throw (IbusepmException);
	indices_map* calc_landscape_indices(cover_type _cover_id) throw (IbusepmException);

	void write_to_ascii(const char* _filename) const;
	void read_from_ascii(const char* _filename) throw (IbusepmException);
	void report(ostream& strm=cout) const;
	void report_landcovertypes(ostream& strm=cout) const;
	
private:
	void assign_cell_to_patch(int x, int y, Patch* _patch);
	bool setrefcel(int& xref, int& yref) const;
	Patch* allocate_patch(int x, int y);
	void check_area_sums() const throw (IbusepmException);
	void calc_all_patch_perimeters() const throw (IbusepmException);
	void calc_patch_perimeter(Patch* _patch) const;
	int cel_edges(int x, int y, Patch* _patch) const;
	long cover_type_patches(cover_type _cover_id) const;
	void calc_patch_iso_vars(Patch* _patch, float meandispdist) const;
	float calc_cell_dist(int x1, int y1, int x2, int y2) const;
	float scan_ring_around_cell(int x, int y, int r, Patch* p1, dvoidlist& pl, 
								dfloatlist& dl, dipairlist& crefs, int& ptyp_cells) const;

	void create_cox_pattern(LandCoverType& _landcovertype) throw (IbusepmException);

	//functions for the recursive flooding algorithm
	void allocate_right_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_left_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_above_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_below_cel(int x, int y, cover_type _cover_id, Patch* _patch);

	void allocate_ra_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_rb_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_la_cel(int x, int y, cover_type _cover_id, Patch* _patch);
	void allocate_lb_cel(int x, int y, cover_type _cover_id, Patch* _patch);

	bool equal_neighbour_state(int x, int y, cover_type cover_id) const;
	bool torus_xy_transform(int& x, int& y) const;

	//functions for flooding
	Cell* get_right(int& x, int& y, int c=0) const;    
    Cell* get_left(int& x, int& y, int c=0) const; 
    Cell* get_above(int& x, int& y, int c=0) const;
    Cell* get_below(int& x, int& y, int c=0) const; 

    Cell* get_ra(int& x, int& y) const;
    Cell* get_rb(int& x, int& y) const; 
    Cell* get_la(int& x, int& y) const;
    Cell* get_lb(int& x, int& y) const; 

	void resetLandCoverTypes();
};

#endif