/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __MODELCONTROLLER_CPP
#define __MODELCONTROLLER_CPP

#include <stdlib.h>
#include <time.h>
#include "modelcontroller.h"
#include "modelregistry.h"
#include "landscape.h"
#include "defs.h"
#include "simulator.h"
#include "parareader.h"
#include "logging.h"
#include "outputwriter.h"

extern char* version;

ModelController::ModelController(int _argc, char** _argv)
{
	argcount =	_argc;
	arguments = _argv;
	starttime = 0;	
	endtime =	0;
	simtime =	0;
}

void ModelController::execute()
{
	Logging::log_info(false, "EDGE version ",version," Copyright (C) 2005 Lutz Tischendorf\n\n",NULL);
	try
	{
		time(&starttime);
		read_parameters();
		randomize();
		do
		{
			initialize();
			simulate();
			calculate_stats();
			write_results();
			//enforce log rotation, when log > 10MB
			Logging::set_destination("edge.log"); 
		}
		while(next_parameter_combination());
		time(&endtime);
		final_message();
	}
	catch (IbusepmException &e)
	{	
		Logging::log_error("\n\n", e.what(), "\n",NULL);
		Logging::log_error("Program terminated abnormally!",NULL);
	}	
}

void ModelController::randomize()
{
	srand(registry.parameters->getStartSeed());
}

void ModelController::read_parameters() throw (IbusepmException)
{
	if(argcount <= 1)
		help();

	char* argument; 	
	char string[5];
	for(int arg=1; arg<argcount; arg++)
	{
		argument=strdup(arguments[arg]);
		if(!strcmp("/help",argument) || !strcmp("-help",argument))
			help();	

		int ti=0;
		while(*(argument+ti++));
		ti-=4;
		for(int l=0; l<4; l++)
			string[l]=*(argument+l+ti);
		if(!strcmp("xml",string))
		{
			XMLParameterReader* XMLReader = new XMLParameterReader(&registry, argument);
			XMLReader->readFile();
			XMLReader->parseParameters();
		}
		else if(!strcmp("par",string))
        {
			ASCIIParameterReader* ASCIIReader = new ASCIIParameterReader(&registry, argument);
			ASCIIReader->parseParameters();
        }
		else
			help();
	}
	OutputWriter* writer = new OutputWriter(&registry);
}

void ModelController::initialize()
{
	registry.parameters->initModel();
}
	
void ModelController::write_results()
{
	registry.writer->report_stat_observations();
	registry.writer->report_raw_observations();
}

void ModelController::simulate()
{
	registry.simulator->simulate();
}

bool ModelController::next_parameter_combination()
{
	return registry.parameters->nextParameterCombination();
}

void ModelController::calculate_stats()
{
	observer_list* observers = registry.getObservers();
	observer_list::iterator observer_iterator = observers->begin();
	while(observer_iterator != observers->end())
		(*observer_iterator++)->calc_stats();
}

void ModelController::help() const
{
	cout<<"EDGE-1 requires one parameter file to run. Please consult the EDGE-1 User Guide!\n";
	exit(1);
}

void ModelController::final_message()
{
	char* message = new char[200];
	sprintf(message,"Simulation successfully completed after %d seconds! ;-)\n",(int)difftime(endtime,starttime));
	Logging::log_info(false,"\n",NULL);
	Logging::log_info(true,message,NULL);
}

#endif