/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __MODEL_REGISTRY_CPP
#define __MODEL_REGISTRY_CPP

#include "modelregistry.h"
#include "observer.h"
#include <algorithm>

ModelRegistry::ModelRegistry()
{
	simulator = NULL;
	landscape = NULL;
	parameters = NULL;
}

void ModelRegistry::register_simulator(Simulator* _simulator)
{
	simulator = _simulator;
}

void ModelRegistry::register_landscape(Landscape* _landscape)
{
	landscape = _landscape;
}

void ModelRegistry::register_parameters(ParameterReader* _parameters)
{
	parameters = _parameters;
}

void ModelRegistry::register_outputwriter(OutputWriter* _writer)
{
	writer = _writer;
}

void ModelRegistry::register_population(Population* _population)
{
	populations[_population->getId()] = _population;
}

void ModelRegistry::register_observer(Observer* _observer)
{
	observers.push_back(_observer);
}

Population* ModelRegistry::getPopulation(int _id)	
{
	return populations[_id]; 
}

Observer* ModelRegistry::getObserver(Observer* _observer)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		if(*(*observer_iterator++) == *_observer)
			return *observer_iterator;
	}
	return NULL;
}

void ModelRegistry::resetObservers()
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(tempObserver)
		{
			tempObserver->reset();
			delete tempObserver;
		}
	}
	observers.clear();
}

void ModelRegistry::updateCellImmigrationObserver(unsigned short _population_id, unsigned int _individual_id, 
												  cover_type _cover_id, float _xpos, float _ypos)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(!strcmp(tempObserver->getName(),"CellImmigrationObserver"))
		{
			CellImmigrationObserver* cimObserver = (CellImmigrationObserver*)tempObserver;
			if(cimObserver->get_population_id() == _population_id && 
			   cimObserver->get_cover_id() == _cover_id)
			{
				cimObserver->update(_individual_id, (int)_xpos, (int)_ypos);
				observer_iterator = observers.end();
			}
		}
	}
}

void ModelRegistry::updatePatchImmigrationObserver(unsigned short _population_id, unsigned int _frompatch_id, unsigned int _topatch_id)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(!strcmp(tempObserver->getName(),"PatchImmigrationObserver"))
		{
			PatchImmigrationObserver* pimObserver = (PatchImmigrationObserver*)tempObserver;
			if(pimObserver->get_population_id() == _population_id)
			{
				pimObserver->update(_frompatch_id, _topatch_id);
				observer_iterator = observers.end();
			}
		}
	}
}

void ModelRegistry::updateDistanceObserver(unsigned short _population_id, unsigned int _individual_id, bool _start, float _xpos, float _ypos)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(!strcmp(tempObserver->getName(),"DistanceObserver"))
		{
			DistanceObserver* distObserver = (DistanceObserver*)tempObserver;
			if(distObserver->get_population_id() == _population_id)
			{
				distObserver->update(_individual_id, _start, _xpos, _ypos);
				observer_iterator = observers.end();
			}
		}
	}
}

void ModelRegistry::updateCrossingObserver(unsigned short _population_id, cover_type _from_cover, cover_type _to_cover)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(!strcmp(tempObserver->getName(),"CrossingObserver"))
		{
			CrossingObserver* crossObserver = (CrossingObserver*)tempObserver;
			if(crossObserver->get_population_id() == _population_id && 
			   crossObserver->get_from_cover() == _from_cover &&
			   crossObserver->get_to_cover() == _to_cover)
			{
				crossObserver->update();
				observer_iterator = observers.end();
			}
		}
	}
}

void ModelRegistry::updateSearchTimeObserver(unsigned short _population_id, unsigned int _steps)
{
	observer_list::iterator observer_iterator = observers.begin();
	while(observer_iterator != observers.end())
	{
		Observer* tempObserver = *observer_iterator++;
		if(!strcmp(tempObserver->getName(),"SearchTimeObserver"))
		{
			SearchTimeObserver* searchObserver = (SearchTimeObserver*)tempObserver;
			if(searchObserver->get_population_id() == _population_id) 
			{
				searchObserver->update(_steps);
				observer_iterator = observers.end();
			}
		}
	}
}

#endif