/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __OBSERVER_CPP
#define __OBSERVER_CPP

#include "observer.h"
#include "population.h"
#include "modelregistry.h"
#include "logging.h"
#include "ibusutil.h"
#include "landscape.h"
#include "patch.h"
#include "individual.h"

bool Observer::msg = false;

Observer::Observer(ModelRegistry* _registry, char* _name, unsigned short _runs, unsigned short _obsteps)
{
	registry = _registry;
	name = _name;

	varname = NULL;
	runs = _runs;         	
	obsteps	= _obsteps;
	report_raw_obs = false;
	registry->register_observer(this);
}

Observer::~Observer()
{
}

void Observer::reset()
{
}

bool Observer::operator == (const Observer& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs;
}

bool Observer::operator!=(const Observer& _observer) const
{
	return !((*this)==_observer);
}

const char* Observer::getUniqueAbbreviatedName()
{
	return name;
}


//***************************************************************************************************************

PopulationObserver::PopulationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps)
				   : Observer(_registry, "PopulationObserver", _runs, _obsteps)
{
	population_id = _population_id;
	varname = "AB_P";
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	abbrev_name.append(varname);
	abbrev_name.append("");
	abbrev_name.append(Utility::int_to_char(population_id));
}

PopulationObserver::~PopulationObserver()
{
	Observer::~Observer();
}

void PopulationObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
}

const char* PopulationObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool PopulationObserver::operator == (const PopulationObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id;
}

void PopulationObserver::observe(int run, int obstep)
{
	Logging::log_debug("PopulationObserver::observe() ...",NULL);
	
	Population* pop = registry->getPopulation(population_id);
	if(pop && run < runs && obstep < obsteps)
   		(*raw_observations)[obstep][run]=pop->getSize();
	else
		Logging::log_error("PopulationObserver::observe() population not found, or run or obstep out of range!",NULL);
	
	Logging::log_debug("PopulationObserver::observe() done.",NULL);
}

void PopulationObserver::calc_stats()
{
	Logging::log_debug("PopulationObserver::calc_stats() ...",NULL);
	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}
	Logging::log_debug("PopulationObserver::calc_stats() done.",NULL);
}

void PopulationObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void PopulationObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void PopulationObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void PopulationObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

ExtinctionObserver::ExtinctionObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps)
				   : Observer(_registry, "ExtinctionObserver", _runs, _obsteps)
{
	population_id = _population_id;
	varname = "EX_P";
	record = true;
	raw_extinctions = new floatlist(obsteps);
	cumulative_extinctions = new floatlist(obsteps);
	relative_extinctions = new floatlist(obsteps);
	abbrev_name.append(varname);
	abbrev_name.append(Utility::int_to_char(population_id));
}

ExtinctionObserver::~ExtinctionObserver()
{
	Observer::~Observer();
}

void ExtinctionObserver::reset()
{
	raw_extinctions->reset();
    delete raw_extinctions;
	cumulative_extinctions->reset();
    delete cumulative_extinctions;
	relative_extinctions->reset();
    delete relative_extinctions;
	record = true;
}

const char* ExtinctionObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool ExtinctionObserver::operator == (const ExtinctionObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id;
}

void ExtinctionObserver::observe(int run, int obstep)
{
	Logging::log_debug("ExtinctionObserver::observe() ...",NULL);

	Population* pop = registry->getPopulation(population_id);
	if(pop && run<=runs && obstep<=obsteps)
	{
		if(obstep == 0)
			record = true;
		if(pop->getSize() == 0)
		{
			if(record)
			{
   				(*raw_extinctions)[obstep]++;
				record = false;
			}
			(*cumulative_extinctions)[obstep]++;
		}
	}
	else
		Logging::log_error("ExtinctionObserver::observe() population not found, or run or obstep out of range!",NULL);

	Logging::log_debug("ExtinctionObserver::observe() done.",NULL);
}

void ExtinctionObserver::calc_stats()
{
	Logging::log_debug("ExtinctionObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*relative_extinctions)[obstep] = (*raw_extinctions)[obstep]/(double)runs;
		(*cumulative_extinctions)[obstep]/=(double)runs;
	}
	Logging::log_debug("ExtinctionObserver::calc_stats() done.",NULL);
}

void ExtinctionObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_CT,";
	strm<<varname<<pop_id<<"_RE,";
	strm<<varname<<pop_id<<"_CU,";
}

void ExtinctionObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	strm<<(*raw_extinctions)[_timestep]<<",";
	strm<<(*relative_extinctions)[_timestep]<<",";
	strm<<(*cumulative_extinctions)[_timestep]<<",";
}

//***************************************************************************************************************

LandCoverObserver::LandCoverObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id)
				   : Observer(_registry, "LandCoverObserver", _runs, _obsteps)
{
	population_id = _population_id;
	cover_id = _cover_id;
	if(cover_id == 1)
		varname = "HA_P";
	else
		varname = "MA_P";
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	abbrev_name.append(varname);
	abbrev_name.append("");
	abbrev_name.append(Utility::int_to_char(population_id));
}

LandCoverObserver::~LandCoverObserver()
{
	Observer::~Observer();
}

void LandCoverObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
}

const char* LandCoverObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool LandCoverObserver::operator == (const LandCoverObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id;
}

void LandCoverObserver::observe(int run, int obstep)
{
	Logging::log_debug("LandCoverObserver::observe() ...",NULL);

	Population* pop = registry->getPopulation(population_id);
	if(pop && run < runs && obstep < obsteps)
   		(*raw_observations)[obstep][run]=pop->getSize_on_covertype(cover_id);
	else
		Logging::log_error("LandCoverObserver::observe() population not found, or run or obstep out of range!",NULL);
	
	Logging::log_debug("LandCoverObserver::observe() done.",NULL);
}

void LandCoverObserver::calc_stats()
{
	Logging::log_debug("LandCoverObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	Logging::log_debug("LandCoverObserver::calc_stats() done.",NULL);
}

void LandCoverObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void LandCoverObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void LandCoverObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void LandCoverObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

CellImmigrationObserver::CellImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id)
						 : Observer(_registry, "CellImmigrationObserver", _runs, _obsteps)
{
	population_id = _population_id;
	cover_id = _cover_id;
	varname = "CI_P";
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	cim_observations = new cim_counter();
	type_cover = registry->landscape->getLandCoverAmount(cover_id);
	long cells = registry->landscape->getXdim() * registry->landscape->getYdim();
	if(cells > 100000 && !msg)
	{
		string* message = new string("Current grid size is ");
		message->append(Utility::int_to_char(cells));
		message->append(".\nIt is not advised to use Cell Immigration Observers at grid sizes > 100000 cells");
		message->append("\nbecause of potential memory overuse. Please use smaller grids with this Observer to");
		message->append("\navoid potential memory shortage.\n\n");
		Logging::log_warning(message->c_str(),NULL);
		delete message;
		msg = true;
	}	
	abbrev_name.append(varname);
	abbrev_name.append(Utility::int_to_char(population_id));
}

CellImmigrationObserver::~CellImmigrationObserver()
{
	Observer::~Observer();
}

void CellImmigrationObserver::reset()
{
	reset_cim_counter();
	delete cim_observations;
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
}

const char* CellImmigrationObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool CellImmigrationObserver::operator == (const CellImmigrationObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
			population_id == _observer.population_id;
}

void CellImmigrationObserver::reset_cim_counter()
{
	Logging::log_debug("CellImmigrationObserver::reset_cim_counter() ...",NULL);

	cim_counter::iterator cim_iterator = cim_observations->begin();
	while(cim_iterator != cim_observations->end())
	{
		dipairlist* tmplist = (*cim_iterator++).second;
		if(tmplist)
		{
			tmplist->reset();
			delete tmplist;
		}
    }
	cim_observations->clear();

	Logging::log_debug("CellImmigrationObserver::reset_cim_counter() done.",NULL);
}

//this function is called from an individual at the end of its
//activity function
void CellImmigrationObserver::update(unsigned int _individual_id, int _xcel, int _ycel)
{
	if(!(*cim_observations)[_individual_id])
	{
		dipairlist* xylist = new dipairlist();
		xylist->add(_xcel,_ycel);
		(*cim_observations)[_individual_id] = xylist;
	}
	else if((*cim_observations)[_individual_id]->index_of(_xcel,_ycel) == -1)
		(*cim_observations)[_individual_id]->add(_xcel,_ycel);
}

//this function is called by the simulator at the regular
//observation inverval
void CellImmigrationObserver::observe(int _run, int _obstep)
{
	Logging::log_debug("CellImmigrationObserver::observe() ...",NULL);

	if(_run < runs && _obstep < obsteps)
	{
		unsigned int cim_count = 0;
		cim_counter::const_iterator cim_iterator = cim_observations->begin();
		while(cim_iterator != cim_observations->end())
			cim_count += (*cim_iterator++).second->entries();
		unsigned int max_popsize = registry->getPopulation(population_id)->getMaximum_size();
		if(max_popsize * type_cover > 0)
			(*raw_observations)[_obstep][_run]=(float)cim_count*100/((float)max_popsize*(float)type_cover);
		if(_obstep == obsteps-1)
			reset_cim_counter();
	}
	else
		Logging::log_error("CellImmigrationObserver::observe() run or obstep out of range!",NULL);

	Logging::log_debug("CellImmigrationObserver::observe() done.",NULL);
}

void CellImmigrationObserver::calc_stats()
{
	Logging::log_debug("CellImmigrationObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	Logging::log_debug("CellImmigrationObserver::calc_stats() done.",NULL);
}

void CellImmigrationObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void CellImmigrationObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void CellImmigrationObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void CellImmigrationObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

PatchImmigrationObserver::PatchImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id)
						 : Observer(_registry, "PatchImmigrationObserver", _runs, _obsteps)
{
	population_id = _population_id;
	cover_id = _cover_id;
	varname = "PI_P";
	num_patches = 0;
	img_counter = 0;
	current_run = 0;
	header = false;
	patch_sizes = new index_map();
	patch_index_ids = new index_map();
	patch_pop = new patch_population();
	patch_list* plist = registry->landscape->getPatchList();
	patch_list::const_iterator patch_iterator = plist->begin();
	while(patch_iterator != plist->end())
	{
		Patch* tmpPatch = *patch_iterator++;
		if(tmpPatch->getCover_id() == cover_id)
		{
			(*patch_pop)[num_patches] = 0;
			(*patch_sizes)[num_patches] = (float)tmpPatch->getSize();
			(*patch_index_ids)[tmpPatch->getPatch_id()] = num_patches++;
		}
	}
	pim_cube = new dfloatcube(runs,num_patches,num_patches);
	pim_matrix = new dfloatarray(num_patches,num_patches);
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	long cells = registry->landscape->getXdim() * registry->landscape->getYdim();
	if(cells > 100000 && !msg)
	{
		string* message = new string("Current grid size is ");
		message->append(Utility::int_to_char(cells));
		message->append(".\nIt is not advised to use Patch Immigration Observers at grid sizes > 100000 cells");
		message->append("\nbecause of potential memory overuse. Please use smaller grids with this Observer to");
		message->append("\navoid potential memory shortage.\n\n");
		Logging::log_warning(message->c_str(),NULL);
		delete message;
		msg = true;
	}
	abbrev_name.append(varname);
	abbrev_name.append(Utility::int_to_char(population_id));
}

PatchImmigrationObserver::~PatchImmigrationObserver()
{
	Observer::~Observer();
}

void PatchImmigrationObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
	pim_cube->reset();
	delete pim_cube;
	pim_matrix->reset();
	delete pim_matrix;
	patch_sizes->clear();
	patch_index_ids->clear();
	patch_pop->clear();
	delete patch_sizes;
	delete patch_index_ids;
	delete patch_pop;
	img_counter = 0;
	current_run = 0;
}

const char* PatchImmigrationObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool PatchImmigrationObserver::operator == (const PatchImmigrationObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
			population_id == _observer.population_id &&
			cover_id == _observer.cover_id;
}

void PatchImmigrationObserver::record_patch_population_size()
{
	Logging::log_debug("PatchImmigrationObserver::record_patch_population_size() ...",NULL);

	patch_list* plist = registry->landscape->getPatchList();
	patch_list::const_iterator patch_iterator = plist->begin();
	while(patch_iterator != plist->end())
	{
		Patch* tmpPatch = *patch_iterator++;
		if(tmpPatch->getCover_id() == cover_id)
			(*patch_pop)[(*patch_index_ids)[tmpPatch->getPatch_id()]] += tmpPatch->getIndividuals(population_id);
	}

	Logging::log_debug("PatchImmigrationObserver::record_patch_population_size() done.",NULL);
}

void PatchImmigrationObserver::update(unsigned int _frompatch_id, unsigned int _topatch_id)
{
	(*pim_cube)[(*patch_index_ids)[_frompatch_id]][(*patch_index_ids)[_topatch_id]][current_run]++;
	img_counter++;
}

void PatchImmigrationObserver::observe(int _run, int _obstep)
{
	Logging::log_debug("PatchImmigrationObserver::observe() ...",NULL);

	if(_run < runs && _obstep < obsteps)
	{
		record_patch_population_size();
		(*raw_observations)[_obstep][_run] = (float)img_counter;

		if(_obstep == obsteps-1)
		{
			float max_pop = (float)registry->getPopulation(population_id)->getMaximum_size();
			for(int o = 0; o < obsteps; o++)
				(*raw_observations)[o][_run] /= max_pop != 0 ? max_pop : 1;
			current_run++;
			img_counter = 0;
		}
	}
	else
		Logging::log_error("PatchImmigrationObserver::observe() run or obstep out of range!",NULL);

	Logging::log_debug("PatchImmigrationObserver::observe() done.",NULL);
}

void PatchImmigrationObserver::calc_stats()
{
	Logging::log_debug("PatchImmigrationObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	//relative average patch population size over time and runs 
	float factor = (float)runs * (float)obsteps;
	if(factor > 0)
	{
		patch_population::iterator iterator = patch_pop->begin();
		while(iterator != patch_pop->end())
			(*iterator++).second /= factor;
	}

	float pfactor = 1;
	for(unsigned int i=0; i<num_patches; i++)
	{
		if(i == 0)
			pfactor = ((*patch_pop)[0] != 0 ? (*patch_pop)[0] : 1);
		for(unsigned int j=0; j<num_patches; j++)
		{
			if(i > 0)
				pfactor = ((*patch_pop)[j] != 0 ? (*patch_pop)[j] : 1);
			(*pim_matrix)[i][j] = (*pim_cube)[i][j].mean() / pfactor;
		}
	}

	Logging::log_debug("PatchImmigrationObserver::calc_stats() done.",NULL);
}

void PatchImmigrationObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void PatchImmigrationObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void PatchImmigrationObserver::report_run_names(ostream& strm)
{
}

void PatchImmigrationObserver::report_raw_observations(int _timestep, ostream& strm)
{
	if(_timestep == obsteps - 1)
	{
		if(!header)
		{
			header = true;
			strm<<"\nPatch Immigration Matrix\n,";
			for(unsigned int i=1; i<=num_patches; i++)
				strm<<"P"<<i<<"_"<<(int)(*patch_sizes)[i-1]<<",";
			strm<<"\n";
		}
		for(int i=0; i<num_patches; i++)
		{
			strm<<"P"<<(i+1)<<"_"<<(int)(*patch_sizes)[i]<<",";
			for(unsigned int j=0; j<num_patches; j++)
			{
				strm<<(*pim_matrix)[i][j]<<",";
				if(j == num_patches-1)
					strm<<"\n";
			}
		}
	}
}

//***************************************************************************************************************

DistanceObserver::DistanceObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps)
				 : Observer(_registry, "DistanceObserver", _runs, _obsteps)
{
	population_id = _population_id;
	varname = "DI_P";
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	start_positions = new position_map();
	end_positions = new position_map();
	init_start_positions();
	last_run = 0;
	abbrev_name.append(varname);
	abbrev_name.append("");
	abbrev_name.append(Utility::int_to_char(population_id));

}

DistanceObserver::~DistanceObserver()
{
	Observer::~Observer();
}

void DistanceObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
	start_positions->clear();
	end_positions->clear();
	delete start_positions;
	delete end_positions;
}

const char* DistanceObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool DistanceObserver::operator == (const DistanceObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id;
}

void DistanceObserver::init_start_positions()
{
	Logging::log_debug("DistanceObserver::init_start_positions() ...",NULL);

	start_positions->clear();
	end_positions->clear();
	individual_list* individuals = registry->getPopulation(population_id)->get_individuals();
	individual_list::const_iterator iterator = individuals->begin();
	while(iterator != individuals->end())
	{
		Individual* tmpInd = *iterator++;
		(*start_positions)[tmpInd->getId()].x = tmpInd->getXpos();
		(*start_positions)[tmpInd->getId()].y = tmpInd->getYpos();
	}

	Logging::log_debug("DistanceObserver::init_start_positions() done.",NULL);
}

void DistanceObserver::update_end_positions()
{
	Logging::log_debug("DistanceObserver::update_end_positions() ...",NULL);

	individual_list* individuals = registry->getPopulation(population_id)->get_individuals();
	individual_list::const_iterator iterator = individuals->begin();
	while(iterator != individuals->end())
	{
		Individual* tmpInd = *iterator++;
		(*end_positions)[tmpInd->getId()].x = tmpInd->getXpos();
		(*end_positions)[tmpInd->getId()].y = tmpInd->getYpos();
	}

	Logging::log_debug("DistanceObserver::update_end_positions() done.",NULL);
}

void DistanceObserver::update(unsigned int _individual_id, bool _start, float _xpos, float _ypos)
{
	if(_start)
	{
		(*start_positions)[_individual_id].x = _xpos;
		(*start_positions)[_individual_id].y = _ypos;
	}
	else
	{
		(*end_positions)[_individual_id].x = _xpos;
		(*end_positions)[_individual_id].y = _ypos;
	}
}

void DistanceObserver::observe(int _run, int _obstep)
{
	Logging::log_debug("DistanceObserver::observe() ...",NULL);

	if(_run < runs && _obstep < obsteps)
	{
		if(_run > last_run)
		{
			init_start_positions();
			last_run = _run;
		}
		float distance = 0;
		unsigned int individuals = 0;
		update_end_positions();

		position_map::iterator iterator = end_positions->begin();
		while(iterator != end_positions->end())
		{	
			float x_end = (*iterator).second.x;
			float y_end = (*iterator).second.y;
			float x_start = (*start_positions)[(*iterator).first].x;
			float y_start = (*start_positions)[(*iterator++).first].y;
			distance += sqrt(pow(x_end-x_start,2) + pow(y_end-y_start,2));
			individuals++;
		}
		distance /= individuals > 0 ? individuals : 1;
   		(*raw_observations)[_obstep][_run] = distance;
	}
	else
		Logging::log_error("DistanceObserver::observe() run or obstep out of range!",NULL);

	Logging::log_debug("DistanceObserver::observe() done.",NULL);
}


void DistanceObserver::calc_stats()
{
	Logging::log_debug("DistanceObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	Logging::log_debug("DistanceObserver::calc_stats() done.",NULL);
}

void DistanceObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void DistanceObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void DistanceObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void DistanceObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

CrossingObserver::CrossingObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _from, cover_type _to)
				   : Observer(_registry, "CrossingObserver", _runs, _obsteps)
{
	population_id = _population_id;
	from_cover = _from;
	to_cover = _to;
	varname = "C";
	crossings = 0;
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	abbrev_name.append(varname);
	const char* cross_type = (from_cover == LandCoverType::matrix && to_cover == LandCoverType::habitat) ? "MH" : "HM";
	abbrev_name.append("");
	abbrev_name.append(Utility::int_to_char(population_id));
	abbrev_name.append("_");
	abbrev_name.append(cross_type);
}

CrossingObserver::~CrossingObserver()
{
	Observer::~Observer();
}

void CrossingObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
	crossings = 0;
}

const char* CrossingObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool CrossingObserver::operator == (const CrossingObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id &&
			to_cover == _observer.to_cover &&
			from_cover == _observer.from_cover;
}

void CrossingObserver::update()
{
	crossings++;
}

void CrossingObserver::observe(int _run, int _obstep)
{
	Logging::log_debug("CrossingObserver::observe() ...",NULL);

	if(_run < runs && _obstep < obsteps)
	{
		unsigned int movement_steps = registry->getPopulation(population_id)->getSteps();
		(*raw_observations)[_obstep][_run] = (float)crossings / (float)(movement_steps > 0 ? movement_steps : 1);
		if(_obstep == obsteps-1)
			crossings = 0;
	}
	else
		Logging::log_error("CrossingObserver::observe() run or obstep out of range!",NULL);

	Logging::log_debug("CrossingObserver::observe() done.",NULL);
}

void CrossingObserver::calc_stats()
{
	Logging::log_debug("CrossingObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	Logging::log_debug("CrossingObserver::calc_stats() done.",NULL);
}

void CrossingObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	const char* cross_type = (from_cover == LandCoverType::matrix && to_cover == LandCoverType::habitat) ? "MH" : "HM";
	strm<<varname<<pop_id<<"_"<<cross_type<<"_AV,";
	strm<<varname<<pop_id<<"_"<<cross_type<<"_MI,";
	strm<<varname<<pop_id<<"_"<<cross_type<<"_MA,";
	strm<<varname<<pop_id<<"_"<<cross_type<<"_VA,";
	strm<<varname<<pop_id<<"_"<<cross_type<<"_KO,";
}

void CrossingObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void CrossingObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void CrossingObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

SearchTimeObserver::SearchTimeObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps)
				   : Observer(_registry, "SearchTimeObserver", _runs, _obsteps)
{
	population_id = _population_id;
	varname = "ST_P";
	search_steps = 0;
	individuals = 0;
	raw_observations = new dfloatarray(runs,obsteps);
	stat_observations = new dfloatarray(STATSVAR,obsteps);
	abbrev_name.append(varname);
	abbrev_name.append("");
	abbrev_name.append(Utility::int_to_char(population_id));
}

SearchTimeObserver::~SearchTimeObserver()
{
	Observer::~Observer();
}

void SearchTimeObserver::reset()
{
	raw_observations->reset();
    delete raw_observations;
	stat_observations->reset();
	delete stat_observations;
	search_steps = 0;
	individuals = 0;
}

const char* SearchTimeObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool SearchTimeObserver::operator == (const SearchTimeObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs && 
			population_id == _observer.population_id;
}

void SearchTimeObserver::update(unsigned int _steps)
{
	search_steps += _steps;
	individuals++;
}

void SearchTimeObserver::observe(int _run, int _obstep)
{
	Logging::log_debug("SearchTimeObserver::observe() ...",NULL);

	if(_run < runs && _obstep < obsteps)
	{
		(*raw_observations)[_obstep][_run] = float(search_steps) / (float)(individuals > 0 ? individuals : 1);
		if(_obstep == obsteps-1)
		{
			search_steps = 0;
			individuals = 0;
		}
	}
	else
		Logging::log_error("SearchTimeObserver::observe() run or obstep out of range!",NULL);

	Logging::log_debug("SearchTimeObserver::observe() done.",NULL);
}

void SearchTimeObserver::calc_stats()
{
	Logging::log_debug("SearchTimeObserver::calc_stats() ...",NULL);

	for(int obstep=0; obstep<obsteps; obstep++)
	{
		(*stat_observations)[obstep][MEAN]	=(*raw_observations)[obstep].mean();
      	(*stat_observations)[obstep][MIN]	=(*raw_observations)[obstep].min();
      	(*stat_observations)[obstep][MAX]	=(*raw_observations)[obstep].max();
      	(*stat_observations)[obstep][VAR]	=(*raw_observations)[obstep].varianz();
      	(*stat_observations)[obstep][STDERR]=(*raw_observations)[obstep].stderro();
	}

	Logging::log_debug("SearchTimeObserver::calc_stats() done.",NULL);
}

void SearchTimeObserver::report_variable_names(ostream& strm) const
{
	unsigned short pop_id = registry->getPopulation(population_id)->getId();
	strm<<varname<<pop_id<<"_AV,";
	strm<<varname<<pop_id<<"_MI,";
	strm<<varname<<pop_id<<"_MA,";
	strm<<varname<<pop_id<<"_VA,";
	strm<<varname<<pop_id<<"_KO,";
}

void SearchTimeObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	(*stat_observations)[_timestep].report(strm);
}

void SearchTimeObserver::report_run_names(ostream& strm)
{
	for(int run = 1; run <= (*raw_observations)[0].entries(); run++)
		strm<<"RUN_"<<run<<",";
}

void SearchTimeObserver::report_raw_observations(int _timestep, ostream& strm)
{
	(*raw_observations)[_timestep].report(strm);
}

//***************************************************************************************************************

LandscapeObserver::LandscapeObserver(ModelRegistry* _registry, unsigned short _runs, unsigned short _obsteps)
				   : Observer(_registry, "LandscapeObserver", _runs, _obsteps)
{
	Logging::log_debug("LandscapeObserver::LandscapeObserver() ...",NULL);

	varname = "LAND";
	indices = registry->landscape->calc_landscape_indices(LandCoverType::habitat);
	abbrev_name.append(varname);

	Logging::log_debug("LandscapeObserver::LandscapeObserver() done.",NULL);
}

LandscapeObserver::~LandscapeObserver()
{
	Observer::~Observer();
}

const char* LandscapeObserver::getUniqueAbbreviatedName()
{
	return abbrev_name.c_str();
}

bool LandscapeObserver::operator == (const LandscapeObserver& _observer) const
{
	return 	name == _observer.name &&
			runs == _observer.runs &&
			obsteps == _observer.obsteps &&
	 		report_raw_obs == _observer.report_raw_obs; 
}

void LandscapeObserver::report_variable_names(ostream& strm) const
{
	indices_map::iterator iterator = indices->begin();
	while(iterator != indices->end())
		strm<<(*iterator++).first<<",";
}

void LandscapeObserver::report_stat_observations(int _timestep, ostream& strm) const
{
	indices_map::iterator iterator = indices->begin();
	while(iterator != indices->end())
		strm<<(*iterator++).second<<",";
}

#endif