/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __OBSERVER_H
#define __OBSERVER_H

//number of statisic variables to be calculated after observation
#define STATSVAR 5

//list or layer positions for the statistical parameters
#define MEAN 0
#define MIN 1
#define MAX 2
#define VAR 3
#define STDERR 4

#include <iostream>
#include <map>
#include "dfarr.h"
#include "diarr.h"
#include "dipair.h"
#include "defs.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class ModelRegistry;
typedef map<const char*, float> indices_map;

/**
	Observers record information about the model status during the course of 
	simulation.
*/

class Observer
{
protected:
	char*    		name;			//observer name
	char* 			varname;		//abbreviated variable name
	string			abbrev_name;	//unique abbreviated name
	unsigned short	runs;         	//number of runs for one parameter combination
	unsigned short 	obsteps;		//gap between two following observation steps
	bool			report_raw_obs;	//if true, raw data will be reported
	
	ModelRegistry*	registry;
	static bool		msg;			//flag for message control
	
public:
	Observer() {};
	Observer(ModelRegistry* _registry, char* _name, unsigned short _runs, unsigned short _obsteps);
	~Observer();
	virtual void reset();

	char* getName() const								{ return name; };
	virtual const char* getUniqueAbbreviatedName();

	void setRuns(unsigned short _runs)					{ runs = _runs; };
	unsigned short getRuns() const						{ return runs; };

	void setVarname(char* _varname)						{ varname = _varname; };
	char* getVarname() const							{ return varname; };

	void setObsteps(unsigned short _obsteps)			{ obsteps = _obsteps; };
	unsigned short getObsteps() const					{ return obsteps; };

	void setReport_raw_obs(bool _raw_obs)				{ report_raw_obs = _raw_obs; };
	bool getReport_raw_obs() const						{ return report_raw_obs; };

	bool operator == (const Observer& _observer) const;
	bool operator != (const Observer& _observer) const;
   
	virtual void observe(int _run, int _obstep) {};
	virtual void calc_stats() {};

	virtual void report_run_names(ostream& strm=cout) {};
	virtual void report_variable_names(ostream& strm=cout) const {};
	virtual void report_stat_observations(int _timestep, ostream& strm=cout) const {};
	virtual void report_raw_observations(int _timestep, ostream& strm=cout) {};
};

/**
	Population observers record the abundance of a population during the course of simulation.
*/

class PopulationObserver : public Observer
{
private:
	unsigned short		population_id;					//id of the population
		
	dfloatarray*		raw_observations;				//data container for raw observations	
	dfloatarray*		stat_observations;				//data container for aggregated observations

public:
	PopulationObserver() {};
	PopulationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~PopulationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const PopulationObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

/**
	Extinction observers record the frequency of population extinctions during the course of simulation.
*/

class ExtinctionObserver : public Observer
{
private:
	unsigned short		population_id;					//id of the population
	
	floatlist*			raw_extinctions;				//data container for raw extinction events
	floatlist*			cumulative_extinctions;			//data container for cummulative extinction rate
	floatlist*			relative_extinctions;			//data container for relative extinction rate
	bool				record;							//control flag

public:
	ExtinctionObserver() {};
	ExtinctionObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~ExtinctionObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const ExtinctionObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
};

/**
	LandCover observers record the abundance of a population on a certain landcover type 
	during the course of simulation.
*/

class LandCoverObserver : public Observer
{
private:
	unsigned short		population_id;					//id of the population
	cover_type			cover_id;						//id of the landcover type
	
	dfloatarray*		raw_observations;				//data container for raw observations	
	dfloatarray*		stat_observations;				//data container for aggregated observations	

public:
	LandCoverObserver() {};
	LandCoverObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~LandCoverObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const LandCoverObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

/**
	CellImmigration observers record the frequency of immigrations into habitat cells.
*/

typedef map<unsigned int, dipairlist*> cim_counter;

class CellImmigrationObserver : public Observer
{
private:
	unsigned short		population_id;					//id of the population
	cim_counter*		cim_observations;				//immigration counter per cell
	dfloatarray*		raw_observations;				//data container for raw observations
	dfloatarray*		stat_observations;				//data container for aggregated observations
	unsigned long		type_cover;						//amount of landcover type in the landscape		
	cover_type			cover_id;						//id of the landcover type

public:
	CellImmigrationObserver() {};
	CellImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~CellImmigrationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const CellImmigrationObserver& _observer) const;
	
	void update(unsigned int _individual_id, int _xcel, int _ycel);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_cover_id() const					{ return cover_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
private:
	void reset_cim_counter();
};

/**
	PatchImmigration observers record the frequency of immigrations into habitat patches.
*/

typedef map<unsigned int, float> patch_population;
typedef map<unsigned int, unsigned int> index_map;

class PatchImmigrationObserver : public Observer
{
private:
	patch_population*	patch_pop;				
	index_map*			patch_index_ids;		//map to relate patch ids to indexes in patch_population
	index_map*			patch_sizes;			//map to relate patch ids to patch sizes
	unsigned short		population_id;			//id of the population
	cover_type			cover_id;				//id of the landcover type
	dfloatcube*			pim_cube;				//transition matrix between patches for each run	
	dfloatarray*		pim_matrix;				//average transition matrix across runs
	dfloatarray*		raw_observations;		//data container for raw observations
	dfloatarray*		stat_observations;		//data container for aggregated observations
	unsigned int		img_counter;			//global immigration counter
	unsigned short		current_run;			//control flag
	unsigned int		num_patches;			//number of patches in landscape
	bool				header;					//control flag
	
public:
	PatchImmigrationObserver() {};
	PatchImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~PatchImmigrationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const PatchImmigrationObserver& _observer) const;
	
	void update(unsigned int _frompatch_id, unsigned int _topatch_id);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_cover_id() const					{ return cover_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
private:
	void record_patch_population_size();
	void reset_patch_population_size();
};

typedef struct { float x, y; } position;
typedef map<unsigned int, position> position_map; 

/**
	Distance observers record the dispersal distances of individuals during the course of simulation.
*/

class DistanceObserver : public Observer
{
private:
	unsigned short		population_id;			//id of the population
	position_map*		start_positions;		//map to relate individual ids to start x,y locations
	position_map*		end_positions;			//map to relate individual ids to end x,y locations
	dfloatarray*		raw_observations;		//data container for raw observations
	dfloatarray*		stat_observations;		//data container for aggregated observations
	unsigned int		last_run;				//control flag
	
public:
	DistanceObserver() {};
	DistanceObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~DistanceObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const DistanceObserver& _observer) const;
	
	void update(unsigned int _individual_id, bool _start, float _xpos, float _ypos);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);

private:
	void init_start_positions();
	void update_end_positions();
};

/**
	Crossing observers record the boundary crossing frequency between two different landcover types
	during the course of simulation.
*/

class CrossingObserver : public Observer
{
private:
	unsigned short		population_id;			//id of the population
	unsigned int		crossings;				//counter for boundary crossings	
	cover_type			from_cover;				//id of source landcover type
	cover_type			to_cover;				//id of target landcover type
	
	dfloatarray*		raw_observations;		//data container for raw observations	
	dfloatarray*		stat_observations;		//data container for aggregated observations

public:
	CrossingObserver() {};
	CrossingObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _from, cover_type _to);
	~CrossingObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const CrossingObserver& _observer) const;

	void update();
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_from_cover( ) const				{ return from_cover; };
	cover_type get_to_cover() const					{ return to_cover; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

/**
	SearchTime observers record the number of simulation steps to move from one habitat patch to another
	during the course of simulation.
*/

class SearchTimeObserver : public Observer
{
private:
	unsigned short		population_id;			//id of the population
	unsigned int		search_steps;			//counter for simulation time steps
	unsigned int	    individuals;			//number of individuals
	dfloatarray*		raw_observations;		//data container for raw observations
	dfloatarray*		stat_observations;		//data container for aggregated observations

public:
	SearchTimeObserver() {};
	SearchTimeObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~SearchTimeObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const SearchTimeObserver& _observer) const;

	void update(unsigned int _steps);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

/**
	Landscape observers record landscape indices from the model landscapes.
*/

class LandscapeObserver : public Observer
{
private:
	indices_map*		indices;			//map for landscape indices

public:
	LandscapeObserver() {};
	LandscapeObserver(ModelRegistry* _registry, unsigned short _runs, unsigned short _obsteps);
	~LandscapeObserver();
	void reset() {};

	const char* getUniqueAbbreviatedName();
	bool operator == (const LandscapeObserver& _observer) const;

	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
};

#endif
