#ifndef __OBSERVER_H
#define __OBSERVER_H

//number of statisic variables to be calculated after observation
#define STATSVAR 5

//list or layer positions for the statistical parameters
#define MEAN 0
#define MIN 1
#define MAX 2
#define VAR 3
#define STDERR 4

#include <iostream>
#include <map>
#include "dfarr.h"
#include "diarr.h"
#include "dipair.h"
#include "defs.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class ModelRegistry;

//VIRTUAL BASE CLASS FOR ALL OBSERVERS

class Observer
{
protected:
	char*    		name;			//observer name
	char* 			varname;		//abbreviated variable name
	unsigned short	runs;         	//number of runs for one parameter combination
	unsigned short 	obsteps;		//gap between two following observation steps
	bool			report_raw_obs;	//if true, raw data will be reported
	
	ModelRegistry*	registry;	
	
public:
	Observer() {};
	Observer(ModelRegistry* _registry, char* _name, unsigned short _runs, unsigned short _obsteps);
	~Observer();
	virtual void reset();

	char* getName() const								{ return name; };
	virtual const char* getUniqueAbbreviatedName();

	void setRuns(unsigned short _runs)					{ runs = _runs; };
	unsigned short getRuns() const						{ return runs; };

	void setVarname(char* _varname)						{ varname = _varname; };
	char* getVarname() const							{ return varname; };

	void setObsteps(unsigned short _obsteps)			{ obsteps = _obsteps; };
	unsigned short getObsteps() const					{ return obsteps; };

	void setReport_raw_obs(bool _raw_obs)				{ report_raw_obs = _raw_obs; };
	bool getReport_raw_obs() const						{ return report_raw_obs; };

	virtual bool operator == (const Observer& _observer) const;
	bool operator != (const Observer& _observer) const;
   
	virtual void observe(int _run, int _obstep) {};
	virtual void calc_stats() {};

	virtual void report_run_names(ostream& strm=cout) {};
	virtual void report_variable_names(ostream& strm=cout) const {};
	virtual void report_stat_observations(int _timestep, ostream& strm=cout) const {};
	virtual void report_raw_observations(int _timestep, ostream& strm=cout) {};
};

class PopulationObserver : public Observer
{
private:
	unsigned short		population_id;	
	
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;

public:
	PopulationObserver() {};
	PopulationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~PopulationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const PopulationObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

class ExtinctionObserver : public Observer
{
private:
	unsigned short		population_id;	
	
	floatlist*			raw_extinctions;
	floatlist*			cumulative_extinctions;
	floatlist*			relative_extinctions;
	bool				record;

public:
	ExtinctionObserver() {};
	ExtinctionObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~ExtinctionObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const ExtinctionObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
};

//Abundance on landcover type
class LandCoverObserver : public Observer
{
private:
	unsigned short		population_id;	
	cover_type			cover_id;
	
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;

public:
	LandCoverObserver() {};
	LandCoverObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~LandCoverObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const LandCoverObserver& _observer) const;

	void observe(int _run, int _obstep);
	void calc_stats();

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

//Immigration into habitat cells
typedef map<unsigned int, dipairlist*> cim_counter;

class CellImmigrationObserver : public Observer
{
private:
	unsigned short		population_id;
	cim_counter			cim_observations;		//immigration counter per cell
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;
	unsigned long		type_cover;
	cover_type			cover_id;

public:
	CellImmigrationObserver() {};
	CellImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~CellImmigrationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const CellImmigrationObserver& _observer) const;
	
	void update(unsigned int _individual_id, int _xcel, int _ycel);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_cover_id() const					{ return cover_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
private:
	void reset_cim_counter();
};

//Immigration into habitat patches
typedef map<unsigned int, float> patch_population;
typedef map<unsigned int, unsigned int> index_map;

class PatchImmigrationObserver : public Observer
{
private:
	patch_population	patch_pop;
	index_map			patch_index_ids;		//map to relate patch ids to indexes in patch_population
	index_map			patch_sizes;
	unsigned short		population_id;
	cover_type			cover_id;
	dfloatcube*			pim_cube;				//transition matrix between patches for each run	
	dfloatarray*		pim_matrix;				//average transition matrix across runs
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;
	unsigned int		img_counter;
	unsigned short		current_run;
	unsigned int		num_patches;			//number of patches in landscape
	
public:
	PatchImmigrationObserver() {};
	PatchImmigrationObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _cover_id);
	~PatchImmigrationObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const PatchImmigrationObserver& _observer) const;
	
	void update(unsigned int _frompatch_id, unsigned int _topatch_id);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_cover_id() const					{ return cover_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
private:
	void record_patch_population_size();
	void reset_patch_population_size();
};

typedef struct { float x, y; } position;
typedef map<unsigned int, position> position_map; 

//Movement distances
class DistanceObserver : public Observer
{
private:
	unsigned short		population_id;
	position_map		start_positions;
	position_map		end_positions;
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;
	unsigned int		last_run;
	
public:
	DistanceObserver() {};
	DistanceObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~DistanceObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const DistanceObserver& _observer) const;
	
	void update(unsigned int _individual_id, bool _start, float _xpos, float _ypos);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);

private:
	void init_start_positions();
	void update_end_positions();
};

//Boundary Crossing
class CrossingObserver : public Observer
{
private:
	unsigned short		population_id;
	unsigned int		crossings;
	cover_type			from_cover;
	cover_type			to_cover;
	
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;

public:
	CrossingObserver() {};
	CrossingObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps, cover_type _from, cover_type _to);
	~CrossingObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const CrossingObserver& _observer) const;

	void update();
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };
	cover_type get_from_cover( ) const				{ return from_cover; };
	cover_type get_to_cover() const					{ return to_cover; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

//Search Time
class SearchTimeObserver : public Observer
{
private:
	unsigned short		population_id;
	unsigned int		search_steps;
	unsigned int	    individuals;
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;

public:
	SearchTimeObserver() {};
	SearchTimeObserver(ModelRegistry* _registry, unsigned short _population_id, unsigned short _runs, unsigned short _obsteps);
	~SearchTimeObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const SearchTimeObserver& _observer) const;

	void update(unsigned int _steps);
	void observe(int _run, int _obstep);
	void calc_stats();
	unsigned short get_population_id() const		{ return population_id; };

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

//Landscape Indices
class LandscapeObserver : public Observer
{
private:
	dfloatarray*		raw_observations;	
	dfloatarray*		stat_observations;

public:
	LandscapeObserver() {};
	LandscapeObserver(ModelRegistry* _registry, unsigned short _runs, unsigned short _obsteps);
	~LandscapeObserver();
	void reset();

	const char* getUniqueAbbreviatedName();
	bool operator == (const LandscapeObserver& _observer) const;

	void calc_stats();

	void report_run_names(ostream& strm=cout);
	void report_variable_names(ostream& strm=cout) const;
	void report_stat_observations(int _timestep, ostream& strm=cout) const;
	void report_raw_observations(int _timestep, ostream& strm=cout);
};

#endif
