/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __OUTPUTWRITER_CPP
#define __OUTPUTWRITER_CPP

#include "outputwriter.h"
#include "observer.h"
#include "modelregistry.h"
#include "simulator.h"
#include "parareader.h"
#include "landscape.h"
#include "ibusutil.h"
#include "logging.h"
#include "gif.h"
#include <time.h>
#include <fstream>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

OutputWriter::OutputWriter(ModelRegistry* _registry)
{
	registry = _registry;
	
	char* buffer = new char[50];	
	sprintf(buffer,"%s",strtok((char*)registry->parameters->getFilename(),"."));
	basename = buffer;
	outputfilestream = NULL;
	obsrawfilestream = NULL;
	write_stat_header = true;
	write_raw_header = true;
	registry->register_outputwriter(this);
}

OutputWriter::~OutputWriter()
{
	if(outputfilestream)
	{
		outputfilestream->close();
		delete outputfilestream;
	}
	if(obsrawfilestream)
	{
		obsrawfilestream->close();
		delete obsrawfilestream;
	}
}

void OutputWriter::report_stat_observations() throw (IbusepmException)
{
	openOutputFileStream();

	observer_list* observers = registry->getObservers();
	observer_list::const_iterator observer_iterator = observers->begin();
	
	if(write_stat_header)
	{
		(*outputfilestream)<<"YEAR,STEP,";
		registry->parameters->reportVariationNames(*outputfilestream);
		while(observer_iterator != observers->end())
			(*observer_iterator++)->report_variable_names((*outputfilestream));
		(*outputfilestream)<<"\n";
		write_stat_header = false;
	}
	
	unsigned short years = registry->simulator->getYears();
	unsigned short steps = registry->simulator->getSteps_per_year();
	unsigned short obsgap = registry->simulator->getObs_gap();
	unsigned short obsteps = 0;

	if(obsgap != 0)
		obsteps = ((years * steps) / obsgap) + 1;
	else
		throw IbusepmException("OutputWriter::report_stat_observations() obsgap was 0!");

	for(int obstep = 0; obstep < obsteps; obstep++)
	{
   		observer_iterator = observers->begin();
		(*outputfilestream)<<((((obstep*obsgap)-1)/steps)+1)<<",";
      	(*outputfilestream)<<(obstep*obsgap)<<",";

		registry->parameters->reportCurrentVariations(*outputfilestream);

      	while(observer_iterator != observers->end())
			(*observer_iterator++)->report_stat_observations(obstep, *outputfilestream);

        (*outputfilestream)<<"\n";	
	}
	outputfilestream->close();
}

void OutputWriter::report_raw_observations() throw (IbusepmException)
{
	observer_list* observers = registry->getObservers();
	observer_list::const_iterator observer_iterator = observers->begin();
	
	if(write_raw_header)
	{
		while(observer_iterator != observers->end())
		{
			Observer* tempObserver = *observer_iterator;
			if(tempObserver && tempObserver->getReport_raw_obs())
			{
				openRawFileStream(tempObserver);
				(*obsrawfilestream)<<"YEAR,STEP,";
				registry->parameters->reportVariationNames(*obsrawfilestream);
				tempObserver->report_run_names(*obsrawfilestream);
				(*obsrawfilestream)<<"\n";
				obsrawfilestream->close();
			}
			observer_iterator++;
		}
		write_raw_header = false;
	}
	
	unsigned short years = registry->simulator->getYears();
	unsigned short steps = registry->simulator->getSteps_per_year();
	unsigned short obsgap = registry->simulator->getObs_gap();
	unsigned short obsteps = 0;

	if(obsgap != 0)
		obsteps = ((years * steps) / obsgap) + 1;
	else
		throw IbusepmException("OutputWriter::report_raw_observations() obsgap was 0!");

	observer_iterator = observers->begin();

	while(observer_iterator != observers->end())
	{
		Observer* tempObserver = *observer_iterator;
		if(tempObserver && tempObserver->getReport_raw_obs())
		{
			openRawFileStream(tempObserver);
			for(int obstep = 0; obstep < obsteps; obstep++)
			{	
				(*obsrawfilestream)<<((((obstep*obsgap)-1)/steps)+1)<<",";
				(*obsrawfilestream)<<(obstep*obsgap)<<",";
				registry->parameters->reportCurrentVariations(*obsrawfilestream);
				tempObserver->report_raw_observations(obstep, *obsrawfilestream);
				(*obsrawfilestream)<<"\n";
			}
			obsrawfilestream->close();
		}
		observer_iterator++;
	}
}

void OutputWriter::openRawFileStream(Observer* _observer) throw (IbusepmException)
{
	string* filename = new string(basename);
	filename->append("-");
	filename->append(_observer->getUniqueAbbreviatedName());
	filename->append("-raw.csv");

	if(write_raw_header)
	{
		FILE* dummyfile = fopen(filename->c_str(),"w");
		unsigned short attempts = 0;
		while(!dummyfile)
		{
			if(attempts == 3)
				throw IbusepmException("OutputWriter::openRawFileStream() ", filename->c_str(),
										" could not be opened!");

			Logging::log_error("Cannot open '",filename->c_str(),"'. Please close file and press Enter!",NULL);
			getchar();
			dummyfile = fopen(filename->c_str(),"w");
			attempts++;
		}
		fclose(dummyfile);
		obsrawfilestream = new ofstream(filename->c_str(), ios::app);
	}
	else if(obsrawfilestream && !obsrawfilestream->is_open())
	{
		obsrawfilestream->open(filename->c_str(), ios::app);
	}
	obsrawfilestream->setf(ios::scientific, ios::fixed);
	obsrawfilestream->precision(10);
	delete filename;
}

void OutputWriter::openOutputFileStream() throw (IbusepmException)
{
	string* filename = new string(basename);
	filename->append("_out.csv");

	if(!outputfilestream)
	{	
		FILE* dummyfile = fopen(filename->c_str(),"w");
		unsigned short attempts = 0;
		while(!dummyfile)
		{	
			if(attempts == 3)
				throw IbusepmException("OutputWriter::openOutputFileStream() ", filename->c_str(),
										" could not be opened!");
			
			Logging::log_error("Cannot open '",filename->c_str(),"'. Please close file and press Enter!",NULL);
			getchar();
			dummyfile = fopen(filename->c_str(),"w");
			attempts++;
		}
		fclose(dummyfile);
		outputfilestream = new ofstream(filename->c_str(), ios::app);
	}
	else if(!outputfilestream->is_open())
	{
		outputfilestream->open(filename->c_str(), ios::app);
	}
	outputfilestream->setf(ios::scientific, ios::fixed);
	outputfilestream->precision(10);
	delete filename;
}

void OutputWriter::write_asc_landscape()
{
	string* filename = new string(basename);
	filename->append("_land_w");
	filename->append(Utility::int_to_char(registry->landscape->getXdim()));
	filename->append("_l");
	filename->append(Utility::int_to_char(registry->landscape->getYdim()));
	filename->append("_ha");
	filename->append(Utility::float_to_char(registry->landscape->getLandCoverType(1)->getCoverage(),1));
	filename->append("_hf");
	filename->append(Utility::float_to_char(registry->landscape->getLandCoverType(1)->getFragmentation(),1));
	filename->append(".asc");
	registry->landscape->write_to_ascii(filename->c_str());
	delete filename;
}


void OutputWriter::write_gif_landscape(char* _filename)
{
	Gifsaver gs(registry->landscape);
	if(_filename == NULL)
	{
		string* filename = new string(basename);
		filename->append("_land_w");
		filename->append(Utility::int_to_char(registry->landscape->getXdim()));
		filename->append("_l");
		filename->append(Utility::int_to_char(registry->landscape->getYdim()));
		filename->append("_ha");
		filename->append(Utility::float_to_char(registry->landscape->getLandCoverType(1)->getCoverage(),1));
		filename->append("_hf");
		filename->append(Utility::float_to_char(registry->landscape->getLandCoverType(1)->getFragmentation(),1));
		filename->append(".gif");
		gs.save_landscape((char*)filename->c_str());
		delete filename;
	}
	else
	{
		char* giffilename = new char[50];
		char* filename = new char[50];
		sprintf(filename, "%s", _filename);
		sprintf(giffilename,"%s.gif",strtok(filename,"."));
		Gifsaver gs(registry->landscape);
		gs.save_landscape(giffilename);
		delete giffilename;
		delete filename;
	}
}



#endif