/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PARAMETER_CPP
#define __PARAMETER_CPP

#include "parameter.h"
#include "logging.h"
#include "ibusutil.h"

Parameter::Parameter(const char* _name, const char* _abbrev_name, const char* _type, data_types _data_type, char* _valid_values)
{
	name = _name;
	type = _type;
	data_type = _data_type;
	value_iterator = values.begin();
	min = NULL;
	max = NULL;
	valid_values = _valid_values;
	abbrev_name = _abbrev_name;
	current_rvalue = new char[10];
}

Parameter::~Parameter()
{
	Logging::log_debug("Parameter(",name,")::~Parameter()",NULL);
	values.clear();
	delete current_rvalue;
}

void Parameter::addValue(char* _value)
{
	bool found = false;
	v_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		if(!strcmp(*iterator++,_value))
		{ 
			found=true;
			break;
		}
	}
	if(!found)
		values.push_back(_value);
	value_iterator = values.begin();
}

bool Parameter::nextValue()
{
	if(value_iterator != --values.end())
	{
		 value_iterator++;
		 return true;
	}
	return false;
}

char* Parameter::getCurrentValue() throw (IbusepmException)
{
	if(value_iterator != values.end())
		return *value_iterator;
	else
		throw IbusepmException("Parameter ",name,"::getCurrentValue() value_iterator > values.end()");
}

float Parameter::getCurrentValueAsFloat() throw (IbusepmException)
{
	if(data_type == _FLOAT)
		return atof((const char*)getCurrentValue());
	throw IbusepmException("Parameter::getCurrentValueAsFloat() data_type not _FLOAT!\n");
}

int Parameter::getCurrentValueAsInt() throw (IbusepmException)
{
	if(data_type == _INTEGER)
		return atoi((const char*)getCurrentValue());
	throw IbusepmException("Parameter::getCurrentValueAsInt() data_type not _INTEGER!\n");
}

bool Parameter::validate() throw (IbusepmException)
{
	char* delims={";,:"};
	string message("Parameter::isValid() '");
	if(valid_values && data_type == _STRING)
	{
		v_iterator val_iterator = values.begin();
		while(val_iterator != values.end())
		{
			char buf[200];
			sprintf(buf,"%s",*val_iterator++);
			char* value = strtok(buf,delims);
			while(value)
			{
				string valvalstr(valid_values);
				string valstr(value);
				if(valvalstr.find(valstr) > valvalstr.size())
				{
					message.append(abbrev_name);
					message.append("' has invalid value '");
					message.append(value);
					message.append("'!\n");
					throw IbusepmException(message.c_str());
				}
				value = strtok(NULL,delims);
			}
		}
	}
	else if(valid_values)
	{
		char buf[20];
		sprintf(buf,"%s",valid_values);
		char* min_val = strtok(buf,delims);
		char* max_val = strtok(NULL,delims);

		if(atof(getMinValue()) < atof(min_val))
		{
			message.append(abbrev_name);
			message.append("' minimum value smaller than lowest possible value of '");
			message.append(min_val);
			message.append("'!\n");
			throw IbusepmException(message.c_str());
		}
		if(atof(getMaxValue()) > atof(max_val))
		{
			message.append(abbrev_name);
			message.append("' maximum value exceeds highest possible value of '");
			message.append(max_val);
			message.append("'!\n");
			throw IbusepmException(message.c_str());
		}
	}
	return true;
}

char* Parameter::getMinValue()
{	
	if(min)
		return min;

	char* minchar = "zzzzzz";
	char* minnumber = "2147483647";

	v_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		char* currentValue = *iterator;
		if(data_type == _STRING)
		{
			if(**iterator < *minchar)
				minchar = *iterator;
		}
		else
		{
			if(atof(currentValue) < atof(minnumber))
				minnumber = currentValue;
		}
		iterator++;
	}
	if(data_type == _STRING)
		return min = minchar;
	return min = minnumber;
}

char* Parameter::getMaxValue()
{
	if(max)
		return max;

	char* maxval = "-99999";

	v_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		char* currentValue = *iterator;
		if(data_type == _STRING)
		{
			if(**iterator > *maxval)
				maxval = *iterator;
		}
		else
		{
			if(atof(currentValue) > atof(maxval))
				maxval = currentValue;
		}
		iterator++;
	}
	return max = maxval;
}

char* Parameter::getRandomValue()
{
	switch(data_type)
	{
		case _INTEGER:	sprintf(current_rvalue,"%s",(char*)Utility::int_to_char(Utility::randint(atoi(getMinValue()),atoi(getMaxValue())))); break;
		case _FLOAT:	sprintf(current_rvalue,"%s",(char*)Utility::float_to_char(Utility::randfloat(atof(getMinValue()),atof(getMaxValue())))); break;
		case _STRING:
		{
			int i = 0;
			int index = Utility::randint(values.size());
			list<char*>::const_iterator iterator = values.begin();
			while(iterator != values.end() && i++ < index)
				iterator++;
			sprintf(current_rvalue,"%s",*iterator);
			break;
		}
		default: sprintf(current_rvalue,"%s","invalid");
	}
	return current_rvalue;
}

int Parameter::getRandomValueAsInt()
{
	if(data_type == _INTEGER)
		return atoi((const char*)getRandomValue());
	throw IbusepmException("Parameter::getRandomValueAsInt() data_type not _INTEGER!\n");
}

float Parameter::getRandomValueAsFloat()
{
	if(data_type == _FLOAT)
		return atof((const char*)getRandomValue());
	throw IbusepmException("Parameter::getRandomValueAsFloat() data_type not _FLOAT!\n");
}

void Parameter::reset()
{
	value_iterator = values.begin();
}

int Parameter::getNumberOfValues() const
{
	return values.size();
}

const char* Parameter::getUniqueAbbreviatedName()
{
	return abbrev_name;
}

void Parameter::report(ostream& strm) const
{
	strm<<type<<"."<<name;
	switch(data_type)
	{
		case _FLOAT:	strm<<"(float)"; break;
		case _INTEGER:	strm<<"(int)"; break;
		case _STRING:	strm<<"(string)"; break;
		default:		;
	}
	strm<<"[";
	list<char*>::const_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		strm<<*iterator++;
		if(iterator != values.end())
			strm<<",";
	}
	strm<<"]\n";
}

bool Parameter::operator==(const Parameter& _parameter) const
{
	return values.size() == _parameter.values.size();
}

bool Parameter::operator!=(const Parameter& _parameter) const
{
	return !((*this) == _parameter);
}

bool Parameter::operator<(const Parameter& _parameter) const
{
	return values.size() < _parameter.values.size();
}

bool Parameter::operator>(const Parameter& _parameter) const
{
	return values.size() > _parameter.values.size();
}


LandCoverTypeParameter::LandCoverTypeParameter(const char* _name, const char* _abbrev_name, data_types _data_type, cover_type _cover_id, char* _valid_values)
											   : Parameter(_name, _abbrev_name, "landcover", _data_type, _valid_values)
{
	cover_id = _cover_id;
	sequence = NULL_INT;
}

LandCoverTypeParameter::~LandCoverTypeParameter()
{
	Parameter::~Parameter();
}

void LandCoverTypeParameter::report(ostream& strm) const
{
	strm<<type<<"(id="<<(int)cover_id;
	if(sequence != NULL_INT)
		strm<<" seq="<<sequence;
	strm<<")."<<name;

	switch(data_type)
	{
		case _FLOAT:	strm<<"(float)"; break;
		case _INTEGER:	strm<<"(int)"; break;
		case _STRING:	strm<<"(string)"; break;
		default:		;
	}
	strm<<"[";
	list<char*>::const_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		strm<<*iterator++;
		if(iterator != values.end())
			strm<<",";
	}
	strm<<"]\n";
}

gifcolors LandCoverTypeParameter::getGif_color() const
{
	if(!strcmp(gif_color,"white") || !strcmp(gif_color,"WHITE"))
		return WHITE;
	if(!strcmp(gif_color,"light_green") || !strcmp(gif_color,"LIGHT_GREEN"))
		return LIGHTGREEN;
	if(!strcmp(gif_color,"dark_green") || !strcmp(gif_color,"DARK_GREEN"))
		return DARKGREEN;
	if(!strcmp(gif_color,"yellow") || !strcmp(gif_color,"YELLOW"))
		return YELLOW;
	if(!strcmp(gif_color,"grey") || !strcmp(gif_color,"GREY"))
		return GREY;
	if(!strcmp(gif_color,"blue") || !strcmp(gif_color,"BLUE"))
		return BLUE;
	if(!strcmp(gif_color,"red") || !strcmp(gif_color,"RED"))
		return RED;
	if(!strcmp(gif_color,"black") || !strcmp(gif_color,"BLACK"))
		return BLACK;
	return RED;
}

PopulationParameter::PopulationParameter(const char* _name, const char* _abbrev_name, data_types _data_type, unsigned short _population_id, char* _valid_values)
											: Parameter(_name, _abbrev_name, "population", _data_type, _valid_values)
{
	population_id = _population_id;
}

PopulationParameter::~PopulationParameter()
{
	Parameter::~Parameter();
}

void PopulationParameter::report(ostream& strm) const
{
	strm<<type<<"(id="<<population_id<<")."<<name;
	switch(data_type)
	{
		case _FLOAT:	strm<<"(float)"; break;
		case _INTEGER:	strm<<"(int)"; break;
		case _STRING:	strm<<"(string)"; break;
		default:		;
	}
	strm<<"[";
	list<char*>::const_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		strm<<*iterator++;
		if(iterator != values.end())
			strm<<",";
	}
	strm<<"]\n";
}

BehaviourParameter::BehaviourParameter(const char* _name, const char* _abbrev_name, data_types _data_type, unsigned int _population_id,
										cover_type _cover_id, age_type _age, gender_types _gender, char* _valid_values)
										: Parameter(_name, _abbrev_name, "behaviour", _data_type, _valid_values)
{
	population_id = _population_id;
	cover_id = _cover_id;
	age = _age;
	gender = _gender;
	cross_cover_id = NULL_TYPE;
}

BehaviourParameter::~BehaviourParameter()
{
	Parameter::~Parameter();
}

void BehaviourParameter::report(ostream& strm) const
{
	strm<<type<<"(pop_id="<<population_id;
	
	if(cross_cover_id != NULL_TYPE)
		strm<<" from="<<(int)cover_id<<" to="<<(int)cross_cover_id;
	else
		strm<<" cov_id="<<(int)cover_id;
	strm<<" age="<<(int)age;
	strm<<" gender="<<((gender == MALE) ? "male" : "female")<<")."<<name;
	switch(data_type)
	{
		case _FLOAT:	strm<<"(float)"; break;
		case _INTEGER:	strm<<"(int)"; break;
		case _STRING:	strm<<"(string)"; break;
		default:		;
	}
	int counter = 0;
	strm<<"[";
	list<char*>::const_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		counter++;
		strm<<*iterator++;
		if(iterator != values.end())
			strm<<",";
	}
	if(counter == 0)
		strm<<"default";
	strm<<"]\n";
}

ObserverParameter::ObserverParameter(const char* _name, data_types _data_type, unsigned short _population_id, char* _valid_values)
									: Parameter(_name, "OBS_PARAM", "observer", _data_type, _valid_values)
{		
	population_id = _population_id;
	cover_id = NULL_TYPE;	
	age = NULL_TYPE;
	gender = NOTHING;
	report_details = false;
}

ObserverParameter::~ObserverParameter()
{
	Parameter::~Parameter();
}

void ObserverParameter::report(ostream& strm) const
{
	strm<<type<<"(";
	
	if(population_id != NULL_SHORT)
		strm<<"pop_id="<<population_id;
	if(cover_id != NULL_TYPE)
		strm<<" cov_id="<<(int)cover_id;
	if(age != NULL_TYPE)
		strm<<" age="<<(int)age;
	if(gender != NOTHING)
		strm<<" gender="<<((gender == MALE) ? "male" : "female");
		
	strm<<")."<<name;
	
	switch(data_type)
	{
		case _FLOAT:	strm<<"(float)"; break;
		case _INTEGER:	strm<<"(int)"; break;
		case _STRING:	strm<<"(string)"; break;
		default:		;
	}
	int counter = 0;
	strm<<"[";
	list<char*>::const_iterator iterator = values.begin();
	while(iterator != values.end())
	{
		counter++;
		strm<<*iterator++;
		if(iterator != values.end())
			strm<<",";
	}
	if(counter == 0)
		strm<<"default";
	strm<<"]\n";
}

const char* Parameter::name_population =						"population";
const char* Parameter::name_landscape =							"landscape";
const char* Parameter::name_simulation =						"simulation";
const char* Parameter::name_observer =							"observer";
const char* Parameter::name_landscape_file =					"grid_file";
const char* Parameter::name_landscape_width =					"width";
const char* Parameter::name_landscape_length =					"length"; 
const char* Parameter::name_landscape_border =					"border"; 
const char* Parameter::name_landscape_cell_adjacency =			"cell_adjacency";
const char* Parameter::name_landscape_landcover =				"landcover";
const char* Parameter::name_simulation_years =					"years";
const char* Parameter::name_simulation_steps =					"steps_per_year";
const char* Parameter::name_simulation_runs =					"runs";
const char* Parameter::name_simulation_combination =			"parameter_combination";
const char* Parameter::name_simulation_gap =					"observation_gap";

const char* PopulationParameter::name_initial_size =			"initial_size";
const char* PopulationParameter::name_overlapping_generations =	"overlapping_generations";
const char* PopulationParameter::name_sex_ratio =				"sex_ratio";
const char* PopulationParameter::name_distribution =			"distribution";
const char* PopulationParameter::name_border_reflection_angle =	"border_reflection_angle";
const char* PopulationParameter::name_age_classes =				"age_classes";
const char* PopulationParameter::name_behaviour =				"behaviour";
const char* PopulationParameter::name_activity_sequence =		"activity_sequence";
const char* PopulationParameter::name_repro_step =				"repro_step";	

const char* BehaviourParameter::name_landcover =				"landcover";
const char* BehaviourParameter::name_gender =					"gender";
const char* BehaviourParameter::name_age =						"age";
const char* BehaviourParameter::name_step_angle	=				"step_angle";
const char* BehaviourParameter::name_step_length =				"step_length";
const char* BehaviourParameter::name_offspring =				"offspring";
const char* BehaviourParameter::name_mortality =				"mortality";
const char* BehaviourParameter::name_boundary_crossing=			"boundary_crossing";
const char* BehaviourParameter::name_resource_factor=			"resource_factor";
const char* BehaviourParameter::name_parameter =				"parameter";
const char* BehaviourParameter::name_dependency =				"dependency";

const char* LandCoverTypeParameter::name_cell_capacity =		"cell_capacity";
const char* LandCoverTypeParameter::name_cell_reduction =		"cell_reduction";
const char* LandCoverTypeParameter::name_coxpattern =			"coxpattern";
const char* LandCoverTypeParameter::name_cover =				"cover";
const char* LandCoverTypeParameter::name_fragmentation =		"fragmentation";

const char* ObserverParameter::name_pop_size =					"pop_size";
const char* ObserverParameter::name_extinction =				"extinction";
const char* ObserverParameter::name_landcover =					"landcover";
const char* ObserverParameter::name_emigration =				"emigration";
const char* ObserverParameter::name_immigration =				"immigration";
const char* ObserverParameter::name_search_time =				"search_time";
const char* ObserverParameter::name_patch_immigration =			"patch_immigration";
const char* ObserverParameter::name_cell_immigration =			"cell_immigration";
const char* ObserverParameter::name_size =						"size";
const char* ObserverParameter::name_time_to_extinction =		"time_to_extinction";
const char* ObserverParameter::name_movement_distance =			"movement_distance";
const char* ObserverParameter::name_boundary_crossing =			"boundary_crossing";
const char* ObserverParameter::name_landscape =					"landscape";

#endif 