/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PARAMETER_H
#define __PARAMETER_H

#include <list>
#include <iostream>
#include "defs.h"
#include "ibusepmexception.h"
#include "behaviour.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

typedef list<char*>::iterator v_iterator;

/**
	A Parameter object represents a model parameter, i.e.
	parameter name, data type, valid range and actual values. 
	Each Parameter object maintains an iterator on the value
	list, which points to the current value. Parameters
	are the basis for the factorial combination algorithm
	in the ParameterReader object.
*/

class Parameter
{
protected:
	const char* type;			//parameter type, e.g. landscape
	const char* name;			//name of the parameter 
	const char* abbrev_name;    //abbreviation for name
	data_types data_type;		//data_type of parameter
	list<char*> values;			//list of parameter values
	char* min;					//minimum value
	char* max;					//maximum value
	char* valid_values;			//string of valid values
	char* current_rvalue;		//current random value
	v_iterator value_iterator;	//iterator on values

public:
	static const char* name_population;
	static const char* name_landscape;
	static const char* name_simulation;
	static const char* name_observer;
	static const char* name_landscape_width;
	static const char* name_landscape_length;
	static const char* name_landscape_file;
	static const char* name_landscape_border; 
	static const char* name_landscape_cell_adjacency; 
	static const char* name_landscape_landcover; 
	static const char* name_simulation_years;
	static const char* name_simulation_steps;
	static const char* name_simulation_runs;
	static const char* name_simulation_combination;
	static const char* name_simulation_gap;

	Parameter() {};
	Parameter(const char* _name, const char* _abbrev_name, const char* _type, data_types _data_type, char* _valid_values = NULL);
	~Parameter();

	bool operator==(const Parameter& _parameter) const;
	bool operator!=(const Parameter& _parameter) const;
	bool operator<(const Parameter& _parameter) const;
	bool operator>(const Parameter& _parameter) const;

	const char* getName() const				{ return name; };
	const char* getType() const				{ return type; };
	data_types getData_type() const			{ return data_type; };
	
	const char* getUniqueAbbreviatedName();

	void addValue(char* _value);						
	char* getCurrentValue() throw (IbusepmException);
	int getCurrentValueAsInt() throw (IbusepmException);
	float getCurrentValueAsFloat() throw (IbusepmException);
	
	char* getCurrentRandomValue() const		{ return current_rvalue; };
	char* getRandomValue();
	int getRandomValueAsInt();
	float getRandomValueAsFloat();

	char* getMinValue();
	char* getMaxValue();
	bool nextValue();

	bool validate() throw (IbusepmException);
	void reset();								//put the iterator to the first value
	int getNumberOfValues() const;
	virtual void report(ostream& strm=cout) const;
};

/**
	A LandCoverTypeParameter object is a Parameter with additional information
	related to a LandCoverType of a Landscape.
*/

class LandCoverTypeParameter : public Parameter
{
private:
	cover_type		cover_id;					//landcover type (0..255)
	int				sequence;					//order of assignment (only when generated)
	const char*     cover_name;						
	const char*     gif_color;	

public:
	static const char* name_cell_capacity;
	static const char* name_cell_reduction;
	static const char* name_coxpattern;
	static const char* name_cover;
	static const char* name_fragmentation;

	LandCoverTypeParameter(const char* _name, const char* _abbrev_name, data_types _data_type, cover_type _cover_id, char* _valid_values = NULL);
	~LandCoverTypeParameter();

	cover_type getCover_id() const				{ return cover_id; }

	const char* getCover_name() const			{ return cover_name; };
	void setCover_name(const char* _cover_name)	{ cover_name = _cover_name; };

	gifcolors getGif_color() const;
	void setGif_color(const char* _color)		{ gif_color = _color; };

	int getSequence() const						{ return sequence; };
	void setSequence(int _sequence)				{ sequence = _sequence; };

	void report(ostream& strm=cout) const;
};

/**
	A PopulationParameter object is a Parameter with additional information
	related to a Population of the model.
*/

class PopulationParameter : public Parameter
{
private:
	unsigned short	population_id;	
	
public:
	static const char* name_initial_size;
	static const char* name_overlapping_generations;
	static const char* name_sex_ratio;
	static const char* name_distribution;
	static const char* name_border_reflection_angle;
	static const char* name_age_classes;
	static const char* name_behaviour;
	static const char* name_activity_sequence;
	static const char* name_repro_step;

	PopulationParameter(const char* _name, const char* _abbrev_name, data_types _data_type, unsigned short _population_id, char* _valid_values = NULL);
	~PopulationParameter();
	
	unsigned short getPopulation_id() const				{ return population_id; }
	void report(ostream& strm=cout) const;
};

/**
	A BehaviourParameter object is a Parameter with additional information
	related to a Behaviour object of the model.
*/

class BehaviourParameter : public Parameter
{
private:
	unsigned short	population_id;				//population id	
	cover_type		cover_id;					//landcovertype id
	age_type		age;						//age of the individual
	gender_types	gender;						//gender of the individual
	cover_type		cross_cover_id;				//needed for boundary crossing parameter

public:
	static const char* name_landcover;
	static const char* name_gender;
	static const char* name_age;
	static const char* name_step_angle;
	static const char* name_step_length;
	static const char* name_offspring;
	static const char* name_mortality;
	static const char* name_boundary_crossing;
	static const char* name_resource_factor;
	static const char* name_parameter;
	static const char* name_dependency;

	BehaviourParameter(const char* _name, const char* _abbrev_name, data_types _data_type, unsigned int _population_id,
					   cover_type _cover_id, age_type _age, gender_types _gender, char* _valid_values = NULL);
	~BehaviourParameter();
	
	unsigned short getPopulation_id() const		{ return population_id; };
	cover_type getCover_id() const				{ return cover_id; };
	age_type getAge() const						{ return age; };
	gender_types getGender() const				{ return gender; };

	void setCross_cover_id(cover_type _cross_cover_id)	{ cross_cover_id = _cross_cover_id; };
	cover_type getCross_cover_id() const				{ return cross_cover_id; };

	void report(ostream& strm=cout) const;
};

/**
	An ObserverParameter object is a Parameter with additional information
	related to a Observer objects of the model.
*/

class ObserverParameter : public Parameter
{
private:
	unsigned short	population_id;				//population id	
	cover_type		cover_id;					//landcovertype id
	cover_type		cross_cover_id;
	age_type		age;						//age of the individual
	gender_types	gender;						//gender of the individual
	bool			report_details;				//if true, details will be reported

public:
	static const char* name_pop_size;
	static const char* name_extinction;
	static const char* name_landcover;
	static const char* name_emigration;
	static const char* name_immigration;
	static const char* name_search_time;
	static const char* name_patch_immigration;
	static const char* name_cell_immigration;
	static const char* name_size;
	static const char* name_time_to_extinction;
	static const char* name_movement_distance;
	static const char* name_boundary_crossing;
	static const char* name_landscape;

	ObserverParameter(const char* _name, data_types _data_type, unsigned short _population_id, char* _valid_values = NULL);
	~ObserverParameter();
		
	void setPopulation_id(unsigned short _id)	{ population_id = _id; };
	unsigned short getPopulation_id() const		{ return population_id; };

	void setCover_type(cover_type _cover_id)	{ cover_id = _cover_id; };
	cover_type getCover_id() const				{ return cover_id; };

	void setCross_Cover_type(cover_type _cover_id)	{ cross_cover_id = _cover_id; };
	cover_type getCross_Cover_type() const			{ return cross_cover_id; };
	
	void setAge(age_type _age)					{ age = _age; };
	age_type getAge() const						{ return age; };
	
	void setGender(gender_types _gender)		{ gender = _gender; };
	gender_types getGender() const				{ return gender; };
	
	void setReport_details(bool _details)		{ report_details = _details; };
	bool getReport_details() const				{ return report_details; };

	void report(ostream& strm=cout) const;
};
 
#endif