/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PARAMETER_READER_CPP
#define __PARAMETER_READER_CPP

#include "landscape.h"
#include "parareader.h"
#include "parameter.h"
#include "logging.h"
#include "modelregistry.h"
#include "ibusutil.h"
#include "simulator.h"
#include "outputwriter.h"
#include <algorithm>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

char* ParameterReader::buffer = new char[30];

ParameterReader::ParameterReader(ModelRegistry* _registry, const char* _filename)
{
	Logging::log_debug("ParameterReader::ParameterReader(",_filename,")",NULL);
	filename = _filename;
	registry = _registry;
	registry->register_parameters(this);
	parameter_combinations = 1;
	combination_type = P_FACTORIAL;
	current_parameter = NULL;
	current_combination = 0;
	start_seed = 0;
	save_details = false;
	save_gif_landscapes = false;
	save_asc_landscapes = false;
	initParameterCheckList();
}

ParameterReader::~ParameterReader()
{
	Logging::log_debug("ParameterReader(",filename,")::~ParameterReader()",NULL);
}

unsigned int ParameterReader::getStartSeed()
{
	return start_seed == 0 ? (unsigned int)time(NULL) : start_seed; 
}

void ParameterReader::initModel()
{
	init();
	initLandscape();
	initPopulations();
	initSimulator();
	initObservers();
}

void ParameterReader::createPopulations()
{
	if(!registry->getPopulations().empty())
	{
		population_map& populations = registry->getPopulations();
		population_map::iterator population_iterator = populations.begin();
		while(population_iterator != populations.end())
		{
			Population* tempPopulation = (*population_iterator++).second;
			tempPopulation->reset();
		}
		return;
	}

	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		if(!strcmp(tempParameter->getType(), Parameter::name_population))
		{
			PopulationParameter* tempPopulParam = (PopulationParameter*)tempParameter;
			if(!registry->getPopulation(tempPopulParam->getPopulation_id()))
				registry->register_population(new Population(tempPopulParam->getPopulation_id(), registry));
		}
	}
}

void ParameterReader::init()
{
	Logging::log_debug("ParameterReader::init() ...",NULL);
	
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();

	while(parameter_iterator != model_parameters.end())
	{
		Parameter* parameter = *parameter_iterator++;
		if(!strcmp(parameter->getType(),Parameter::name_simulation))
		{
			if(!strcmp(parameter->getName(),Parameter::name_simulation_combination))
			{	
				char* value = parameter->getCurrentValue();
								
				if(!strcmp(value,"factorial") || !strcmp(value,"FACTORIAL"))
					combination_type = P_FACTORIAL;
				else if(!strcmp(value,"random") || !strcmp(value,"RANDOM"))
					combination_type = P_RANDOM;
				else
					Logging::log_error("ParameterReader::init() cannot regognize parameter ",
									    Parameter::name_simulation_combination, ".", NULL);
				break;
			}
		}
	}

	Logging::log_debug("ParameterReader::init() DONE",NULL);
}

void ParameterReader::initPopulations()
{
	createPopulations();

	population_map& populations = registry->getPopulations();
	population_map::iterator population_iterator = populations.begin();
	parameter_list::const_iterator parameter_iterator;

	while(population_iterator != populations.end())
	{
		Population* tempPopulation = (*population_iterator++).second;
		unsigned short population_id = tempPopulation->getId();
		parameter_iterator = model_parameters.begin();

		while(parameter_iterator != model_parameters.end())
		{
			Parameter* tempParameter = *parameter_iterator++;
			if(!strcmp(tempParameter->getType(),Parameter::name_population))
			{
				PopulationParameter* tempPopulParam = (PopulationParameter*)tempParameter;
				if(tempPopulParam->getPopulation_id() == population_id)
				{
					initPopulation(tempPopulation, tempPopulParam);
				}
			}
			else if(!strcmp(tempParameter->getType(),PopulationParameter::name_behaviour))
			{
				BehaviourParameter* tempBehavParam = (BehaviourParameter*)tempParameter;
				if(tempBehavParam->getPopulation_id() == population_id)
				{
					initBehaviour(tempPopulation, tempBehavParam);
				}
			}
		}
		tempPopulation->initialize_individuals();
	}
}

void ParameterReader::initPopulation(Population* _population, PopulationParameter* _parameter)
{
	if(!strcmp(_parameter->getName(),PopulationParameter::name_initial_size))
	{
		_population->setInitial_size(combination_type == P_FACTORIAL ? _parameter->getCurrentValueAsInt() :
												 _parameter->getRandomValueAsInt());
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_overlapping_generations))
	{
		char* value = combination_type == P_FACTORIAL ? _parameter->getCurrentValue() : _parameter->getRandomValue();
		_population->set_Overlap(!strcmp(value,"yes") ? true : false);
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_sex_ratio))
	{
		_population->setSex_ratio(combination_type == P_FACTORIAL ? _parameter->getCurrentValueAsFloat() :
											  _parameter->getRandomValueAsFloat());
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_repro_step))
	{
		_population->setReproStep((unsigned short)combination_type == P_FACTORIAL ? _parameter->getCurrentValueAsInt() :
											  _parameter->getRandomValueAsInt());
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_border_reflection_angle))
	{
		_population->setRefl_angle(combination_type == P_FACTORIAL ? _parameter->getCurrentValueAsFloat() :
											   _parameter->getRandomValueAsFloat());
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_activity_sequence))
	{
		char* value = combination_type == P_FACTORIAL ? _parameter->getCurrentValue() : _parameter->getRandomValue();
		char value_copy[100];
		sprintf(value_copy,"%s",value);
		char* sequence_name = strtok(value_copy,";");

		sequence_types sequence = Utility::sequence_for_name(sequence_name);
		
		if(sequence == CUSTOM)
		{
			char* activity_name = strtok(NULL,";");
			while(activity_name)
			{	
				_population->add_activity(Utility::activity_for_name(activity_name));
				activity_name = strtok(NULL,";");
			}
		}
		else
		{
			_population->set_Sequence_type(sequence);
			_population->init_activities();
		}
	}
	else if(!strcmp(_parameter->getName(),PopulationParameter::name_distribution))
	{
		char* value = combination_type == P_FACTORIAL ? _parameter->getCurrentValue() : _parameter->getRandomValue();

		char value_copy[100];
		sprintf(value_copy,"%s",value);
		char* distribution_name = strtok(value_copy,";");

		distribution_types distribution = Utility::individual_distribution_for_name(distribution_name);
		
		if(distribution == LANDSCAPE)
		{
			_population->set_Init_distribution(LANDSCAPE);
		}
		else if(distribution == HABITAT)
		{
			_population->set_Init_distribution(COVER_TYPE);
			_population->set_Init_cover_id(LandCoverType::habitat);
		}
		else if(distribution == MATRIX)
		{
			_population->set_Init_distribution(COVER_TYPE);
			_population->set_Init_cover_id(LandCoverType::matrix);
		}
		else if(distribution == COVER_TYPE)
		{
			char* landcover_id = strtok(NULL,";");
			_population->set_Init_distribution(COVER_TYPE);
			_population->set_Init_cover_id((cover_type)atoi(landcover_id));
		}
		else
		{
			if(distribution == CELL)
				_population->set_Init_distribution(CELL);
			else if(distribution == PATCH)
				_population->set_Init_distribution(PATCH);

			char* x = strtok(NULL,";");
			char* y = strtok(NULL,";");
			while(x && y)
			{	
				_population->add_cellCoordinate(atoi(x), atoi(y));
				x = strtok(NULL,";");
				y = strtok(NULL,";");
			}
		}
	}
	/*else if(!strcmp(_parameter->getName(),PopulationParameter::name_age_classes))
	{
		_population->se(factorial ? parameter->getCurrentValueAsFloat() :
								    parameter->getRandomValueAsFloat());
	}*/
}

void ParameterReader::initBehaviour(Population* _population, BehaviourParameter* _parameter)
{
	Behaviour param_behaviour(_parameter->getCover_id(), _parameter->getAge(), _parameter->getGender());

	if(!_population->find_behaviour(param_behaviour))
		_population->add_behaviour(param_behaviour);

	behaviour_list* behaviours = _population->get_behaviours();
	behaviour_list::iterator behaviour_iterator = behaviours->begin();

	while(behaviour_iterator != behaviours->end())
	{
		Behaviour* tempBehav = behaviour_iterator++;

		if(tempBehav->getAge() == _parameter->getAge() &&
		   tempBehav->getCover_id() == _parameter->getCover_id() &&
		   tempBehav->getGender() == _parameter->getGender())
		{
			try
			{
				char* value = combination_type == P_FACTORIAL ? _parameter->getCurrentValue() : _parameter->getRandomValue();

				if(!strcmp(_parameter->getName(),BehaviourParameter::name_step_angle))
				{
					rdist* step_angle_distr = Utility::distribution_for_name(value);
					if(step_angle_distr)
						tempBehav->setStep_angle(step_angle_distr, getStartSeed());
					else
						Logging::log_error("ParameterReader::initBehaviour() ", value, " not valid!",NULL);
				}
				else if(!strcmp(_parameter->getName(),BehaviourParameter::name_step_length))
				{
					rdist* step_length_distr = Utility::distribution_for_name(value);
					if(step_length_distr)
						tempBehav->setStep_length(step_length_distr, getStartSeed());
					else
						Logging::log_error("ParameterReader::initBehaviour() ", value, " not valid!",NULL);
				}
				else if(!strcmp(_parameter->getName(),BehaviourParameter::name_offspring))
				{
					poisson* offspring_distr = Utility::poisson_for_name(value);
					if(offspring_distr)
						tempBehav->setOffspring(offspring_distr, getStartSeed());
					else
						Logging::log_error("ParameterReader::initBehaviour() ", value, " not valid!",NULL);
				}
				else if(!strcmp(_parameter->getName(),BehaviourParameter::name_mortality))
				{
					tempBehav->setSurvival(atof(value));
				}
				else if(!strcmp(_parameter->getName(),BehaviourParameter::name_resource_factor))
				{
					tempBehav->setResource_factor(atof(value));
				}
				else if(!strcmp(_parameter->getName(),BehaviourParameter::name_boundary_crossing))
				{
					tempBehav->setCrossingProbability(_parameter->getCross_cover_id(), atof(value));
				}
				behaviour_iterator = behaviours->end();
			}
			catch (IbusepmException &e)
			{	
				Logging::log_error(e.what(),NULL);
				Logging::log_error("Initialize parameter '", _parameter->getName(), "' default.",NULL);
			}
		}
	} //while
}

void ParameterReader::initLandscape()
{
	Logging::log_debug("ParameterReader::initLandscape() ...",NULL);

	Landscape* landscape = registry->landscape;

	if(!landscape)
		landscape = new Landscape(registry);
	else
		landscape->reset();
	
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* parameter = *parameter_iterator++;
		if(!strcmp(parameter->getType(),Parameter::name_landscape))
		{
			if(!strcmp(parameter->getName(),Parameter::name_landscape_file))
			{
				landscape->setFileName(combination_type == P_FACTORIAL ? parameter->getCurrentValue() :
											   parameter->getRandomValue());
			}
			else if(!strcmp(parameter->getName(),Parameter::name_landscape_width))
			{
				landscape->setXdim(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
											   parameter->getRandomValueAsInt());
			}
			
			else if(!strcmp(parameter->getName(),Parameter::name_landscape_length))
			{
				landscape->setYdim(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
											   parameter->getRandomValueAsInt());
			}
			else if(!strcmp(parameter->getName(),Parameter::name_landscape_border))
			{
				char* value = combination_type == P_FACTORIAL ? parameter->getCurrentValue() : parameter->getRandomValue();
								
				if(!strcmp(value,"absorb") || !strcmp(value,"ABSORB"))
					landscape->setBorder_type(ABSORB);
				else if(!strcmp(value,"reflect") || !strcmp(value,"REFLECT"))
					landscape->setBorder_type(REFLECT);
				else if(!strcmp(value,"torus") || !strcmp(value,"TORUS"))
					landscape->setBorder_type(TORUS);
				else
					Logging::log_error("ParameterReader::initLandscape() cannot regonize",
									   " parameter ", value, "!", NULL);
			}
			else if(!strcmp(parameter->getName(),Parameter::name_landscape_cell_adjacency))
			{
				char* value = combination_type == P_FACTORIAL ? parameter->getCurrentValue() : parameter->getRandomValue();
				
				if(!strcmp(value,"diagonal") || !strcmp(value,"DIAGONAL"))
					landscape->setDiagonal(true);
				else if(!strcmp(value,"orthogonal") || !strcmp(value,"ORTHOGONAL"))
					landscape->setDiagonal(false);
				else
					Logging::log_error("ParameterReader::initLandscape() cannot regonize",
									   " parameter ", value, "!", NULL);
			}
		}
		else if(!strcmp(parameter->getType(),Parameter::name_landscape_landcover))
		{
			initLandcoverType(landscape, (LandCoverTypeParameter*)parameter);
		}
	}
	if(landscape->getFileName() != NULL)
		landscape->read_from_ascii((const char*)landscape->getFileName());
	else
		landscape->create_all_cox_pattern();
	landscape->create_patch_topology();

	if(save_asc_landscapes && landscape->getFileName() == NULL)
		registry->writer->write_asc_landscape();
	if(save_gif_landscapes)
		registry->writer->write_gif_landscape(landscape->getFileName());

	Logging::log_debug("ParameterReader::initLandscape() DONE",NULL);
}

void ParameterReader::initLandcoverType(Landscape* _landscape, LandCoverTypeParameter* _parameter)
{
	cover_type_map* cover_types = _landscape->getLandCoverTypMap();

	if(cover_types->find(_parameter->getCover_id()) == cover_types->end())
	{
		LandCoverType* tempCover = new LandCoverType();
		tempCover->setCover_id(_parameter->getCover_id());
		tempCover->setCover_name((char*)_parameter->getCover_name());
		tempCover->setColor(_parameter->getGif_color());
		(*cover_types)[_parameter->getCover_id()] = tempCover;
	}
				
	cover_type_map::iterator covertype_iterator = cover_types->begin();

	while(covertype_iterator != cover_types->end())
	{
		pair<cover_type, LandCoverType*> map_element = *covertype_iterator++;
		LandCoverType* tmpLandCoverType = map_element.second;

		if(tmpLandCoverType->getCover_id() == _parameter->getCover_id())
		{
			char* value = combination_type == P_FACTORIAL ? _parameter->getCurrentValue() : _parameter->getRandomValue();

			if(!strcmp(_parameter->getName(),LandCoverTypeParameter::name_cell_capacity))
			{
				tmpLandCoverType->setCell_capacity(atof(value));
			}
			else if(!strcmp(_parameter->getName(),LandCoverTypeParameter::name_cell_reduction))
			{
				tmpLandCoverType->setCell_reduction(atof(value));
			}
			else if(!strcmp(_parameter->getName(),LandCoverTypeParameter::name_cover))
			{
				tmpLandCoverType->setSequence(_parameter->getSequence());
				tmpLandCoverType->setCoverage(atof(value));
			}
			else if(!strcmp(_parameter->getName(),LandCoverTypeParameter::name_fragmentation))
			{
				tmpLandCoverType->setFragmentation(atof(value));
			}
			covertype_iterator = cover_types->end();
		}
	}
}

void ParameterReader::initSimulator()
{
	Logging::log_debug("ParameterReader::initSimulator() ...",NULL);

	Simulator* simulator = registry->simulator;
	if(!simulator)
		simulator = new Simulator(registry);
	else
		simulator->reset();
	
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();

	while(parameter_iterator != model_parameters.end())
	{
		Parameter* parameter = *parameter_iterator++;
		if(!strcmp(parameter->getType(),Parameter::name_simulation))
		{
			if(!strcmp(parameter->getName(),Parameter::name_simulation_years))
			{	
				simulator->setYears((unsigned short)(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
									parameter->getRandomValueAsInt()));
			}
			else if(!strcmp(parameter->getName(),Parameter::name_simulation_steps))
			{
				simulator->setSteps_per_year((unsigned short)(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
											 parameter->getRandomValueAsInt()));
			}
			else if(!strcmp(parameter->getName(),Parameter::name_simulation_gap))
			{
				simulator->setObs_gap((unsigned short)(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
									  parameter->getRandomValueAsInt()));
			}
			else if(!strcmp(parameter->getName(),Parameter::name_simulation_runs))
			{
				simulator->setRuns((unsigned short)(combination_type == P_FACTORIAL ? parameter->getCurrentValueAsInt() :
								   parameter->getRandomValueAsInt()));
			}
		}
	}

	Logging::log_debug("ParameterReader::initSimulator() DONE",NULL);
}

void ParameterReader::setParameterCombinations(unsigned int _combinations)
{
	if(combination_type != P_FACTORIAL)
		parameter_combinations = _combinations;
}

void ParameterReader::report(ostream& strm) const
{
	Logging::log_debug("ParameterReader::report() ...",NULL);
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		tempParameter->report(strm);
	}
	strm<<"#parameters: "<<model_parameters.size()<<" ";
	strm<<"#combinations: "<<parameter_combinations<<"\n";
	Logging::log_debug("ParameterReader::report() DONE",NULL);
}

void ParameterReader::calculate_parameter_combinations()
{
	unsigned int tmp_combinations = 1;
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		tmp_combinations*=tempParameter->getNumberOfValues() == 0 ? 1 : tempParameter->getNumberOfValues();
	}
	if(combination_type == P_FACTORIAL || (combination_type == P_RANDOM && tmp_combinations == 1))
		parameter_combinations = tmp_combinations;
}

void ParameterReader::resetParameters()
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		tempParameter->reset();
	}
	current_parameter = NULL;
}

bool ParameterReader::nextParameterCombination()
{
	return combination_type == P_FACTORIAL ? nextFactorialCombination() : nextRandomCombination();
}

bool ParameterReader::nextFactorialCombination()
{
	if(!current_parameter && model_parameters.size() == 0)
		return false;
	else
		current_parameter = model_parameters.begin();

	START:
	if((*current_parameter)->nextValue())
	{
		if(current_parameter != model_parameters.begin())
		{
			while(current_parameter-- != model_parameters.begin())
				(*current_parameter)->reset();
			current_parameter++;
		}
		return true;
	}
	else
	{
		if(++current_parameter != model_parameters.end())
			goto START;
		resetParameters();
		return false;
	}
}

bool ParameterReader::nextRandomCombination()
{
	if(++current_combination < parameter_combinations)
		return true;
	return false;
}

void ParameterReader::reportFactorialCombinations(ostream& strm)
{
	do
	{
		parameter_list::const_iterator parameter_iterator = model_parameters.begin();
		while(parameter_iterator != model_parameters.end())
		{
			Parameter* tempParameter = *parameter_iterator++;
			if(tempParameter->getNumberOfValues() > 1)
				strm<<tempParameter->getName()<<":"<<tempParameter->getCurrentValue()<<" ";
		}
		strm<<"\n";
	}
	while(nextFactorialCombination());
}

void ParameterReader::reportCurrentCombination(ostream& strm) const
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		if(tempParameter->getNumberOfValues() > 1)
		{	
			strm<<tempParameter->getUniqueAbbreviatedName()<<":";
			if(combination_type == P_FACTORIAL)
				strm<<tempParameter->getCurrentValue()<<" ";
			else
				strm<<tempParameter->getCurrentRandomValue()<<" ";
		}
	}
	strm<<"\n";
}

void ParameterReader::reportCurrentValues(ostream& strm) const
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		if(combination_type == P_FACTORIAL)
			strm<<tempParameter->getCurrentValue();
		else
			strm<<tempParameter->getCurrentRandomValue();
		strm<<";";
	}
}

void ParameterReader::reportParameterNames(ostream& strm) const
{	
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		strm<<(*parameter_iterator)->getName();
		strm<<";";
		strm<<(*parameter_iterator++)->getUniqueAbbreviatedName();
		strm<<"\n";
	}
}

void ParameterReader::reportCurrentVariations(ostream& strm) const
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	char* curr_val;
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		if(tempParameter && tempParameter->getNumberOfValues() > 1)
		{
			if(combination_type == P_FACTORIAL)
				curr_val = tempParameter->getCurrentValue();
			else
				curr_val = tempParameter->getCurrentRandomValue();

			if(!strcmp(tempParameter->getName(), BehaviourParameter::name_offspring) ||
				!strcmp(tempParameter->getName(), BehaviourParameter::name_step_length))
			{
				sprintf(buffer,"%s",curr_val);
				char* val = strtok(buffer,";");
				val = strtok(NULL,";");
					strm<<val;
			}
			else if(!strcmp(tempParameter->getName(), BehaviourParameter::name_step_angle))
			{
				sprintf(buffer,"%s",curr_val);
				char* val = strtok(buffer,";");
				val = strtok(NULL,";");
				val = strtok(NULL,";");
					strm<<val;
			}
			else
				strm<<curr_val;
			strm<<",";
			
		}
	}
}

void ParameterReader::reportVariationNames(ostream& strm) const
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* tempParameter = *parameter_iterator++;
		if(tempParameter && tempParameter->getNumberOfValues() > 1)
		{	
			strm<<tempParameter->getUniqueAbbreviatedName();
			strm<<",";
		}
	}
}

void ParameterReader::initParameterCheckList()
{
	parameter_checklist["PARAMETER_MODE"] = false;
	parameter_checklist["RANDOM_COMBINATIONS"] = false;
	parameter_checklist["START_SEED"] = false;
	parameter_checklist["RUNS"] = false;
	parameter_checklist["OBS_GAP"] = false;
	parameter_checklist["GRID_FILE"] = false;
	parameter_checklist["GRID_WIDTH"] = false;
	parameter_checklist["GRID_LENGTH"] = false;
	parameter_checklist["START_NUM_1"] = false;
	parameter_checklist["START_NUM_2"] = false;
	parameter_checklist["RESOURCES_FACTOR_1_HAB"] = false;
	parameter_checklist["RESOURCES_FACTOR_2_HAB"] = false;
	parameter_checklist["RESOURCES_FACTOR_1_MAT"] = false;
	parameter_checklist["RESOURCES_FACTOR_2_MAT"] = false;
	parameter_checklist["YEARS"] = false;
	parameter_checklist["YEAR_DIV"] = false;
	parameter_checklist["MAX_RESOURCES_PER_CELL_HAB"] = false;
	parameter_checklist["MAX_RESOURCES_PER_CELL_MAT"] = false;
	parameter_checklist["REDUCTION_FACTOR_HAB"] = false;
	parameter_checklist["REDUCTION_FACTOR_MAT"] = false;
	parameter_checklist["HABITAT_AMOUNT"] = false;
	parameter_checklist["HABITAT_FRAGMENTATION"] = false;
	parameter_checklist["SURVIVAL_1_HAB"] = false;
	parameter_checklist["SURVIVAL_2_HAB"] = false;
	parameter_checklist["SURVIVAL_1_MAT"] = false;
	parameter_checklist["SURVIVAL_2_MAT"] = false;
	parameter_checklist["MOVE_DIST_1_HAB"] = false;
	parameter_checklist["MOVE_DIST_2_HAB"] = false;
	parameter_checklist["MOVE_DIST_1_MAT"] = false;
	parameter_checklist["MOVE_DIST_2_MAT"] = false;	
	parameter_checklist["MOVE_ANGLE_1_HAB"] = false;
	parameter_checklist["MOVE_ANGLE_2_HAB"] = false;
	parameter_checklist["MOVE_ANGLE_1_MAT"] = false;
	parameter_checklist["MOVE_ANGLE_2_MAT"] = false;
	parameter_checklist["FECUNDITY_1_HAB"] = false;
	parameter_checklist["FECUNDITY_2_HAB"] = false;
	parameter_checklist["FECUNDITY_1_MAT"] = false;
	parameter_checklist["FECUNDITY_2_MAT"] = false;
	parameter_checklist["GRID_BORDER"] = false;
	parameter_checklist["CELL_ADJACENCY"] = false;
	parameter_checklist["INIT_1"] = false;
	parameter_checklist["INIT_2"] = false;
	parameter_checklist["CROSS_1_HAB_MAT"] = false;
	parameter_checklist["CROSS_1_MAT_HAB"] = false;
	parameter_checklist["CROSS_2_HAB_MAT"] = false;
	parameter_checklist["CROSS_2_MAT_HAB"] = false;
	parameter_checklist["EVENT_CYCLE"] = false;
	parameter_checklist["REPRO_STEP_1"] = false;
	parameter_checklist["REPRO_STEP_2"] = false;
}

void ParameterReader::verifyParameters() throw (IbusepmException)
{
	if(parameter_checklist["GRID_FILE"])
	{
		parameter_checklist["GRID_WIDTH"] = true;
		parameter_checklist["GRID_LENGTH"] = true;
		parameter_checklist["HABITAT_AMOUNT"] = true;
		parameter_checklist["HABITAT_FRAGMENTATION"] = true;
	}
	parameter_checklist["GRID_FILE"] = true;
	reference_list::const_iterator checklist_iterator = parameter_checklist.begin();
	while(checklist_iterator != parameter_checklist.end())
	{
		if(!(*checklist_iterator).second)
			throw IbusepmException("Parameter '",(*checklist_iterator).first,"' was not set!");
		checklist_iterator++;
	}
}

void ParameterReader::validateParameterValues() throw (IbusepmException)
{
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
		((Parameter*)*parameter_iterator++)->validate();
}

XMLParameterReader::XMLParameterReader(ModelRegistry* _registry, const char* _filename)
					: ParameterReader(_registry, _filename)
{
	xml_parameters = new TiXmlDocument(_filename);
}

XMLParameterReader::~XMLParameterReader()
{
	Logging::log_debug("XMLParameterReader(",filename,")::~XMLParameterReader()",NULL);
}

void XMLParameterReader::readFile() throw (IbusepmException)
{
	Logging::log_debug("ParameterReader(",filename,")::readFile()",NULL);
	if(!xml_parameters->LoadFile())
		throw IbusepmException("ParameterReader::readFile() could not read '",
								filename, "'!");	
}

void XMLParameterReader::parseParameters() throw (IbusepmException)
{
	if(!xml_parameters)
		throw IbusepmException("ParameterReader::parseLandscapeParameters() parameters not initialized!");

	parseLandscapeParameters();
	parseLandCoverTypeParameters();
	parsePopulationParameters();
	parseSimulationParameters();
	calculate_parameter_combinations();
}

void XMLParameterReader::parseLandscapeParameters() throw (IbusepmException)
{
	TiXmlElement* landscape = xml_parameters->FirstChildElement()->FirstChildElement(Parameter::name_landscape);
	if(!landscape)
		throw IbusepmException("ParameterReader::parseLandscapeParameters() landscape not found!");
	
	TiXmlElement* landscape_parameter = landscape->FirstChildElement();
	while(landscape_parameter)
	{
		if(!strcmp(landscape_parameter->Value().c_str(),Parameter::name_landscape_width))
		{
			Parameter* landscape_width = new Parameter(Parameter::name_landscape_width, "", Parameter::name_landscape,_INTEGER);
			TiXmlElement* value = landscape_parameter->FirstChildElement();
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseLandscapeParameters() no value for",
											"parameter 'width'");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parseLandscapeParameters() value ",
										value_text, " not valid for landscape.width!",NULL);
				else
					landscape_width->addValue((char*)value_text);
				value = value->NextSiblingElement();
			}
			model_parameters.push_back(landscape_width);
		}
		else if(!strcmp(landscape_parameter->Value().c_str(),Parameter::name_landscape_length))
		{
			Parameter* landscape_length = new Parameter(Parameter::name_landscape_length, "",Parameter::name_landscape,_INTEGER);
			TiXmlElement* value = landscape_parameter->FirstChildElement();
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseLandscapeParameters() no value for",
											"parameter 'length'");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parseLandscapeParameters() value ",
										value_text, " not valid for landscape.length!",NULL);
				else
					landscape_length->addValue((char*)value_text);
				value = value->NextSiblingElement();
			}
			model_parameters.push_back(landscape_length);
		}
		else if(!strcmp(landscape_parameter->Value().c_str(),Parameter::name_landscape_border))
		{
			Parameter* landscape_border = new Parameter(Parameter::name_landscape_border, "",Parameter::name_landscape,_STRING);
			TiXmlElement* value = landscape_parameter->FirstChildElement();
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseLandscapeParameters() no value for",
											"parameter 'border'");

				const char* value_text = value_content->Value().c_str();
				if(!isalpha(*value_text))
					Logging::log_error("ParameterReader::parseLandscapeParameters() value ",
										value_text, " not valid for landscape.border!",NULL);
				else
					landscape_border->addValue((char*)value_text);
				value = value->NextSiblingElement();
			}
			model_parameters.push_back(landscape_border);
		}
		else if(!strcmp(landscape_parameter->Value().c_str(),Parameter::name_landscape_cell_adjacency))
		{
			Parameter* landscape_cell_adjacency = new Parameter(Parameter::name_landscape_cell_adjacency, "",Parameter::name_landscape,_STRING);
			TiXmlElement* value = landscape_parameter->FirstChildElement();
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseLandscapeParameters() no value for",
											"parameter 'cell_adjacency'");

				const char* value_text = value_content->Value().c_str();
				if(!isalpha(*value_text))
					Logging::log_error("ParameterReader::parseLandscapeParameters() value ",
										value_text, " not valid for landscape.border!",NULL);
				else
					landscape_cell_adjacency->addValue((char*)value_text);
				value = value->NextSiblingElement();
			}
			model_parameters.push_back(landscape_cell_adjacency);
		}
		landscape_parameter = landscape_parameter->NextSiblingElement();
	}
}

void XMLParameterReader::parseLandCoverTypeParameters() throw (IbusepmException)
{
	TiXmlElement* landcover = xml_parameters->FirstChildElement()->FirstChildElement(Parameter::name_landscape)->FirstChildElement(Parameter::name_landscape_landcover);
	if(!landcover)
		throw IbusepmException("ParameterReader::parseLandCoverTypeParameters() landcover not found!");
	
	while(landcover)
	{
		int landcover_id = atoi((char*)landcover->Attribute("id")->c_str());
		const char* cover_name = landcover->Attribute("name")->c_str();
		const char* gif_color = landcover->Attribute("gif_color")->c_str();
		
		TiXmlElement* landcover_parameter = landcover->FirstChildElement();
		while(landcover_parameter)
		{
			if(!strcmp(landcover_parameter->Value().c_str(),LandCoverTypeParameter::name_cell_capacity))
			{
				LandCoverTypeParameter* landcover_cell_capacity = new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_capacity, "",_FLOAT, (cover_type)landcover_id);
				landcover_cell_capacity->setCover_name(cover_name);
				landcover_cell_capacity->setGif_color(gif_color);
				TiXmlElement* value = landcover_parameter->FirstChildElement();
				while(value)
				{
					TiXmlNode* value_content = value->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parseLandCoverTypeParameters() no value for",
												"parameter 'cell_capacity'");

					const char* value_text = value_content->Value().c_str();
					if(!isdigit(*value_text))
						Logging::log_error("ParameterReader::parseLandCoverTypeParameters() value ",
											value_text, " not valid for landcover.cell_capacity!",NULL);
					else
						landcover_cell_capacity->addValue((char*)value_text);
					value = value->NextSiblingElement();
				}
				model_parameters.push_back(landcover_cell_capacity);
			}
			else if(!strcmp(landcover_parameter->Value().c_str(),LandCoverTypeParameter::name_cell_reduction))
			{
				LandCoverTypeParameter* landcover_cell_reduction = new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_reduction, "", _FLOAT,(cover_type)landcover_id);
				landcover_cell_reduction->setCover_name(cover_name);
				landcover_cell_reduction->setGif_color(gif_color);
				TiXmlElement* value = landcover_parameter->FirstChildElement();
				while(value)
				{
					TiXmlNode* value_content = value->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parseLandCoverTypeParameters() no value for",
												"parameter 'cell_reduction'");

					const char* value_text = value_content->Value().c_str();
					if(!isdigit(*value_text))
						Logging::log_error("ParameterReader::parseLandCoverTypeParameters() value ",
											value_text, " not valid for landcover.cell_reduction!",NULL);
					else
						landcover_cell_reduction->addValue((char*)value_text);
					value = value->NextSiblingElement();
				}
				model_parameters.push_back(landcover_cell_reduction);
			}
			else if(!strcmp(landcover_parameter->Value().c_str(),LandCoverTypeParameter::name_coxpattern))
			{
				int sequence = atoi((char*)landcover_parameter->Attribute("generation_sequence")->c_str());
				TiXmlElement* cox_pattern_parameter = landcover_parameter->FirstChildElement();
				while(cox_pattern_parameter)
				{
					if(!strcmp(cox_pattern_parameter->Value().c_str(),LandCoverTypeParameter::name_cover))
					{
						LandCoverTypeParameter* landcover_cover = new LandCoverTypeParameter(LandCoverTypeParameter::name_cover, "",_FLOAT,(cover_type)landcover_id);
						landcover_cover->setCover_name(cover_name);
						landcover_cover->setGif_color(gif_color);
						landcover_cover->setSequence(sequence);
						TiXmlElement* value = cox_pattern_parameter->FirstChildElement();
						while(value)
						{
							TiXmlNode* value_content = value->FirstChild();
							if(!value_content)
								throw IbusepmException("ParameterReader::parseLandCoverTypeParameters() no value for",
														"parameter 'cover'");

							const char* value_text = value_content->Value().c_str();
							if(!isdigit(*value_text))
								Logging::log_error("ParameterReader::parseLandCoverTypeParameters() value ",
													value_text, " not valid for landcover.cover!",NULL);
							else
								landcover_cover->addValue((char*)value_text);
							value = value->NextSiblingElement();
						}
						model_parameters.push_back(landcover_cover);
					}
					else if(!strcmp(cox_pattern_parameter->Value().c_str(),LandCoverTypeParameter::name_fragmentation))
					{
						LandCoverTypeParameter* landcover_fragmentation = new LandCoverTypeParameter(LandCoverTypeParameter::name_fragmentation, "",_FLOAT,(cover_type)landcover_id);
						landcover_fragmentation->setSequence(sequence);
						landcover_fragmentation->setCover_name(cover_name);
						landcover_fragmentation->setGif_color(gif_color);
						TiXmlElement* value = cox_pattern_parameter->FirstChildElement();
						while(value)
						{
							TiXmlNode* value_content = value->FirstChild();
							if(!value_content)
								throw IbusepmException("ParameterReader::parseLandCoverTypeParameters() no value for",
														"parameter 'fragmentation'");

							const char* value_text = value_content->Value().c_str();
							if(!isdigit(*value_text))
								Logging::log_error("ParameterReader::parseLandCoverTypeParameters() value ",
													value_text, " not valid for landcover.fragmentation!",NULL);
							else
								landcover_fragmentation->addValue((char*)value_text);
							value = value->NextSiblingElement();
						}
						model_parameters.push_back(landcover_fragmentation);
					}
					cox_pattern_parameter = cox_pattern_parameter->NextSiblingElement();
				} //while cox_pattern_parameter
			} //else if coxpattern
			landcover_parameter = landcover_parameter->NextSiblingElement();
		} //while landcover
		landcover = landcover->NextSiblingElement(Parameter::name_landscape_landcover);
	}
}

void XMLParameterReader::parsePopulationParameters() throw (IbusepmException)
{
	TiXmlElement* population = xml_parameters->FirstChildElement()->FirstChildElement(Parameter::name_population);
	if(!population)
		throw IbusepmException("ParameterReader::parsePopulationParameters() population not found!");

	while(population)
	{
		unsigned short population_id = (unsigned short)atoi((char*)population->Attribute("id")->c_str());
		TiXmlElement* population_parameter = population->FirstChildElement();
		while(population_parameter)
		{
			if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_initial_size))
			{
				PopulationParameter* population_initial_size = new PopulationParameter(PopulationParameter::name_initial_size, "",_INTEGER,population_id);
				TiXmlElement* value = population_parameter->FirstChildElement();
				while(value)
				{
					TiXmlNode* value_content = value->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
												"parameter 'initial_size'");

					const char* value_text = value_content->Value().c_str();
					if(!isdigit(*value_text))
						Logging::log_error("ParameterReader::parsePopulationParameters() value ",
											value_text, " not valid for population.initial_size!",NULL);
					else
						population_initial_size->addValue((char*)value_text);
					value = value->NextSiblingElement();
				}
				model_parameters.push_back(population_initial_size);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_overlapping_generations))
			{
				PopulationParameter* population_overlap = new PopulationParameter(PopulationParameter::name_overlapping_generations, "", _STRING,population_id);
				TiXmlElement* value = population_parameter->FirstChildElement();
				while(value)
				{
					TiXmlNode* value_content = value->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
												"parameter 'overlapping_generations'");

					const char* value_text = value_content->Value().c_str();
					population_overlap->addValue((char*)value_text);
					value = value->NextSiblingElement();
				}
				model_parameters.push_back(population_overlap);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_sex_ratio))
			{
				PopulationParameter* population_sex_ratio = new PopulationParameter(PopulationParameter::name_sex_ratio, "",_FLOAT,population_id);
				TiXmlElement* value = population_parameter->FirstChildElement();
				while(value)
				{
					TiXmlNode* value_content = value->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
												"parameter 'sex_ratio'");

					const char* value_text = value_content->Value().c_str();
					if(!isdigit(*value_text))
						Logging::log_error("ParameterReader::parsePopulationParameters() value ",
											value_text, " not valid for population.sex_ratio!",NULL);
					else
						population_sex_ratio->addValue((char*)value_text);
					value = value->NextSiblingElement();
				}
				model_parameters.push_back(population_sex_ratio);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_border_reflection_angle))
			{
				PopulationParameter* population_border_refl = new PopulationParameter(PopulationParameter::name_border_reflection_angle, "", _FLOAT,population_id);
				TiXmlNode* value_content = population_parameter->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
											"parameter 'border_reflection_angle'");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parsePopulationParameters() value ",
										value_text, " not valid for population.border_reflection_angle!",NULL);
				else
					population_border_refl->addValue((char*)value_text);
				model_parameters.push_back(population_border_refl);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_age_classes))
			{
				PopulationParameter* population_age_classes = new PopulationParameter(PopulationParameter::name_age_classes, "", _INTEGER,population_id);
				TiXmlNode* value_content = population_parameter->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
											"parameter 'age_classes'");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parsePopulationParameters() value ",
										value_text, " not valid for population.age_classes!",NULL);
				else
					population_age_classes->addValue((char*)value_text);
				model_parameters.push_back(population_age_classes);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_activity_sequence))
			{
				PopulationParameter* population_act_seq = new PopulationParameter(PopulationParameter::name_activity_sequence, "", _STRING,population_id);
				const char* type = population_parameter->Attribute("type")->c_str();
				
				if(!strcmp(type,"random") || !strcmp(type, "RANDOM") ||
				   !strcmp(type,"default") || !strcmp(type, "DEFAULT"))
				{
					population_act_seq->addValue((char*)type);
				}
				else if(!strcmp(type,"custom") || !strcmp(type, "CUSTOM"))
				{
					string* attributes = new string("custom;");
					
					TiXmlNode* value_content = population_parameter->FirstChild();
					if(!value_content)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no values for",
											   " parameter ", PopulationParameter::name_activity_sequence);

					while(value_content)
					{
						TiXmlNode* value = value_content->FirstChild();
						if(value)
						{
							attributes->append(value->Value().c_str());
							attributes->append(";");
						}
						else
							throw IbusepmException("ParameterReader::parsePopulationParameters() no value for",
												   " parameter ", PopulationParameter::name_activity_sequence);

						value_content = value_content->NextSibling();
					}
					const char* value_text = attributes->c_str();
					population_act_seq->addValue((char*)value_text);
				}
				model_parameters.push_back(population_act_seq);
			}
			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_distribution))
			{
				PopulationParameter* population_distr = new PopulationParameter(PopulationParameter::name_distribution, "", _STRING,population_id);
				const char* type = population_parameter->Attribute("type")->c_str();
				
				if(!strcmp(type,"landscape") || !strcmp(type, "LANDSCAPE"))
				{
					population_distr->addValue((char*)type);
				}
				else if(!strcmp(type,"landcover") || !strcmp(type, "LANDCOVER"))
				{
					string* values = new string(type);
					values->append(";");
					const char* landcover_id = population_parameter->Attribute("landcover_id")->c_str();
					
					if(!landcover_id)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no landcover_id for",
											   " parameter ", PopulationParameter::name_distribution);
					values->append(landcover_id);
					const char* value_text = values->c_str();
					population_distr->addValue((char*)value_text);	
				}
				else if(!strcmp(type,"patch") || !strcmp(type, "PATCH") ||
						!strcmp(type,"cell") || !strcmp(type, "CELL"))
				{
					string* values = new string(type);
					values->append(";");
					
					TiXmlElement* cell_element = population_parameter->FirstChildElement();
					if(!cell_element)
						throw IbusepmException("ParameterReader::parsePopulationParameters() no cell(s) for",
											   " parameter ", PopulationParameter::name_distribution);

					while(cell_element)
					{
						const char* x = cell_element->Attribute("x")->c_str();
						const char* y = cell_element->Attribute("y")->c_str();
						
						if(!x || isdigit(atoi(x)))
							throw IbusepmException("ParameterReader::parsePopulationParameters() x coordinate for",
												   " parameter ", PopulationParameter::name_distribution, " invalid.");
						
						if(!y || isdigit(atoi(y)))
							throw IbusepmException("ParameterReader::parsePopulationParameters() y coordinate for",
											       " parameter ", PopulationParameter::name_distribution, " invalid.");
						values->append(x);
						values->append(";");
						values->append(y);
						values->append(";");

						cell_element = cell_element->NextSiblingElement();
					}
					const char* value_text = values->c_str();
					population_distr->addValue((char*)value_text);
				}
				model_parameters.push_back(population_distr);
			}

			else if(!strcmp(population_parameter->Value().c_str(),PopulationParameter::name_behaviour))
			{
				parseBehaviourParameters(population_parameter, population_id);
			}
			population_parameter = population_parameter->NextSiblingElement();
		}
		population = population->NextSiblingElement(Parameter::name_population);
	}
}

void XMLParameterReader::parseBehaviourParameters(TiXmlElement* _behaviour, int population_id) throw (IbusepmException)
{
	TiXmlElement* dependency = _behaviour->FirstChildElement(BehaviourParameter::name_dependency);
	if(!dependency)
		throw IbusepmException("ParameterReader::parseBehaviourParameters() dependency not found!");

	TiXmlElement* dependency_landcover = dependency->FirstChildElement(BehaviourParameter::name_landcover);
	TiXmlElement* dependency_gender = dependency->FirstChildElement(BehaviourParameter::name_gender);
	TiXmlElement* dependency_age = dependency->FirstChildElement(BehaviourParameter::name_age);
	
	if(!dependency_landcover)
		throw IbusepmException("ParameterReader::parseBehaviourParameters() landcover_id not found!");
	if(!dependency_gender)
		throw IbusepmException("ParameterReader::parseBehaviourParameters() gender not found!");
	if(!dependency_age)
		throw IbusepmException("ParameterReader::parseBehaviourParameters() age not found!");
	
	TiXmlElement* dependency_gender_value;
	TiXmlElement* dependency_age_value;
	
	int boundary_crossing_parameters = count_boundary_crossing_parameters(_behaviour);
	parameter_list behaviour_parameters;

	int landcover_id = atoi((char*)dependency_landcover->Attribute("id")->c_str());

	dependency_gender_value = dependency_gender->FirstChildElement();
	while(dependency_gender_value)
	{
		gender_types gender = !strcmp(dependency_gender_value->FirstChild()->Value().c_str(),"male") ? MALE : FEMALE;
		dependency_age_value = dependency_age->FirstChildElement();
		while(dependency_age_value)
		{
			age_type age = (age_type)atoi((char*)dependency_age_value->FirstChild()->Value().c_str());

			BehaviourParameter* step_angle_parameter = new BehaviourParameter(BehaviourParameter::name_step_angle, "", _STRING,population_id,(cover_type)landcover_id,age,gender);
			BehaviourParameter* step_length_parameter = new BehaviourParameter(BehaviourParameter::name_step_length, "", _STRING,population_id,(cover_type)landcover_id,age,gender);
			BehaviourParameter* offspring_parameter = new BehaviourParameter(BehaviourParameter::name_offspring, "", _STRING,population_id,(cover_type)landcover_id,age,gender);
			BehaviourParameter* mortality_parameter = new BehaviourParameter(BehaviourParameter::name_mortality, "", _FLOAT,population_id,(cover_type)landcover_id,age,gender);
			BehaviourParameter* resource_parameter = new BehaviourParameter(BehaviourParameter::name_resource_factor, "", _FLOAT,population_id,(cover_type)landcover_id,age,gender);

			model_parameters.push_back(step_angle_parameter);
			model_parameters.push_back(step_length_parameter);
			model_parameters.push_back(offspring_parameter);
			model_parameters.push_back(mortality_parameter);
			model_parameters.push_back(resource_parameter);

			behaviour_parameters.push_back(step_angle_parameter);
			behaviour_parameters.push_back(step_length_parameter);
			behaviour_parameters.push_back(offspring_parameter);
			behaviour_parameters.push_back(mortality_parameter);
			behaviour_parameters.push_back(resource_parameter);
			
			for(int i=0; i<boundary_crossing_parameters; i++)
			{
				BehaviourParameter* tempParameter = new BehaviourParameter(BehaviourParameter::name_boundary_crossing, "", _FLOAT,population_id,(cover_type)landcover_id,age,gender);
				model_parameters.push_back(tempParameter);
				behaviour_parameters.push_back(tempParameter);
			}
			dependency_age_value = dependency_age_value->NextSiblingElement();
		}
		dependency_gender_value = dependency_gender_value->NextSiblingElement();
	}
		

	TiXmlElement* parameter = _behaviour->FirstChildElement(BehaviourParameter::name_parameter);
	if(!parameter)
		throw IbusepmException("ParameterReader::parseBehaviourParameters() parameter not found!");

	TiXmlElement* behaviour_parameter = parameter->FirstChildElement();
	while(behaviour_parameter)
	{
		if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_step_angle))
		{
			TiXmlElement* value = behaviour_parameter->FirstChildElement();	
			while(value)
			{
				string* attributes = (string*)value->Attribute("type");
				string* min_value = (string*)value->Attribute("mean-min");
				string* max_value = (string*)value->Attribute("max");
				if(min_value != NULL)
				{
					attributes->append(";");
					attributes->append(min_value->c_str());
				}
				if(max_value != NULL)
				{
					attributes->append(";");
					attributes->append(max_value->c_str());
				}

				const char* value_text = attributes->c_str();
				parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
				while(behaviour_iterator != behaviour_parameters.end())
				{
					Parameter* tempParameter = *behaviour_iterator;
					if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_step_angle))
						tempParameter->addValue((char*)value_text);
					behaviour_iterator++;
				}
				value = value->NextSiblingElement();
			}
		}
		else if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_step_length))
		{
			TiXmlElement* value = behaviour_parameter->FirstChildElement();	
			while(value)
			{
				string* attributes = (string*)value->Attribute("type");
				string* min_value = (string*)value->Attribute("mean-min");
				string* max_value = (string*)value->Attribute("max");
				if(min_value != NULL)
				{
					attributes->append(";");
					attributes->append(min_value->c_str());
				}
				if(max_value != NULL)
				{
					attributes->append(";");
					attributes->append(max_value->c_str());
				}

				const char* value_text = attributes->c_str();
				parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
				while(behaviour_iterator != behaviour_parameters.end())
				{
					Parameter* tempParameter = *behaviour_iterator;
					if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_step_length))
						tempParameter->addValue((char*)value_text);
					behaviour_iterator++;
				}
				value = value->NextSiblingElement();
			}
		}
		else if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_offspring))
		{
			TiXmlElement* value = behaviour_parameter->FirstChildElement();	
			while(value)
			{
				string* attributes = (string*)value->Attribute("type");
				string* min_value = (string*)value->Attribute("mean-min");
				string* max_value = (string*)value->Attribute("max");
				if(min_value != NULL)
				{
					attributes->append(";");
					attributes->append(min_value->c_str());
				}
				if(max_value != NULL)
				{
					attributes->append(";");
					attributes->append(max_value->c_str());
				}

				const char* value_text = attributes->c_str();
				parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
				while(behaviour_iterator != behaviour_parameters.end())
				{
					Parameter* tempParameter = *behaviour_iterator;
					if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_offspring))
						tempParameter->addValue((char*)value_text);
					behaviour_iterator++;
				}
				value = value->NextSiblingElement();
			}
		}
		else if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_mortality))
		{
			TiXmlElement* value = behaviour_parameter->FirstChildElement();	
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseBehaviourParameters() no value for",
						" parameter 'mortality'");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parseBehaviourParameters() value ",
											value_text, " not valid for behaviour.mortality!",NULL);
				else
				{
					parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
					while(behaviour_iterator != behaviour_parameters.end())
					{
						Parameter* tempParameter = *behaviour_iterator;
						if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_mortality))
							tempParameter->addValue((char*)value_text);
						behaviour_iterator++;
					}
				}
				value = value->NextSiblingElement();
			}
		}
		else if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_resource_factor))
		{
			TiXmlElement* value = behaviour_parameter->FirstChildElement();	
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseBehaviourParameters() no value for",
						" parameter ", BehaviourParameter::name_resource_factor, ".");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parseBehaviourParameters() value ",
											value_text, " not valid for behaviour.resource_factor",NULL);
				else
				{
					parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
					while(behaviour_iterator != behaviour_parameters.end())
					{
						Parameter* tempParameter = *behaviour_iterator;
						if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_resource_factor))
							tempParameter->addValue((char*)value_text);
						behaviour_iterator++;
					}
				}
				value = value->NextSiblingElement();
			}
		}
		else if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_boundary_crossing))
		{
			int to_cover_id = 0;
			string* to_cover = (string*)behaviour_parameter->Attribute("to_landcover_id");

			if(to_cover != NULL)
				to_cover_id = atoi((char*)to_cover->c_str());
			
			gender_types gender = NOTHING;
			age_type age;
			parameter_list::iterator behaviour_iterator = behaviour_parameters.begin();
			while(behaviour_iterator != behaviour_parameters.end())
			{
				BehaviourParameter* tempParameter = (BehaviourParameter*)*behaviour_iterator;
				if(tempParameter && !strcmp(tempParameter->getName(),BehaviourParameter::name_boundary_crossing) && 
				   tempParameter->getCross_cover_id() == NULL_TYPE)
				{
					if(tempParameter->getGender() != gender || tempParameter->getAge() != age)
					{
						gender = tempParameter->getGender();
						age = tempParameter->getAge();

						tempParameter->setCross_cover_id((cover_type)to_cover_id);
						TiXmlElement* value = behaviour_parameter->FirstChildElement();	
						while(value)
						{
							TiXmlNode* value_content = value->FirstChild();
							if(!value_content)
								throw IbusepmException("ParameterReader::parseBehaviourParameters() no value for",
								" parameter 'boundary_crossing'");

							const char* value_text = value_content->Value().c_str();
							if(!isdigit(*value_text))
								Logging::log_error("ParameterReader::parseBehaviourParameters() value ",
													value_text, " not valid for behaviour.boundary_crossing!",NULL);

							tempParameter->addValue((char*)value_text);
							value = value->NextSiblingElement();
						}
					}
				}
				behaviour_iterator++;
			}
		}
		behaviour_parameter = behaviour_parameter->NextSiblingElement();
	}
}

void XMLParameterReader::parseSimulationParameters() throw (IbusepmException)
{
	TiXmlElement* simulation = xml_parameters->FirstChildElement()->FirstChildElement(Parameter::name_simulation);
	if(!simulation)
		throw IbusepmException("ParameterReader::parseSimulationParameters() simulation not found!");
	
	TiXmlElement* simulation_parameter = simulation->FirstChildElement();
	while(simulation_parameter)
	{
		if(!strcmp(simulation_parameter->Value().c_str(),Parameter::name_simulation_steps))
		{
			Parameter* simulation_steps = new Parameter(Parameter::name_simulation_steps, "", Parameter::name_simulation,_INTEGER);
			
			TiXmlNode* value_content = simulation_parameter->FirstChild();
			if(!value_content)
				throw IbusepmException("ParameterReader::parseSimulationParameters() no value for parameter '",
										Parameter::name_simulation_steps, "'.");

			const char* value_text = value_content->Value().c_str();
			if(!isdigit(*value_text))
				Logging::log_error("ParameterReader::parseSimulationParameters() value ",
										value_text, " not valid for ", Parameter::name_simulation_steps, NULL);
			else
				simulation_steps->addValue((char*)value_text);

			model_parameters.push_back(simulation_steps);
		}
		else if(!strcmp(simulation_parameter->Value().c_str(),Parameter::name_simulation_years))
		{
			Parameter* simulation_years = new Parameter(Parameter::name_simulation_years, "", Parameter::name_simulation,_INTEGER);
			
			TiXmlNode* value_content = simulation_parameter->FirstChild();
			if(!value_content)
				throw IbusepmException("ParameterReader::parseSimulationParameters() no value for parameter '",
										Parameter::name_simulation_years, "'.");

			const char* value_text = value_content->Value().c_str();
			if(!isdigit(*value_text))
				Logging::log_error("ParameterReader::parseSimulationParameters() value ",
									value_text, " not valid for ", Parameter::name_simulation_years, NULL);
			else
				simulation_years->addValue((char*)value_text);

			model_parameters.push_back(simulation_years);
		}
		else if(!strcmp(simulation_parameter->Value().c_str(),Parameter::name_simulation_gap))
		{
			Parameter* simulation_gap = new Parameter(Parameter::name_simulation_gap, "", Parameter::name_simulation,_INTEGER);
			
			TiXmlNode* value_content = simulation_parameter->FirstChild();
			if(!value_content)
				throw IbusepmException("ParameterReader::parseSimulationParameters() no value for parameter '",
										Parameter::name_simulation_gap, "'.");

			const char* value_text = value_content->Value().c_str();
			if(!isdigit(*value_text))
				Logging::log_error("ParameterReader::parseSimulationParameters() value ",
									value_text, " not valid for ", Parameter::name_simulation_gap, NULL);
			else
				simulation_gap->addValue((char*)value_text);

			model_parameters.push_back(simulation_gap);
		}
		else if(!strcmp(simulation_parameter->Value().c_str(),Parameter::name_simulation_combination))
		{
			Parameter* simulation_combination = new Parameter(Parameter::name_simulation_combination, "", Parameter::name_simulation,_STRING);
			
			TiXmlNode* value_content = simulation_parameter->FirstChild();
			if(!value_content)
				throw IbusepmException("ParameterReader::parseSimulationParameters() no value for parameter '",
										Parameter::name_simulation_combination, "'.");

			const char* value_text = value_content->Value().c_str();	
			simulation_combination->addValue((char*)value_text);

			model_parameters.push_back(simulation_combination);
		}
		else if(!strcmp(simulation_parameter->Value().c_str(),Parameter::name_simulation_runs))
		{
			Parameter* simulation_runs = new Parameter(Parameter::name_simulation_runs, "", Parameter::name_simulation,_INTEGER);
			TiXmlElement* value = simulation_parameter->FirstChildElement();
			while(value)
			{
				TiXmlNode* value_content = value->FirstChild();
				if(!value_content)
					throw IbusepmException("ParameterReader::parseSimulationParameters() no value for parameter '",
											Parameter::name_simulation_runs, "'.");

				const char* value_text = value_content->Value().c_str();
				if(!isdigit(*value_text))
					Logging::log_error("ParameterReader::parseSimulationParameters() value ",
										value_text, " not valid for ", Parameter::name_simulation_runs, NULL);
				else
					simulation_runs->addValue((char*)value_text);
				value = value->NextSiblingElement();
			}
			model_parameters.push_back(simulation_runs);
		}
		simulation_parameter = simulation_parameter->NextSiblingElement();
	}
}

void XMLParameterReader::initObservers() throw (IbusepmException)
{
	registry->resetObservers();

	if(!registry->simulator)
		throw IbusepmException("XMLParameterReader::parseObserverParameters() simulator not initialized!");

	TiXmlElement* observer = xml_parameters->FirstChildElement()->FirstChildElement(Parameter::name_observer);
	if(!observer)
		throw IbusepmException("XMLParameterReader::parseObserverParameters() observer not found!");

	TiXmlElement* landcover = observer->FirstChildElement("landcover");
	if(!landcover)
		throw IbusepmException("XMLParameterReader::parseObserverParameters() landcover not found!");

	TiXmlElement* population = observer->FirstChildElement("population");
	if(!population)
		throw IbusepmException("XMLParameterReader::parseObserverParameters() population not found!");

	TiXmlElement* population_observer = population->FirstChildElement();
	while(population_observer)
	{
		if(!strcmp(population_observer->Value().c_str(), ObserverParameter::name_size))
		{
			string* population_id = (string*)population_observer->Attribute("population_id");
			string* report_details = (string*)population_observer->Attribute("report_details");
			
			if(population_id != NULL && !isdigit(atoi(population_id->c_str())) &&
			   report_details != NULL)
			{
				PopulationObserver* pop_obs = new PopulationObserver(registry, (unsigned short)atoi(population_id->c_str()),
																	registry->simulator->getRuns(), registry->simulator->getObs_steps());
				pop_obs->setReport_raw_obs(!strcmp(report_details->c_str(),"no") ? false : true);
			}
		}
		population_observer = population_observer->NextSiblingElement();
	}

	if(registry->getObservers()->empty())
		throw IbusepmException("At lease one observer must be defined.");
}

int XMLParameterReader::count_boundary_crossing_parameters(TiXmlElement* _behaviour) throw (IbusepmException)
{
	TiXmlElement* parameter = _behaviour->FirstChildElement("parameter");
	if(!parameter)
		throw IbusepmException("ParameterReader::count_boundary_crossing_parameters() parameter not found!");

	int counter = 0;
	TiXmlElement* behaviour_parameter = parameter->FirstChildElement();
	while(behaviour_parameter)
	{
		if(!strcmp(behaviour_parameter->Value().c_str(),BehaviourParameter::name_boundary_crossing))
			counter++;
		behaviour_parameter = behaviour_parameter->NextSiblingElement();
	}
	return counter;
}


ASCIIParameterReader::ASCIIParameterReader(ModelRegistry* _registry, const char* _filename)
											: ParameterReader(_registry, _filename)
{
}

int ASCIIParameterReader::read_number_of_lines_from_parameterfile() const throw (IbusepmException)
{
	int lines=0;
	ifstream is(filename, ios::in);
	if(!is)
		throw IbusepmException("ASCIIParameterReader::read_number_of_lines_from_parameterfile() ",
							   "cannot open parameter file '", filename, "'.");
	char ch;
	while(is.get(ch))
	if(ch == '\n')
		lines++;
	
	is.close();
	return lines;
}

void ASCIIParameterReader::parseParameters() throw (IbusepmException)
{
	char buf[200];
	char* argument;
	char* eqp;
	char* delims={" ,"};
	char* val;
	int number_of_lines_in_parameter_file = read_number_of_lines_from_parameterfile();
	char* year_div = 0;
	char* years = 0;

	ifstream is(filename, ios::in);
	if(!is)
		throw IbusepmException("ASCIIParameterReader::parseParameters() ",
							   "cannot open parameter file '", filename, "'.");

	for(int l=0;l<=number_of_lines_in_parameter_file;l++)
	{
   		is.getline(buf,200,'\n');
		argument=strdup(buf);
		eqp=strpbrk(argument," ");
		if(eqp != NULL)
		{
      		*eqp=0;
			eqp++;
			val=strtok(eqp,delims);
			if(val)
			{
         		if(!strcmp(argument,"PARAMETER_MODE"))
				{
					if(val && isdigit(*val) && (atoi(val) == 0 || atoi(val) == 1))
					{
						combination_type = atoi(val) == 0 ? P_FACTORIAL : P_RANDOM;
						parameter_checklist["PARAMETER_MODE"] = true;
					}
				}
				if(!strcmp(argument,"RANDOM_COMBINATIONS"))
				{
					if(val && isdigit(*val))
					{
						setParameterCombinations(atoi(val));
						parameter_checklist["RANDOM_COMBINATIONS"] = true;
					}
				}
				else if(!strcmp(argument,"START_SEED"))
				{
					if(val && isdigit(*val))
					{
						setStartSeed(atoi(val));
						parameter_checklist["START_SEED"] = true;
					}
				}
				else if(!strcmp(argument,"SAVE_DETAILS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
						save_details = true;
				}
				else if(!strcmp(argument,"SAVE_GIF_LANDSCAPES"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
						save_gif_landscapes = true;
				}
				else if(!strcmp(argument,"SAVE_ASC_LANDSCAPES"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
						save_asc_landscapes = true;
				}
				else if(!strcmp(argument,"RUNS"))
				{
					Parameter* simulation_runs = new Parameter(Parameter::name_simulation_runs,"RUNS", Parameter::name_simulation,_INTEGER,"1;1000");
					if(val && isdigit(*val))
					{
						simulation_runs->addValue(val);
						model_parameters.push_back(simulation_runs);
						parameter_checklist["RUNS"] = true;
					}
				}
				else if(!strcmp(argument,"OBS_GAP"))
				{
					char valid[20];
					sprintf(valid,"1:%d",atoi(years)*atoi(year_div));
					Parameter* simulation_gap = new Parameter(Parameter::name_simulation_gap,"OBS_GAP", Parameter::name_simulation,_INTEGER,valid);
					if(val && isdigit(*val))
					{
						simulation_gap->addValue(val);
						model_parameters.push_back(simulation_gap);
						parameter_checklist["OBS_GAP"] = true;
					}
				}
				else if(!strcmp(argument,"GRID_WIDTH") && !parameter_checklist["GRID_FILE"])
				{	
					Parameter* landscape_width = new Parameter(Parameter::name_landscape_width, "G_WIDTH", Parameter::name_landscape,_INTEGER,"1;1000");
					while(val && isdigit(*val))
					{
						landscape_width->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["GRID_WIDTH"] = true;
					}
					model_parameters.push_back(landscape_width);
				}
				else if(!strcmp(argument,"GRID_LENGTH") && !parameter_checklist["GRID_FILE"])
				{	
					Parameter* landscape_length = new Parameter(Parameter::name_landscape_length,"G_LENGTH", Parameter::name_landscape,_INTEGER,"1;1000");
					while(val && isdigit(*val))
					{
						landscape_length->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["GRID_LENGTH"] = true;
					}
					model_parameters.push_back(landscape_length);
				}
				else if(!strcmp(argument,"START_NUM_1"))
				{	
					PopulationParameter* population_1_initial_size = new PopulationParameter(PopulationParameter::name_initial_size, "P1_START", _INTEGER,1,"0;100000");
					while(val && isdigit(*val))
					{
						population_1_initial_size->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["START_NUM_1"] = true;
					}
					model_parameters.push_back(population_1_initial_size);
				}
				else if(!strcmp(argument,"START_NUM_2"))
				{	
					PopulationParameter* population_2_initial_size = new PopulationParameter(PopulationParameter::name_initial_size, "P2_START", _INTEGER,2,"0;100000");
					while(val && isdigit(*val))
					{
						population_2_initial_size->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["START_NUM_2"] = true;
					}
					model_parameters.push_back(population_2_initial_size);
				}
				else if(!strcmp(argument,"RESOURCES_FACTOR_1_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(
																  BehaviourParameter::name_resource_factor, "H_P1_RES",
																  _FLOAT,1,LandCoverType::habitat,1,FEMALE,"0;1000");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["RESOURCES_FACTOR_1_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"RESOURCES_FACTOR_2_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(
																  BehaviourParameter::name_resource_factor, "H_P2_RES",
																  _FLOAT,2,LandCoverType::habitat,1,FEMALE,"0;1000");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["RESOURCES_FACTOR_2_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"RESOURCES_FACTOR_1_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(
																  BehaviourParameter::name_resource_factor, "M_P1_RES", 
																  _FLOAT,1,LandCoverType::matrix,1,FEMALE,"0;1000");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["RESOURCES_FACTOR_1_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"RESOURCES_FACTOR_2_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(
																  BehaviourParameter::name_resource_factor, "M_P2_RES",
																  _FLOAT,2,LandCoverType::matrix,1,FEMALE,"0;1000");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["RESOURCES_FACTOR_2_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"YEARS"))
				{
					Parameter* simulation_years = new Parameter(Parameter::name_simulation_years, "YEARS", Parameter::name_simulation,_INTEGER, "1;1000");

					if(val && isdigit(*val))
					{
						simulation_years->addValue(val);
						model_parameters.push_back(simulation_years);
						years = val;
						parameter_checklist["YEARS"] = true;
					}
				}
				else if(!strcmp(argument,"YEAR_DIV"))
				{
					Parameter* simulation_steps = new Parameter(Parameter::name_simulation_steps, "STEPS", Parameter::name_simulation,_INTEGER, "1;12");

					if(val && isdigit(*val))
					{
						simulation_steps->addValue(val);
						model_parameters.push_back(simulation_steps);
						year_div = val;
						parameter_checklist["YEAR_DIV"] = true;
					}
				}
				else if(!strcmp(argument,"MAX_RESOURCES_PER_CELL_HAB"))
				{
					LandCoverTypeParameter* landcover_habitat_cell_capacity = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_capacity, "H_CELRES", _INTEGER, LandCoverType::habitat,"1;10000");					
					landcover_habitat_cell_capacity->setCover_name("habitat");
					landcover_habitat_cell_capacity->setGif_color("dark_green");
					
					while(val && isdigit(*val))
					{
						landcover_habitat_cell_capacity->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["MAX_RESOURCES_PER_CELL_HAB"] = true;
					}
					model_parameters.push_back(landcover_habitat_cell_capacity);
				}
				else if(!strcmp(argument,"MAX_RESOURCES_PER_CELL_MAT"))
				{
					LandCoverTypeParameter* landcover_matrix_cell_capacity = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_capacity,"M_CELRES", _INTEGER, LandCoverType::matrix,"1;10000");
					landcover_matrix_cell_capacity->setCover_name("matrix"); 
					landcover_matrix_cell_capacity->setGif_color("white");
					
					while(val && isdigit(*val))
					{
						landcover_matrix_cell_capacity->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["MAX_RESOURCES_PER_CELL_MAT"] = true;
					}
					model_parameters.push_back(landcover_matrix_cell_capacity);
				}
				else if(!strcmp(argument,"REDUCTION_FACTOR_HAB"))
				{
					LandCoverTypeParameter* landcover_habitat_cell_reduction = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_reduction, "H_REDFAC", _FLOAT, LandCoverType::habitat,"0;1");
					landcover_habitat_cell_reduction->setCover_name("habitat");
					landcover_habitat_cell_reduction->setGif_color("dark_green");
					
					while(val && isdigit(*val))
					{
						landcover_habitat_cell_reduction->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["REDUCTION_FACTOR_HAB"] = true;
					}
					model_parameters.push_back(landcover_habitat_cell_reduction);
				}
				else if(!strcmp(argument,"REDUCTION_FACTOR_MAT"))
				{
					LandCoverTypeParameter* landcover_matrix_cell_reduction = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_cell_reduction, "M_REDFAC", _FLOAT, LandCoverType::matrix,"0;1");
					landcover_matrix_cell_reduction->setCover_name("matrix");
					landcover_matrix_cell_reduction->setGif_color("white");
					
					while(val && isdigit(*val))
					{
						landcover_matrix_cell_reduction->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["REDUCTION_FACTOR_MAT"] = true;
					}
					model_parameters.push_back(landcover_matrix_cell_reduction);
				}
				else if(!strcmp(argument,"HABITAT_AMOUNT") && !parameter_checklist["GRID_FILE"])
				{	
					LandCoverTypeParameter* landcover_cover = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_cover, "H_AMOUNT", _FLOAT, LandCoverType::habitat,"0.01;0.9");
					landcover_cover->setCover_name("habitat");
					landcover_cover->setGif_color("dark_green");
					landcover_cover->setSequence(1);
					
					while(val && isdigit(*val))
					{
						landcover_cover->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["HABITAT_AMOUNT"] = true;
					}
					model_parameters.push_back(landcover_cover);
				}
				else if(!strcmp(argument,"HABITAT_FRAGMENTATION") && !parameter_checklist["GRID_FILE"])
				{	
					LandCoverTypeParameter* landcover_fragmentation = 
						new LandCoverTypeParameter(LandCoverTypeParameter::name_fragmentation, "H_FRAG", _FLOAT, LandCoverType::habitat,"0.01;1");
					landcover_fragmentation->setCover_name("habitat");
					landcover_fragmentation->setGif_color("dark_green");
					landcover_fragmentation->setSequence(1);
					
					while(val && isdigit(*val))
					{
						landcover_fragmentation->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["HABITAT_FRAGMENTATION"] = true;
					}
					model_parameters.push_back(landcover_fragmentation);
				}
				else if(!strcmp(argument,"SURVIVAL_1_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_mortality, "H_P1_SUR",
																  _FLOAT,1,LandCoverType::habitat,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["SURVIVAL_1_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"SURVIVAL_2_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_mortality, "H_P2_SUR",
																  _FLOAT,2, LandCoverType::habitat,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["SURVIVAL_2_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"SURVIVAL_1_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_mortality, "M_P1_SUR",
																  _FLOAT,1,LandCoverType::matrix,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["SURVIVAL_1_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"SURVIVAL_2_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_mortality, "M_P2_SUR",
																  _FLOAT,2,LandCoverType::matrix,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						behaviour_parameter->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["SURVIVAL_2_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"MOVE_DIST_1_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_step_length, "H_P1_MOV",
																  _STRING,1,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 100)
							throw IbusepmException("Parameter MOVE_DIST_1_HAB has invalid value\n");
						string* parameter = new string("negexp");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_DIST_1_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"MOVE_DIST_2_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_step_length, "H_P2_MOV",
																  _STRING,2,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 100)
							throw IbusepmException("Parameter MOVE_DIST_2_HAB has invalid value\n");
						string* parameter = new string("negexp");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_DIST_2_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"MOVE_DIST_1_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_step_length, "M_P1_MOV",
																  _STRING,1,LandCoverType::matrix,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 100)
							throw IbusepmException("Parameter MOVE_DIST_1_MAT has invalid value\n");
						string* parameter = new string("negexp");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_DIST_1_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"MOVE_DIST_2_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_step_length, "M_P2_MOV",
																  _STRING,2,LandCoverType::matrix,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 100)
							throw IbusepmException("Parameter MOVE_DIST_2_MAT has invalid value\n");
						string* parameter = new string("negexp");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_DIST_2_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"MOVE_ANGLE_1_HAB"))
				{	
					BehaviourParameter* angle_1_habitat = new BehaviourParameter(BehaviourParameter::name_step_angle, "H_P1_ANG",
																_STRING,1,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 3.14159)
							throw IbusepmException("Parameter MOVE_ANGLE_1_HAB has invalid value\n");
						string* parameter = new string("uniform");
						parameter->append(";-");
						parameter->append(val);
						parameter->append(";");
						parameter->append(val);

						angle_1_habitat->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_ANGLE_1_HAB"] = true;
					}
					model_parameters.push_back(angle_1_habitat);
				}
				else if(!strcmp(argument,"MOVE_ANGLE_2_HAB"))
				{	
					BehaviourParameter* angle_2_habitat = new BehaviourParameter(BehaviourParameter::name_step_angle, "H_P2_ANG",
																_STRING,2,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 3.14159)
							throw IbusepmException("Parameter MOVE_ANGLE_2_HAB has invalid value\n");
						string* parameter = new string("uniform");
						parameter->append(";-");
						parameter->append(val);
						parameter->append(";");
						parameter->append(val);

						angle_2_habitat->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_ANGLE_2_HAB"] = true;
					}
					model_parameters.push_back(angle_2_habitat);
				}
				else if(!strcmp(argument,"MOVE_ANGLE_1_MAT"))
				{	
					BehaviourParameter* angle_1_matrix = new BehaviourParameter(BehaviourParameter::name_step_angle, "M_P1_ANG",
																				_STRING,1,LandCoverType::matrix,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 3.14159)
							throw IbusepmException("Parameter MOVE_ANGLE_1_MAT has invalid value\n");
						string* parameter = new string("uniform");
						parameter->append(";-");
						parameter->append(val);
						parameter->append(";");
						parameter->append(val);

						angle_1_matrix->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_ANGLE_1_MAT"] = true;
					}
					model_parameters.push_back(angle_1_matrix);
				}
				else if(!strcmp(argument,"MOVE_ANGLE_2_MAT"))
				{	
					BehaviourParameter* angle_2_matrix = new BehaviourParameter(BehaviourParameter::name_step_angle, "M_P2_ANG",
																			_STRING,2,LandCoverType::matrix,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 3.14159)
							throw IbusepmException("Parameter MOVE_ANGLE_2_MAT has invalid value\n");
						string* parameter = new string("uniform");
						parameter->append(";-");
						parameter->append(val);
						parameter->append(";");
						parameter->append(val);

						angle_2_matrix->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["MOVE_ANGLE_2_MAT"] = true;
					}
					model_parameters.push_back(angle_2_matrix);
				}
				else if(!strcmp(argument,"FECUNDITY_1_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_offspring, "H_P1_FEC",
																  _STRING,1,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 1000)
							throw IbusepmException("Parameter FECUNDITY_1_HAB has invalid value\n");
						string* parameter = new string("poisson");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["FECUNDITY_1_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"FECUNDITY_2_HAB"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_offspring, "H_P2_FEC",
																  _STRING,2,LandCoverType::habitat,1,FEMALE);
					
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 1000)
							throw IbusepmException("Parameter FECUNDITY_2_HAB has invalid value\n");
						string* parameter = new string("poisson");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["FECUNDITY_2_HAB"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"FECUNDITY_1_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_offspring, "M_P1_FEC",
																  _STRING,1,LandCoverType::matrix,1,FEMALE);				
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 1000)
							throw IbusepmException("Parameter FECUNDITY_1_MAT has invalid value\n");
						string* parameter = new string("poisson");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["FECUNDITY_1_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"FECUNDITY_2_MAT"))
				{	
					BehaviourParameter* behaviour_parameter = new BehaviourParameter(BehaviourParameter::name_offspring, "M_P2_FEC",
																  _STRING,2,LandCoverType::matrix,1,FEMALE);			
					while(val && isdigit(*val))
					{
						if(atof(val) < 0 || atof(val) > 1000)
							throw IbusepmException("Parameter FECUNDITY_2_MAT has invalid value\n");
						string* parameter = new string("poisson");
						parameter->append(";");
						parameter->append(val);

						behaviour_parameter->addValue((char*)parameter->c_str());
						val=strtok(NULL,delims);
						parameter_checklist["FECUNDITY_2_MAT"] = true;
					}
					model_parameters.push_back(behaviour_parameter);
				}
				else if(!strcmp(argument,"GRID_BORDER"))
				{	
					Parameter* landscape_border = new Parameter(Parameter::name_landscape_border, "G_BORDER",
						Parameter::name_landscape, _STRING,"torus;absorb;reflect");
					while(val)
					{
						landscape_border->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["GRID_BORDER"] = true;
					}
					model_parameters.push_back(landscape_border);
				}
				else if(!strcmp(argument,"CELL_ADJACENCY"))
				{	
					Parameter* landscape_cell_adjacency = new Parameter(Parameter::name_landscape_cell_adjacency, "CELL_ADJ",
						Parameter::name_landscape,_STRING,"orthogonal;diagonal");
					while(val)
					{
						landscape_cell_adjacency->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["CELL_ADJACENCY"] = true;
					}
					model_parameters.push_back(landscape_cell_adjacency);
				}
				else if(!strcmp(argument,"INIT_1"))
				{	
					PopulationParameter* population_1_distr = new PopulationParameter(PopulationParameter::name_distribution, "P1_INIT", 
						_STRING,1,"landscape;habitat;matrix");
					while(val)
					{
						population_1_distr->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["INIT_1"] = true;
					}
					model_parameters.push_back(population_1_distr);
				}
				else if(!strcmp(argument,"INIT_2"))
				{	
					PopulationParameter* population_2_distr = new PopulationParameter(PopulationParameter::name_distribution,"P2_INIT",
						_STRING,2,"landscape;habitat;matrix");
					while(val)
					{
						population_2_distr->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["INIT_2"] = true;
					}
					model_parameters.push_back(population_2_distr);
				}
				else if(!strcmp(argument,"CROSS_1_HAB_MAT"))
				{	
					BehaviourParameter* pop_1_bcross_hab_mat = new BehaviourParameter(BehaviourParameter::name_boundary_crossing,
						"HM_P1_CR", _FLOAT,1,LandCoverType::habitat,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						pop_1_bcross_hab_mat->setCross_cover_id(LandCoverType::matrix);
						pop_1_bcross_hab_mat->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["CROSS_1_HAB_MAT"] = true;
					}
					model_parameters.push_back(pop_1_bcross_hab_mat);
				}
				else if(!strcmp(argument,"CROSS_1_MAT_HAB"))
				{	
					BehaviourParameter* pop_1_bcross_mat_hab= new BehaviourParameter(BehaviourParameter::name_boundary_crossing,
						"MH_P1_CR",_FLOAT,1,LandCoverType::matrix,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						pop_1_bcross_mat_hab->setCross_cover_id(LandCoverType::habitat);
						pop_1_bcross_mat_hab->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["CROSS_1_MAT_HAB"] = true;
					}
					model_parameters.push_back(pop_1_bcross_mat_hab);
				}
				else if(!strcmp(argument,"CROSS_2_HAB_MAT"))
				{	
					BehaviourParameter* pop_2_bcross_hab_mat = new BehaviourParameter(BehaviourParameter::name_boundary_crossing,
						"HM_P2_CR", _FLOAT,2,LandCoverType::habitat,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						pop_2_bcross_hab_mat->setCross_cover_id(LandCoverType::matrix);
						pop_2_bcross_hab_mat->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["CROSS_2_HAB_MAT"] = true;
					}
					model_parameters.push_back(pop_2_bcross_hab_mat);
				}
				else if(!strcmp(argument,"CROSS_2_MAT_HAB"))
				{	
					BehaviourParameter* pop_2_bcross_mat_hab = new BehaviourParameter(BehaviourParameter::name_boundary_crossing,
						"MH_P2_CR", _FLOAT,2,LandCoverType::matrix,1,FEMALE,"0;1");
					while(val && isdigit(*val))
					{
						pop_2_bcross_mat_hab->setCross_cover_id(LandCoverType::habitat);
						pop_2_bcross_mat_hab->addValue(val);
						val=strtok(NULL,delims);
						parameter_checklist["CROSS_2_MAT_HAB"] = true;
					}
					model_parameters.push_back(pop_2_bcross_mat_hab);
				}
				else if(!strcmp(argument,"EVENT_CYCLE"))
				{	
					PopulationParameter* population_1_act_seq = new PopulationParameter(PopulationParameter::name_activity_sequence,
						"EVENT", _STRING,1,"custom;random;repro;move;death");
					PopulationParameter* population_2_act_seq = new PopulationParameter(PopulationParameter::name_activity_sequence,
						"EVENT", _STRING,2,"custom;random;repro;move;death");
					
					string* par_val;
					if(val && !strcmp(val,"random"))
						par_val = new string("random;");
					else
					{
						par_val = new string("custom");
						while(val)
						{
							par_val->append(";");
							par_val->append(val);
							val=strtok(NULL,delims);
						}
					}
					parameter_checklist["EVENT_CYCLE"] = true;
					population_1_act_seq->addValue((char*)par_val->c_str());
					population_2_act_seq->addValue((char*)par_val->c_str());
					
					model_parameters.push_back(population_1_act_seq);
					model_parameters.push_back(population_2_act_seq);
				}
				else if(!strcmp(argument,"ABUNDANCE_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_1_par = new ObserverParameter(ObserverParameter::name_pop_size,_INTEGER,1);
						abund_1_par->setReport_details(save_details);
						model_parameters.push_back(abund_1_par);
					}
				}
				else if(!strcmp(argument,"ABUNDANCE_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_2_par = new ObserverParameter(ObserverParameter::name_pop_size,_INTEGER,2);
						abund_2_par->setReport_details(save_details);
						model_parameters.push_back(abund_2_par);
					}
				}
				else if(!strcmp(argument,"EXTINCTION_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* ext_1_par = new ObserverParameter(ObserverParameter::name_extinction,_INTEGER,1);
						model_parameters.push_back(ext_1_par);
					}
				}
				else if(!strcmp(argument,"EXTINCTION_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* ext_2_par = new ObserverParameter(ObserverParameter::name_extinction,_INTEGER,2);
						model_parameters.push_back(ext_2_par);
					}
				}
				else if(!strcmp(argument,"ABUNDANCE_HAB_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_par = new ObserverParameter(ObserverParameter::name_landcover,_INTEGER,1);
						abund_par->setCover_type(LandCoverType::habitat);
						abund_par->setReport_details(save_details);
						model_parameters.push_back(abund_par);
					}
				}
				else if(!strcmp(argument,"ABUNDANCE_HAB_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_par = new ObserverParameter(ObserverParameter::name_landcover,_INTEGER,2);
						abund_par->setCover_type(LandCoverType::habitat);
						abund_par->setReport_details(save_details);
						model_parameters.push_back(abund_par);
					}
				}
				else if(!strcmp(argument,"ABUNDANCE_MAT_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_par = new ObserverParameter(ObserverParameter::name_landcover,_INTEGER,1);
						abund_par->setCover_type(LandCoverType::matrix);
						abund_par->setReport_details(save_details);
						model_parameters.push_back(abund_par);
					}
				}
				else if(!strcmp(argument,"ABUNDANCE_MAT_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* abund_par = new ObserverParameter(ObserverParameter::name_landcover,_INTEGER,2);
						abund_par->setCover_type(LandCoverType::matrix);
						abund_par->setReport_details(save_details);
						model_parameters.push_back(abund_par);
					}
				}
				else if(!strcmp(argument,"HAB_CELL_IMM_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cim_par = new ObserverParameter(ObserverParameter::name_cell_immigration,_INTEGER,1);
						cim_par->setCover_type(LandCoverType::habitat);
						cim_par->setReport_details(save_details);
						model_parameters.push_back(cim_par);
					}
				}
				else if(!strcmp(argument,"HAB_CELL_IMM_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cim_par = new ObserverParameter(ObserverParameter::name_cell_immigration,_INTEGER,2);
						cim_par->setCover_type(LandCoverType::habitat);
						cim_par->setReport_details(save_details);
						model_parameters.push_back(cim_par);
					}
				}
				else if(!strcmp(argument,"HAB_PATCH_IMM_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* pim_par = new ObserverParameter(ObserverParameter::name_patch_immigration,_INTEGER,1);
						pim_par->setCover_type(LandCoverType::habitat);
						pim_par->setReport_details(save_details);
						model_parameters.push_back(pim_par);
					}
				}
				else if(!strcmp(argument,"HAB_PATCH_IMM_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* pim_par = new ObserverParameter(ObserverParameter::name_patch_immigration,_INTEGER,2);
						pim_par->setCover_type(LandCoverType::habitat);
						pim_par->setReport_details(save_details);
						model_parameters.push_back(pim_par);
					}
				}
				else if(!strcmp(argument,"DISPERSAL_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* disp_par = new ObserverParameter(ObserverParameter::name_movement_distance,_INTEGER,1);
						disp_par->setReport_details(save_details);
						model_parameters.push_back(disp_par);
					}
				}
				else if(!strcmp(argument,"DISPERSAL_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* disp_par = new ObserverParameter(ObserverParameter::name_movement_distance,_INTEGER,2);
						disp_par->setReport_details(save_details);
						model_parameters.push_back(disp_par);
					}
				}
				else if(!strcmp(argument,"CROSS_HAB_MAT_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cross_par = new ObserverParameter(ObserverParameter::name_boundary_crossing,_INTEGER,1);
						cross_par->setCover_type(LandCoverType::habitat);
						cross_par->setCross_Cover_type(LandCoverType::matrix);
						cross_par->setReport_details(save_details);
						model_parameters.push_back(cross_par);
					}
				}
				else if(!strcmp(argument,"CROSS_HAB_MAT_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cross_par = new ObserverParameter(ObserverParameter::name_boundary_crossing,_INTEGER,2);
						cross_par->setCover_type(LandCoverType::habitat);
						cross_par->setCross_Cover_type(LandCoverType::matrix);
						cross_par->setReport_details(save_details);
						model_parameters.push_back(cross_par);
					}
				}
				else if(!strcmp(argument,"CROSS_MAT_HAB_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cross_par = new ObserverParameter(ObserverParameter::name_boundary_crossing,_INTEGER,1);
						cross_par->setCover_type(LandCoverType::matrix);
						cross_par->setCross_Cover_type(LandCoverType::habitat);
						cross_par->setReport_details(save_details);
						model_parameters.push_back(cross_par);
					}
				}
				else if(!strcmp(argument,"CROSS_MAT_HAB_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* cross_par = new ObserverParameter(ObserverParameter::name_boundary_crossing,_INTEGER,2);
						cross_par->setCover_type(LandCoverType::matrix);
						cross_par->setCross_Cover_type(LandCoverType::habitat);
						cross_par->setReport_details(save_details);
						model_parameters.push_back(cross_par);
					}
				}
				else if(!strcmp(argument,"SEARCH_TIME_1_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* search_par = new ObserverParameter(ObserverParameter::name_search_time,_INTEGER,1);
						search_par->setReport_details(save_details);
						model_parameters.push_back(search_par);
					}
				}
				else if(!strcmp(argument,"SEARCH_TIME_2_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* search_par = new ObserverParameter(ObserverParameter::name_search_time,_INTEGER,2);
						search_par->setReport_details(save_details);
						model_parameters.push_back(search_par);
					}
				}
				else if(!strcmp(argument,"LAND_INDICES_OBS"))
				{	
					if(val && isdigit(*val) && atoi(val) == 1)
					{
						ObserverParameter* land_par = new ObserverParameter(ObserverParameter::name_landscape,_INTEGER,1);
						model_parameters.push_back(land_par);
					}
				}
				else if(!strcmp(argument,"REPRO_STEP_1"))
				{	
					if(val && isdigit(*val))
					{
						string* values = new string("1;");
						values->append(year_div);
						PopulationParameter* rstep_par = new PopulationParameter(PopulationParameter::name_repro_step,
							"P1_RSTEP",_INTEGER,1,(char*)values->c_str());
						rstep_par->addValue(val);
						model_parameters.push_back(rstep_par);
						parameter_checklist["REPRO_STEP_1"] = true;
					}
				}
				else if(!strcmp(argument,"REPRO_STEP_2"))
				{	
					if(val && isdigit(*val))
					{
						string* values = new string("1;");
						values->append(year_div);
						PopulationParameter* rstep_par = new PopulationParameter(PopulationParameter::name_repro_step,
							"P2_RSTEP",_INTEGER,2,(char*)values->c_str());
						rstep_par->addValue(val);
						model_parameters.push_back(rstep_par);
						parameter_checklist["REPRO_STEP_2"] = true;
					}
				}
				else if(!strcmp(argument,"GRID_FILE"))
				{	
					Parameter* grid_file = new Parameter(Parameter::name_landscape_file, "GRD_FILE", Parameter::name_landscape,_STRING);
					while(val)
					{
						grid_file->addValue(val);
						val=strtok(NULL,delims);
					}
					model_parameters.push_back(grid_file);
					parameter_checklist["GRID_FILE"] = true;
				}
			} //val
		} // end if
	} // end of for

	is.close();

	PopulationParameter* population_1_overlap = new PopulationParameter(PopulationParameter::name_overlapping_generations, "UNDEFINED", _STRING,1);
	population_1_overlap->addValue("no");
	model_parameters.push_back(population_1_overlap);

	PopulationParameter* population_2_overlap = new PopulationParameter(PopulationParameter::name_overlapping_generations, "UNDEFINED", _STRING,2);
	population_2_overlap->addValue("no");
	model_parameters.push_back(population_2_overlap);

	PopulationParameter* population_1_sex_ratio = new PopulationParameter(PopulationParameter::name_sex_ratio, "UNDEFINED", _FLOAT,1);
	population_1_sex_ratio->addValue("0");
	model_parameters.push_back(population_1_sex_ratio);

	PopulationParameter* population_2_sex_ratio = new PopulationParameter(PopulationParameter::name_sex_ratio, "UNDEFINED", _FLOAT,2);
	population_2_sex_ratio->addValue("0");
	model_parameters.push_back(population_2_sex_ratio);

	PopulationParameter* population_1_border_refl = new PopulationParameter(PopulationParameter::name_border_reflection_angle, "UNDEFINED", _FLOAT,1);
	population_1_border_refl->addValue("3.14159");
	model_parameters.push_back(population_1_border_refl);

	PopulationParameter* population_2_border_refl = new PopulationParameter(PopulationParameter::name_border_reflection_angle, "UNDEFINED", _FLOAT,2);
	population_2_border_refl->addValue("3.14159");
	model_parameters.push_back(population_2_border_refl);

	// I think we don't need this parameter ....
	//PopulationParameter* population_1_age_classes = new PopulationParameter(PopulationParameter::name_age_classes,_INTEGER,1);
	//population_1_age_classes->addValue("1");
	//model_parameters.push_back(population_1_age_classes);

	//PopulationParameter* population_2_age_classes = new PopulationParameter(PopulationParameter::name_age_classes,_INTEGER,2);
	//population_2_age_classes->addValue("1");
	//model_parameters.push_back(population_2_age_classes);

	calculate_parameter_combinations();
	verifyParameters();
	validateParameterValues();
}

void ASCIIParameterReader::initObservers() throw (IbusepmException)
{
	registry->resetObservers();

	if(!registry->simulator)
		throw IbusepmException("ASCIIParameterReader::initObservers() simulator not initialized!");

	short obs_counter = 0;
	parameter_list::const_iterator parameter_iterator = model_parameters.begin();
	while(parameter_iterator != model_parameters.end())
	{
		Parameter* parameter = *parameter_iterator++;
		if(!strcmp(parameter->getType(),Parameter::name_observer))
		{
			obs_counter++;
			ObserverParameter* obs_param = (ObserverParameter*)parameter;
			if(!strcmp(obs_param->getName(),ObserverParameter::name_pop_size))
			{
				PopulationObserver* pop_obs = new PopulationObserver(registry, obs_param->getPopulation_id(), 
											  registry->simulator->getRuns(), registry->simulator->getObs_steps());
				pop_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_extinction))
			{
				ExtinctionObserver* ext_obs = new ExtinctionObserver(registry, obs_param->getPopulation_id(), 
											  registry->simulator->getRuns(), registry->simulator->getObs_steps());
				ext_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_landcover))
			{
				LandCoverObserver* landcov_obs = new LandCoverObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps(), obs_param->getCover_id());
				landcov_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_cell_immigration))
			{
				CellImmigrationObserver* cim_obs = new CellImmigrationObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps(), obs_param->getCover_id());
				cim_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_patch_immigration))
			{
				PatchImmigrationObserver* pim_obs = new PatchImmigrationObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps(), obs_param->getCover_id());
				pim_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_movement_distance))
			{
				DistanceObserver* dist_obs = new DistanceObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps());
				dist_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_boundary_crossing))
			{
				CrossingObserver* cross_obs = new CrossingObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps(), obs_param->getCover_id(), obs_param->getCross_Cover_type());
				cross_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_search_time))
			{
				SearchTimeObserver* search_obs = new SearchTimeObserver(registry, obs_param->getPopulation_id(), registry->simulator->getRuns(), 
										            registry->simulator->getObs_steps());
				search_obs->setReport_raw_obs(obs_param->getReport_details());
			}
			else if(!strcmp(obs_param->getName(),ObserverParameter::name_landscape))
			{
				LandscapeObserver* search_obs = new LandscapeObserver(registry, registry->simulator->getRuns(), registry->simulator->getObs_steps());
			}
		}
	}
	if(obs_counter == 0)
		throw IbusepmException("ASCIIParameterReader::initObservers() no observers selected!");
}

#endif 