/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PARAMETER_READER_H
#define __PARAMETER_READER_H

#include "tinyxml.h"
#include "ibusepmexception.h"
#include "parameter.h"
#include "population.h"
#include <vector>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

class ModelRegistry;

typedef vector<Parameter*> parameter_list;
typedef parameter_list::iterator parameter_iterator;
typedef map<const char*, short> reference_list;

/**
	The ParameterReader object is the central interface to parameter
	files. The ParameterReader provides methods for reading ascii 
	and xml parameter files. After reading a parameter file, 
	the ParameterReader object holds a list of all model parameters.
	It provides an algorithm for factorial and random combinations
	as well as Parameter verification and validation.
*/

class ParameterReader
{
protected:
	const char*			filename;	
	parameter_list		model_parameters;
	parameter_iterator	current_parameter;
	reference_list		parameter_checklist;
	unsigned int		parameter_combinations;
	unsigned int		current_combination;
	unsigned int		start_seed;
	bool				save_details;
	bool				save_asc_landscapes;
	bool				save_gif_landscapes;
	combination_types	combination_type;
	ModelRegistry*		registry;
	static char*		buffer;

public:
	ParameterReader(ModelRegistry* _registry, const char* _filename);
	~ParameterReader();

	const char* getFilename() const					{ return filename; };
	void setFilename(const char* _filename)			{ filename = _filename; };

	void setCombinationType(combination_types _type)
													{ combination_type = _type; };

	unsigned int getParameterCombinations() const	{ return parameter_combinations; };
	void setParameterCombinations(unsigned int _combinations);

	void setStartSeed(unsigned int _start_seed)		{ start_seed = _start_seed; }
	unsigned int getStartSeed();

	void setSaveDetails(bool _save_details)			{ save_details = _save_details; }
	bool getSaveDetails() const						{ return save_details; }

	bool nextParameterCombination();
	bool nextFactorialCombination();
	bool nextRandomCombination();

	virtual void readFile() {};
	
	void initModel();
	void report(ostream& strm=cout) const;
	void reportFactorialCombinations(ostream& strm=cout);
	void reportCurrentCombination(ostream& strm=cout) const;

	void reportCurrentValues(ostream& strm=cout) const;
	void reportParameterNames(ostream& strm=cout) const;

	void reportCurrentVariations(ostream& strm=cout) const;
	void reportVariationNames(ostream& strm=cout) const;

protected:
	void calculate_parameter_combinations();
	void resetParameters();
	virtual void initObservers() {};
	void verifyParameters() throw (IbusepmException);
	void validateParameterValues() throw (IbusepmException);

private:
	void initPopulation(Population* _population, PopulationParameter* _parameter);
	void initBehaviour(Population* _population, BehaviourParameter* _parameter);
	void initLandcoverType(Landscape* _landscape, LandCoverTypeParameter* _parameter);
	void createPopulations();
	void initPopulations();
	void initLandscape();
	void initSimulator();	
	void init();
	void initParameterCheckList();
};

class XMLParameterReader : public ParameterReader
{
private:
	TiXmlDocument* xml_parameters;

public:
	XMLParameterReader(ModelRegistry* _registry, const char* _filename);
	~XMLParameterReader();

	void readFile() throw (IbusepmException);
	void parseParameters() throw (IbusepmException);

private:
	void parseLandscapeParameters() throw (IbusepmException);
	void parseLandCoverTypeParameters() throw (IbusepmException);
	void parsePopulationParameters() throw (IbusepmException);
	void parseSimulationParameters() throw (IbusepmException);
	void initObservers() throw (IbusepmException);
	void parseBehaviourParameters(TiXmlElement* _behaviour, int population_id) throw (IbusepmException);
	int count_boundary_crossing_parameters(TiXmlElement* _behaviour) throw (IbusepmException);
};

class ASCIIParameterReader : public ParameterReader
{
public:
	ASCIIParameterReader(ModelRegistry* _registry, const char* _filename);
	~ASCIIParameterReader() {};

	int read_number_of_lines_from_parameterfile() const	throw (IbusepmException);
	void parseParameters() throw (IbusepmException);
private:
	void initObservers() throw (IbusepmException);
};

#endif