/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PATCH_CPP
#define __PATCH_CPP

#include "patch.h"
#include "patchisovar.h"
#include "logging.h"
#include "ibusutil.h"
#include "logging.h"
#include <stdio.h>

Patch::Patch()
{
	patch_id = NULL_LONG;
	cover_id = NULL_TYPE;	
	xref = NULL_SHORT;
	yref = NULL_SHORT;
	xmax = 0;
	ymax = 0;
	size = 0;
	perimeter = 0;
}

Patch::Patch(long _patch_id, cover_type _cover_id) 
{
	patch_id = _patch_id;
	cover_id = _cover_id;	
	xref = NULL_SHORT;
	yref = NULL_SHORT;
	xmax = 0;
	ymax = 0;
	size = 0;
	perimeter = 0;
}

Patch::Patch(unsigned short _xref, unsigned short _yref, long _patch_id, cover_type _cover_id) 
{
	patch_id = _patch_id;
	cover_id = _cover_id;	
	xref = _xref;
	yref = _yref;
	xmax = 0;
	ymax = 0;
	size = 0;
	perimeter = 0;
}

void Patch::reset()
{
	isovar_list::iterator isovar_iterator = isovars.begin();
	while(isovar_iterator != isovars.end())
	{
		PatchIsoVar* tmpIsoVar = *isovar_iterator;	
		isovar_iterator++;	
		if(tmpIsoVar)
			delete tmpIsoVar;	
	}
	isovars.clear();
	individuals.clear();
}

bool Patch::operator==(const Patch& _patch) const
{
	return patch_id==_patch.patch_id;
}

bool Patch::operator!=(const Patch& _patch) const 
{ 
	return patch_id!=_patch.patch_id;
}

void Patch::register_cell(int x, int y)
{
	size++;
	if(x<xref)
		xref=(unsigned short)x;
	if(x>xmax)
		xmax=(unsigned short)x;
	if(y>ymax)
		ymax=(unsigned short)y;
}

void Patch::add_iso_var(PatchIsoVar* _isovar)
{
	isovars.push_back(_isovar);
}

void Patch::report_iso_vars(ostream& strm) const
{
	if(isovars.empty())
		strm<<"No patch isolation variables in list"<<endl;
	else
	{
		strm<<"PATCH ISOLATION VARIABLES:"<<endl<<endl;
		isovar_list::const_iterator isovar_iterator = isovars.begin();

		int perline=5;
		int count=0;
		while(isovar_iterator != isovars.end())
		{
			if(count==perline)
            {
                    strm<<"\n";
                    perline+=5;
            }
			PatchIsoVar* tmpIsoVar = *isovar_iterator; 
			strm<<tmpIsoVar->getName()<<"("<<tmpIsoVar->getRefval()<<")=";
			strm<<tmpIsoVar->getVal()<<"  ";
			isovar_iterator++;
			count++;
		}
		strm<<"\n\n";
	}
}

void Patch::addIndividual(unsigned short _population_id) throw (IbusepmException)
{
	Logging::log_debug("Patch::addIndividual(",Utility::int_to_char(_population_id),")\n",NULL);
	individuals[_population_id]++;
	Logging::log_debug("Patch::addIndividual() #individuals: ",Utility::int_to_char(individuals.size()),"\n",NULL);
}

void Patch::removeIndividual(unsigned short _population_id) throw (IbusepmException)
{
	if(individuals[_population_id] < 1)
		throw IbusepmException("Patch::removeIndividual() value < 1!");
	individuals[_population_id]--;
}

unsigned int Patch::getIndividuals(unsigned short _population_id)
{
	return individuals[_population_id];
}

const char* Patch::report() const
{
	char* buffer = new char[100];
	sprintf(buffer,"Patch_id %d Size %d Perim %d Cover_id %d Xref,Yref %d,%d Xmax,Ymax %d,%d Individuals %d",
			patch_id,size, perimeter, (int)cover_id,xref,yref,xmax,ymax,individuals);
	return (const char*)buffer;
}

void Patch::report(ostream& strm) const
{
	strm<<"Patch_id "<<patch_id;
	strm<<" Size "<<size;
	strm<<" Perim "<<perimeter;
	strm<<" Cover_id "<<(int)cover_id;
	strm<<" Xref,Yref "<<xref<<","<<yref;
	strm<<" Xmax,Ymax "<<xmax<<","<<ymax;
	pop_individuals::const_iterator iterator = individuals.begin();
	while(iterator != individuals.end())
	{
		strm<<" Individuals von Population "<<(*iterator).first<<": ";
		strm<<(*iterator++).second<<"\n";
	}
}

#endif