/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __POPULATION_CPP
#define __POPULATION_CPP

#include "population.h"
#include "individual.h"
#include "landscape.h"
#include "parareader.h"
#include "statisti.h"
#include "patch.h"
#include "logging.h"
#include "ibusutil.h"
#include <algorithm>

Population::Population(unsigned short _id, ModelRegistry* _registry)
{
	id = _id;
	registry = _registry;

	initial_size = NULL_INT;
	maximum_size = 0;
	init_distribution = INVALID;
	init_cover_id = NULL_TYPE;
	sequence_type = DEFAULT;
	individual_id = 0;
	steps = 0;
	repro_step = 1;
	sex_ratio = 0.5;
	sample_size = NULL_INT;
	refl_angle = (float)PIE; 
	overlap = false;
	start_angle = new uniform("rw",-PIE,PIE);
	start_angle->set_seed(registry->parameters->getStartSeed());
}

void Population::init_activities()
{
	activities.clear();
	activities.push_back(REPRODUCTION);
	activities.push_back(MORTALITY);
	//activities.push_back(MATURE);
	activities.push_back(MOVEMENT);
}

void Population::add_activity(activity_types _activity)
{
	sequence_type = CUSTOM;

	if(activities.size() == 3)
		activities.clear();

	if(find(activities.begin(), activities.end(), _activity) == activities.end())
		activities.push_back(_activity);
	else
		Logging::log_warning("Population::add_activity() tried to add '",Utility::name_for_activity(_activity),"' twice.\n",NULL);
}

void Population::add_behaviour(Behaviour& _behaviour)
{	
	behaviours.push_back(_behaviour);
}

bool Population::find_behaviour(Behaviour _behaviour)
{
	behaviour_list::iterator behaviour_iterator = find(behaviours.begin(), behaviours.end(), _behaviour);
	return behaviour_iterator != behaviours.end() ? true : false;
}

void Population::add_individual(Individual* _individual)
{
	individuals.push_back(_individual);
}

void Population::add_cellCoordinate(int _x, int _y)
{
	cell_coordinates[_x] = _y;
}

int Population::getSize_on_covertype(cover_type _cover_id) const
{
	Logging::log_debug("Population::getSize_on_covertype() ...",NULL);

	int counter = 0;
	individual_list::const_iterator individual_iterator = individuals.begin();
	while(individual_iterator != individuals.end())
	{
		if((*individual_iterator++)->getCurrentCoverType() == _cover_id)
			counter++;
    }
	Logging::log_debug("Population::getSize_on_covertype() done.",NULL);
	return counter;
}

Population::~Population()
{
	if(start_angle)
		delete start_angle;
}

void Population::reset()
{
	Logging::log_debug("Population::reset() ...",NULL);

	initial_size = NULL_INT;
	maximum_size = 0;
	init_distribution = INVALID;
	init_cover_id = NULL_TYPE;
	individual_id = 0;
	steps = 0;
	sex_ratio = 0.5;
	sample_size = NULL_INT;
	refl_angle = (float)PIE; 
	overlap = false;

	individual_list::iterator individual_iterator = individuals.begin();
	while(individual_iterator != individuals.end())
	{
		Individual* tmpIndividual = *individual_iterator++;
		if(tmpIndividual)
			delete tmpIndividual;
    }
	individuals.clear();
	activities.clear();
	cell_coordinates.clear();

	behaviour_list::iterator behaviour_iterator = behaviours.begin();
	while(behaviour_iterator != behaviours.end())
	{
		Behaviour* tmpBehaviour = behaviour_iterator++;
		if(tmpBehaviour)
			tmpBehaviour->reset();
	}
	behaviours.clear();

	Logging::log_debug("Population::reset() DONE.",NULL);
}

void Population::activity(unsigned short _step) throw (IbusepmException)
{
	Logging::log_debug("Population::activity() ...",NULL);

	unsigned int current_size = individuals.size();
	short activity_number = (short)activities.size();

	if(current_size > maximum_size)
		maximum_size = current_size;

	if(current_size > PMAX)
	{
		string* message = new string("Current size of population ");
		message->append(Utility::int_to_char(id));
		message->append(" is ");
		message->append(Utility::int_to_char(current_size));
		message->append(". Slow simulation!");
		Logging::log_warning(message->c_str(),NULL);
		delete message;
	}

	unsigned int individuals_index = 0;
	while(individuals_index < current_size)
	{
		Individual* tmpIndividual = individuals[individuals_index];

		short count = 0;
		short index = -1;
		short array[4];
		for(short i=0; i<=4; i++)
			array[i] = -1;

		while(count < activity_number)
		{
			if(sequence_type == RANDOM)
			{
				index = (short)Utility::randint(activity_number);
				if(index >= activity_number)
					index-=1;
			}
			else
				index++;
			if(array[index] == -1)
			{
				array[index] = index;
				switch(activities[index])
				{
					case REPRODUCTION:	if(_step == repro_step) 
											tmpIndividual->reproduce();
										break;
					case MOVEMENT:		tmpIndividual->move();
										break;
					case MORTALITY:		tmpIndividual->die();
										break;
					case MATURE:		tmpIndividual->mature();
										break;
					default:			Logging::log_error("Population::activity()",
											               " no activity in activities!", NULL);
				}
				count++;
			}
		}
		individuals_index++;	
	}
	Logging::log_debug("Population::activity() DONE",NULL);
}

void Population::remove_dead_individuals()
{
	Logging::log_debug("Population::remove_dead_individuals() ...",NULL);

	individual_list::iterator individual_iterator = individuals.begin();
	while(individual_iterator != individuals.end())
	{
		Individual* tmpIndividual = *individual_iterator;
		if(tmpIndividual->isDead())
		{
			registry->updateDistanceObserver(id, tmpIndividual->getId(), false, tmpIndividual->getXpos(), tmpIndividual->getYpos());
			delete tmpIndividual;
			individual_iterator = individuals.erase(individual_iterator);
		}
		else
			individual_iterator++;			        
    }

	Logging::log_debug("Population::remove_dead_individuals() done.",NULL);
}

void Population::reinitialize()
{
	Logging::log_debug("Population::reinitialize() ...",NULL);

	maximum_size = 0;
	individual_id = 0;
	steps = 0;

	individual_list::iterator individual_iterator = individuals.begin();
	while(individual_iterator != individuals.end())
	{
		Individual* tmpIndividual = *individual_iterator++;
		if(tmpIndividual)
		{
			tmpIndividual->checkout_from_old_patch();
			delete tmpIndividual;
		}
    }
	individuals.clear();	
	initialize_individuals();

	Logging::log_debug("Population::reinitialize() DONE",NULL);
}

//creates a list of individuals distributed on the landscape
//according to init_distribution
void Population::initialize_individuals() throw (IbusepmException)
{
	Logging::log_debug("Population::initialize_individuals() ...",NULL);
	verify_distribution_conditions();
	switch (init_distribution)
	{

		case LANDSCAPE:		distribute_individuals_on_landscape();
							break;
		case COVER_TYPE:	distribute_individuals_on_cover_type();
							break;
		case PATCH:			distribute_individuals_on_patches();
							break;
		case CELL:			distribute_individuals_on_cells();
							break;
		default:			cout<<"do nothing\n";
	}
	Logging::log_debug("Population::initialize_individuals() DONE",NULL);
}

void Population::distribute_individuals_on_cells()
{
	Logging::log_debug("Population::distribute_individuals_on_cells() ...",NULL);

	int individuals_per_cell = initial_size / cell_coordinates.size();
	int_pairs::iterator cell_coordinate_iterator = cell_coordinates.begin();

	while(cell_coordinate_iterator != cell_coordinates.end())
	{
		float x = (float)(*cell_coordinate_iterator).first;
		float y = (float)(*cell_coordinate_iterator++).second;

		uniform xrange("xuni",x,x+1);
		uniform yrange("yuni",y,y+1);

		xrange.set_seed(registry->parameters->getStartSeed()/2);
		yrange.set_seed(registry->parameters->getStartSeed()*2);
	
		Individual* tmpIndividual;

		int individual_counter=0;	
		float xpos,ypos;
		while(individual_counter < individuals_per_cell)
		{
			xpos=xrange.sample();
			ypos=yrange.sample();

			tmpIndividual=new Individual(individual_id++, this, determine_Gender(), 1, xpos, ypos);
			tmpIndividual->setRef_angle(start_angle->sample());
			tmpIndividual->setMark(determine_Sample());
			tmpIndividual->register_with_new_patch(xpos,ypos);
			individuals.push_back(tmpIndividual);

			individual_counter++;
		}
	}
	Logging::log_debug("Population::distribute_individuals_on_cells() DONE",NULL);	
}

void Population::distribute_individuals_on_patches()
{
	Logging::log_debug("Population::distribute_individuals_on_patches() ...",NULL);

	int individuals_per_patch = initial_size / cell_coordinates.size();
	int_pairs::iterator cell_coordinate_iterator = cell_coordinates.begin();

	while(cell_coordinate_iterator != cell_coordinates.end())
	{
		float x = (float)(*cell_coordinate_iterator).first;
		float y = (float)(*cell_coordinate_iterator++).second;

		Patch* patch = registry->landscape->getPatch(x, y);

		Logging::log_debug("Population::distribute_individuals_on_patch(",
						 Utility::int_to_char(patch->getPatch_id()),") ...",NULL);

		unsigned short xmin = patch->getXref();
		unsigned short xmax = patch->getXmax();
		unsigned short ymin = patch->getYref();
		unsigned short ymax = patch->getYmax();

		uniform xrange("xuni",(float)xmin,(float)xmax+1);
		uniform yrange("yuni",(float)ymin,(float)ymax+1);

		xrange.set_seed(registry->parameters->getStartSeed()/2);
		yrange.set_seed(registry->parameters->getStartSeed()*2);
	
		Patch* tmpPatch;
		Individual* tmpIndividual;
		int individual_counter=0;	
		float xpos,ypos;
		while(individual_counter < individuals_per_patch)
		{
			xpos=xrange.sample();
			ypos=yrange.sample();
			tmpPatch=registry->landscape->getPatch(xpos, ypos);
			if(*tmpPatch == *patch)
			{
				tmpIndividual=new Individual(individual_id++, this, determine_Gender(), 1, xpos, ypos);
				tmpIndividual->setRef_angle(start_angle->sample());
				tmpIndividual->setMark(determine_Sample());
				individuals.push_back(tmpIndividual);

				tmpIndividual->register_with_new_patch(xpos,ypos);
				individual_counter++;
			}
		}
	}
	Logging::log_debug("Population::distribute_individuals_on_patch() DONE",NULL);
}

void Population::distribute_individuals_on_cover_type()
{
	Logging::log_debug("Population::distribute_individuals_on_cover_type(",
						 Utility::int_to_char(init_cover_id),") ...",NULL);
    uniform xrange("xuni",0.0,(float)registry->landscape->getXdim()-0.01);
    uniform yrange("yuni",0.0,(float)registry->landscape->getYdim()-0.01);

	xrange.set_seed(registry->parameters->getStartSeed()/2);
	yrange.set_seed(registry->parameters->getStartSeed()*2);
	
	Cell* tmpCell;
	Individual* tmpIndividual;
	int individual_counter=0;	
	float xpos,ypos;
	long cover = registry->landscape->getLandCoverAmount(init_cover_id);
	
	if(cover == 0)
	{
		Logging::log_warning("Population::distribute_individuals_on_cover_type(",
							  Utility::int_to_char(init_cover_id),") failed. No cells of this cover type in landscape!\n",NULL);
		return;
	}
	while(individual_counter < initial_size)
	{
		xpos=xrange.sample();
		ypos=yrange.sample();
		tmpCell=registry->landscape->getCell(xpos, ypos);

		if(tmpCell->getCover_id() == init_cover_id)
		{
			tmpIndividual=new Individual(individual_id++, this, determine_Gender(), 1, xpos, ypos);
			tmpIndividual->setRef_angle(start_angle->sample());
			tmpIndividual->setMark(determine_Sample());
			individuals.push_back(tmpIndividual);

			tmpIndividual->register_with_new_patch(xpos,ypos);
			individual_counter++;
		}
	}
	Logging::log_debug("Population::distribute_individuals_on_cover_type(",
						Utility::int_to_char(init_cover_id),") DONE",NULL);
}

void Population::distribute_individuals_on_landscape()
{	
	Logging::log_debug("Population::distribute_individuals_on_landscape() ...",NULL);
    uniform xrange("xuni",0.0,(float)registry->landscape->getXdim()-0.01);
    uniform yrange("yuni",0.0,(float)registry->landscape->getYdim()-0.01);

	xrange.set_seed(registry->parameters->getStartSeed()/2);
	yrange.set_seed(registry->parameters->getStartSeed()*2);
	
	Individual* tmpIndividual;
	int individual_counter=0;	
	float xpos,ypos;
	while(individual_counter < initial_size)
	{
		xpos=xrange.sample();
		ypos=yrange.sample();
		tmpIndividual=new Individual(individual_id++, this, determine_Gender(), 1, xpos, ypos);
		tmpIndividual->setRef_angle(start_angle->sample());
		tmpIndividual->setMark(determine_Sample());
		individuals.push_back(tmpIndividual);

		tmpIndividual->register_with_new_patch(xpos,ypos);
		individual_counter++;
	}
	Logging::log_debug("Population::distribute_individuals_on_landscape() DONE",NULL);
}

void Population::verify_distribution_conditions() throw (IbusepmException)
{
	if(sample_size == NULL_INT || sample_size <= 0)
		sample_size = initial_size;

	if(init_distribution == INVALID)
		throw IbusepmException("population::initialize_individuals() init_distribution",
							   " not defined!\n");

	if(!registry->landscape || registry->landscape->getStatus() < PATCH_TOPOLOGY)
		throw IbusepmException("population::initialize_individuals() landscape NULL or",
							   " not initialized\n");

	if(init_distribution == COVER_TYPE && init_cover_id == NULL_TYPE)
		throw IbusepmException("population::initialize_individuals() init_distribution",
							   " is COVER_TYPE but init_cover_id not initialized\n");

	if((init_distribution == PATCH || init_distribution == CELL) && cell_coordinates.empty())
		throw IbusepmException("population::initialize_individuals() init_distribution ",
							   " is PATCH or CELL but x/y coordinates are not initialized\n");
}

void Population::setSample_size(int _sample_size) throw (IbusepmException)
{ 
	if(_sample_size <= initial_size)
		sample_size = _sample_size;
	else
		throw IbusepmException("Population::setSample_size() sample_size ",
							   " is larger than initial_size!\n");
}

gender_types Population::determine_Gender() const
{		
	if(Utility::randfloat() < sex_ratio)
		return MALE;
	return FEMALE;
}

bool Population::determine_Sample() const
{
	if(Utility::randint(initial_size) <= sample_size)
		return true;
	return false;
}

float Population::calculate_gender_ratio() const
{
	int females=0;
	int males=0;

	individual_list::const_iterator individual_iterator = individuals.begin();
	while(individual_iterator != individuals.end())
	{
		Individual* tmpIndividual = *individual_iterator;
		if(tmpIndividual->getGender() == FEMALE)
			females++;
		else
			males++;
		individual_iterator++;			        
    }
	return (float)females/(float)males;
}

void Population::reportBehaviours(ostream& strm)
{
	behaviour_list::iterator behaviour_iterator = behaviours.begin();
	while(behaviour_iterator != behaviours.end())
	{
		Behaviour* tempBehaviour = behaviour_iterator++;
		tempBehaviour->report(strm);
	}
}

void Population::reportActivities(ostream& strm) const
{
	strm<<"Activities: ";
	activity_list::const_iterator activity_iterator = activities.begin();
	while(activity_iterator != activities.end())
	{
		switch(*activity_iterator++)
		{
			case REPRODUCTION:	strm<<"reproduction,";break;
			case MOVEMENT:		strm<<"movement,";break;
			case MORTALITY:		strm<<"mortality,";break;
			case MATURE:		strm<<"aging,";break;
			default:			strm<<"";
		}
	}
	strm<<"\n";
}

void Population::report(ostream& strm)
{
	Logging::log_debug("Population::report ...",NULL);
	strm<<"Population_id: "<<id<<" initial_size: "<<initial_size;
	strm<<" sex_ratio: "<<sex_ratio;
	strm<<" Sequence: ";
	switch(sequence_type)
	{
		case DEFAULT:	strm<<"default";break;
		case RANDOM:	strm<<"random";break;
		case CUSTOM:	strm<<"custom";break;
		default:		strm<<"not defined";
	}
	strm<<"\n";
	reportActivities(strm);
	reportBehaviours(strm);

	Logging::log_debug("Population::report DONE",NULL);
}

#endif
