/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __POPULATION_H
#define __POPULATION_H

#include <iostream>
#include <list>
#include <vector>
#include "defs.h"
#include "behaviour.h"
#include "ibusepmexception.h"

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

class Landscape;
class Individual;
class uniform;
class ModelRegistry;

typedef vector<Individual*> individual_list;
typedef vector<Behaviour> behaviour_list;
typedef vector<activity_types> activity_list;
typedef map<int, int> int_pairs;

/**
	A Population object represents a list of individuals to the model. In addition,
	population objects provide all information common to all individual such as,
	sex_ratio, activity sequences.
*/

class Population
{
	friend class		Individual;

	individual_list		individuals;
	behaviour_list		behaviours;
	
	distribution_types	init_distribution;	//governs initial distribution of individuals
	int_pairs			cell_coordinates;	//x,y-coordinates for locating cells or patches,
											//when distribution_type = Cell or Patch
	cover_type			init_cover_id;		//cover_id for distribution, when distribution
											//type = COVER_TYPE
	unsigned short		id;	
	unsigned long		individual_id;		//counter for individual ids
	float				sex_ratio;			//ratio for females and males in population
	int					initial_size;    
	unsigned int		maximum_size;
	unsigned int		steps;				//counter for movement steps of all individuals
	unsigned short		repro_step;			//step for reproduction
	int					sample_size;		//numbers of individuals marked 
	float				refl_angle;			//common reflection angle at boundaries
	uniform*			start_angle;		//distributions for starting reference angles for individuals
	bool				overlap;			//if true, overlapping generations
	sequence_types		sequence_type;		//sequence of individual activities
	activity_list		activities;			//stores the sequences of individual activities
	ModelRegistry*		registry;			//central registry for all major model components

	
public:
	Population(unsigned short id, ModelRegistry* _registry);
	~Population();

    void reset();

	void reinitialize();

	void add_behaviour(Behaviour& _behaviour);
	bool find_behaviour(Behaviour _behaviour);
	behaviour_list* get_behaviours() 		{ return &behaviours; };

	void add_individual(Individual* _individual);
	individual_list* get_individuals()		{ return &individuals; };

	unsigned short getId() const			{ return id; };
	int getSize() const						{ return individuals.size(); };

	int getSize_on_covertype(cover_type _cover_id) const;

	void setInitial_size(int _initial_size)	{ initial_size = _initial_size; };
	int getInitial_size() const				{ return initial_size; };

	int getMaximum_size() const				{ return maximum_size; };

	void setSample_size(int _sample_size)	throw (IbusepmException); 
	int getSample_size() const				{ return sample_size; };

	void setRefl_angle(float _refl_angle)	{ refl_angle = _refl_angle; };
	float getRefl_angle() const				{ return refl_angle; };

	void setSex_ratio(float _sex_ratio)		{ sex_ratio = _sex_ratio; };
	float getSex_ratio() const				{ return sex_ratio; };

	void set_Overlap(bool _overlap)			{ overlap = _overlap; };
	bool is_Overlap() const					{ return overlap; }
	
	void addStep()							{ steps++; };
	unsigned int getSteps()	const			{ return steps; };

	void setReproStep(unsigned short _rstp)	{ repro_step = _rstp; };

	void set_Sequence_type(sequence_types _st)	{ sequence_type = _st; }
	sequence_types get_Sequence_type() const	{ return sequence_type; }

	void set_Init_distribution(distribution_types _init_distribution)
											{ init_distribution = _init_distribution; };
	distribution_types get_Init_distribution() const
											{ return init_distribution; };

	void set_Init_cover_id(cover_type _init_cover_id)
											{ init_cover_id = _init_cover_id; };
	cover_type get_Init_cover_id() const	{ return init_cover_id; };

	void add_cellCoordinate(int _x, int _y);
	int_pairs* get_cellCoordinates()		{ return &cell_coordinates; };
   
	void initialize_individuals() throw (IbusepmException);
	void add_activity(activity_types _activity);

	void activity(unsigned short _step) throw (IbusepmException);
	void remove_dead_individuals();
	void init_activities();
	void report(ostream& strm=cout);
	void reportBehaviours(ostream& strm=cout);
	void reportActivities(ostream& strm) const;

private:
	void verify_distribution_conditions() throw (IbusepmException);
	void distribute_individuals_on_cells();
	void distribute_individuals_on_patches();
	void distribute_individuals_on_cover_type();
	void distribute_individuals_on_landscape();

	gender_types determine_Gender() const;
	bool determine_Sample() const;

	float calculate_gender_ratio() const;
};

#endif
