/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __SIMULATOR_CPP
#define __SIMULATOR_CPP

#include "modelregistry.h"
#include "simulator.h"
#include "parareader.h"
#include "landscape.h"
#include "individual.h"
#include "population.h"
#include "ibusutil.h"
#include "logging.h"
#include <algorithm>

Simulator::Simulator(ModelRegistry* _registry)
{
	registry = _registry;
	years =	NULL_SHORT;
	steps =	NULL_SHORT;
	runs =	NULL_SHORT;
	obs_gap = NULL_SHORT;
	combination = 0;
	dot_offset = 40;
	registry->register_simulator(this);
}

void Simulator::reset()
{
	years = NULL_SHORT;
	steps = NULL_SHORT;
	runs =	NULL_SHORT;
	obs_gap = NULL_SHORT;
}	

void Simulator::simulate()
{
	combination++;
	int dot_gap = runs / (runs < dot_offset ? runs : dot_offset);
	showProgress();

	for(unsigned short run = 1; run <= runs; run++)
	{
		reinitialize_populations();
		int nextobstep=obs_gap;
        unsigned short obstep=0;
		observe(run-1, obstep);	

		for(unsigned short year = 1; year <= years; year++)
		{
			for(unsigned short step = 1; step <= steps; step++)
			{
				write_dot(run, dot_gap);
				activate_populations(step);
				
         		if((year-1)*steps+step == nextobstep)
         		{
            		nextobstep += obs_gap;
         			observe(run-1,++obstep);
         		}
			}
		}
	}
	//cout<<"\n";
	Logging::log_info(false,"\n",NULL);
}

void Simulator::reinitialize_populations()
{
	population_map& populations = registry->getPopulations();
	population_map::iterator population_iterator = populations.begin();

	while(population_iterator != populations.end())
	{
		Population* tempPopulation = (*population_iterator++).second;
		tempPopulation->reinitialize();
	}
}

void Simulator::activate_populations(unsigned short _step)
{
	population_map& populations = registry->getPopulations();
	population_map::iterator population_iterator = populations.begin();

	while(population_iterator != populations.end())
	{
		Population* tempPopulation = (*population_iterator++).second;
		tempPopulation->activity(_step);
	}
	
	//do overabundance kill here for both populations
	Cell* tmpCell;
	LandCoverType* tmpLandCoverType;
	Individual* tmpIndividual;
	cover_type tmpCellCover_id;
	float tmpCellCapacity;
	float tmpCellReduction;
	float tmpCellResourceLimit;
	//int counter = 0;
	int xdim = (float)registry->landscape->getXdim();
	int ydim = (float)registry->landscape->getYdim();

	individual_list* allPops = new individual_list();
	individual_list* pop1 = registry->getPopulation(1)->get_individuals();
	individual_list* pop2 = registry->getPopulation(2)->get_individuals();
	individual_list::iterator pop1_iterator = pop1->begin();
	individual_list::iterator pop2_iterator = pop2->begin();
	while(pop1_iterator != pop1->end())
		allPops->push_back(*pop1_iterator++);
	while(pop2_iterator != pop2->end())
		allPops->push_back(*pop2_iterator++);

	random_shuffle(allPops->begin(), allPops->end());

	for(float y=0; y<ydim; y++)
	{
		for(float x=0; x<xdim; x++)
		{
			tmpCell = registry->landscape->getCell(x,y);
			tmpCellCover_id = tmpCell->getCover_id();
			tmpLandCoverType = registry->landscape->getLandCoverType(tmpCellCover_id);
			tmpCellCapacity = tmpLandCoverType->getCell_capacity();
			tmpCellReduction = tmpLandCoverType->getCell_reduction();

			if(tmpCellCapacity != NULL_FLOAT && tmpCell->getUsed_resources() > tmpCellCapacity)
			{
				tmpCellResourceLimit = tmpCellCapacity * tmpCellReduction;
				individual_list::iterator individual_iterator = allPops->begin();

				while(tmpCell->getUsed_resources() > tmpCellResourceLimit && 
					  individual_iterator != allPops->end())

				{
					tmpIndividual = *individual_iterator++;
					if(!tmpIndividual->isDead() && (int)tmpIndividual->getXpos() == x && (int)tmpIndividual->getYpos() == y)
					{
						tmpIndividual->checkout_from_old_patch();
						tmpIndividual->setDead(true);
					}
				}
			}
		}
	}
	allPops->clear();
	delete allPops;

	population_iterator = populations.begin();
	while(population_iterator != populations.end())
	{
		Population* tempPopulation = (*population_iterator++).second;
		tempPopulation->remove_dead_individuals();
	}
}


void Simulator::observe(unsigned short _run, unsigned short _step)
{
	observer_list* observers = registry->getObservers();
	observer_list::iterator observer_iterator = observers->begin();
	while(observer_iterator != observers->end())
		(*observer_iterator++)->observe(_run, _step);
}

void Simulator::verify_status() throw (IbusepmException)
{
	if(years == NULL_SHORT || years <= 0)
		throw IbusepmException("Simulator::verify_status() 'years' not initialized!");
	if(steps == NULL_SHORT || steps <= 0)
		throw IbusepmException("Simulator::verify_status() 'steps' not initialized!");
	if(runs == NULL_SHORT || runs <= 0)
		throw IbusepmException("Simulator::verify_status() 'runs' not initialized!");
	if(obs_gap == NULL_SHORT || obs_gap <= 0)
		throw IbusepmException("Simulator::verify_status() 'obs_gap' not initialized!");
}

unsigned short Simulator::getObs_steps()
{
	verify_status();
	return ((years * steps) / obs_gap) + 1;
}

void Simulator::showProgress()
{
	string message("(");
	message.append(Utility::int_to_char(combination));
	message.append("/");
	message.append(Utility::int_to_char(registry->parameters->getParameterCombinations()));
	message.append(") ");
	Logging::log_info(true, message.c_str(),NULL);
	//cout<<message.c_str();
}

void Simulator::write_dot(int _run, int& _gap)
{
	if(_run == _gap)
	{
		_gap += runs / (runs < dot_offset ? runs : dot_offset);
		Logging::log_info(false,".",NULL);
		//cout<<".";
	}
}

#endif